@@ -2,7 +2,8 @@
 __license__   = 'GPL v3'
 __copyright__ = '2008, Kovid Goyal <kovid@kovidgoyal.net>'
 __docformat__ = 'restructuredtext en'
-import sys, os, re, logging, time, mimetypes, \
+
+import uuid, sys, os, re, logging, time, mimetypes, \
        __builtin__, warnings, multiprocessing
 from urllib import getproxies
 __builtin__.__dict__['dynamic_property'] = lambda(func): func(None)
@@ -13,14 +14,17 @@ from functools import partial
 warnings.simplefilter('ignore', DeprecationWarning)
 
 
-from calibre.startup import plugins, winutil, winutilerror
 from calibre.constants import iswindows, isosx, islinux, isfreebsd, isfrozen, \
                               terminal_controller, preferred_encoding, \
                               __appname__, __version__, __author__, \
                               win32event, win32api, winerror, fcntl, \
-                              filesystem_encoding
+                              filesystem_encoding, plugins, config_dir
+from calibre.startup import winutil, winutilerror
+
 import mechanize
 
+uuid.uuid4() # Imported before PyQt4 to workaround PyQt4 util-linux conflict on gentoo
+
 if False:
     winutil, winutilerror, __appname__, islinux, __version__
     fcntl, win32event, isfrozen, __author__, terminal_controller
@@ -361,6 +365,8 @@ def strftime(fmt, t=None):
     before 1900 '''
     if t is None:
         t = time.localtime()
+    if hasattr(t, 'timetuple'):
+        t = t.timetuple()
     early_year = t[0] < 1900
     if early_year:
         replacement = 1900 if t[0]%4 == 0 else 1901
@@ -409,15 +415,13 @@ def entity_to_unicode(match, exceptions=[], encoding='cp1252',
         return check("'")
     if ent == 'hellips':
         ent = 'hellip'
-    if ent.lower().startswith(u'#x'):
-        num = int(ent[2:], 16)
-        if encoding is None or num > 255:
-            return check(my_unichr(num))
-        return check(chr(num).decode(encoding))
-    if ent.startswith(u'#'):
+    if ent.startswith('#'):
         try:
-            num = int(ent[1:])
-        except ValueError:
+            if ent[1] in ('x', 'X'):
+                num = int(ent[2:], 16)
+            else:
+                num = int(ent[1:])
+        except:
             return '&'+ent+';'
         if encoding is None or num > 255:
             return check(my_unichr(num))
@@ -438,6 +442,9 @@ xml_entity_to_unicode = partial(entity_to_unicode, result_exceptions = {
     '>' : '&gt;',
     '&' : '&amp;'})
 
+def replace_entities(raw):
+    return _ent_pat.sub(entity_to_unicode, raw)
+
 def prepare_string_for_xml(raw, attribute=False):
     raw = _ent_pat.sub(entity_to_unicode, raw)
     raw = raw.replace('&', '&amp;').replace('<', '&lt;').replace('>', '&gt;')
@@ -481,7 +488,6 @@ def ipython(user_ns=None):
     sys.argv = ['ipython']
     if user_ns is None:
         user_ns = locals()
-    from calibre.utils.config import config_dir
     ipydir = os.path.join(config_dir, ('_' if iswindows else '.')+'ipython')
     os.environ['IPYTHONDIR'] = ipydir
     if not os.path.exists(ipydir):
