@@ -8,7 +8,7 @@ from calibre.constants import numeric_version
 from calibre.ptempfile import PersistentTemporaryFile
 
 
-class Plugin(object):
+class Plugin(object): # {{{
     '''
     A calibre plugin. Useful members include:
 
@@ -147,9 +147,9 @@ class Plugin(object):
         if hasattr(it, '__exit__'):
             it.__exit__(*args)
 
+# }}}
 
-
-class FileTypePlugin(Plugin):
+class FileTypePlugin(Plugin): # {{{
     '''
     A plugin that is associated with a particular set of file types.
     '''
@@ -191,7 +191,9 @@ class FileTypePlugin(Plugin):
         # Default implementation does nothing
         return path_to_ebook
 
-class MetadataReaderPlugin(Plugin):
+# }}}
+
+class MetadataReaderPlugin(Plugin): # {{{
     '''
     A plugin that implements reading metadata from a set of file types.
     '''
@@ -219,8 +221,9 @@ class MetadataReaderPlugin(Plugin):
         :return: A :class:`calibre.ebooks.metadata.MetaInformation` object
         '''
         return None
+# }}}
 
-class MetadataWriterPlugin(Plugin):
+class MetadataWriterPlugin(Plugin): # {{{
     '''
     A plugin that implements reading metadata from a set of file types.
     '''
@@ -249,7 +252,9 @@ class MetadataWriterPlugin(Plugin):
         '''
         pass
 
-class CatalogPlugin(Plugin):
+# }}}
+
+class CatalogPlugin(Plugin): # {{{
     '''
     A plugin that implements a catalog generator.
     '''
@@ -262,31 +267,21 @@ class CatalogPlugin(Plugin):
 
     type = _('Catalog generator')
 
-    #: CLI parser options specific to this plugin, declared as namedtuple Option
+    #: CLI parser options specific to this plugin, declared as namedtuple Option::
     #:
-    #: from collections import namedtuple
-    #: Option = namedtuple('Option', 'option, default, dest, help')
-    #: cli_options = [Option('--catalog-title',
+    #:  from collections import namedtuple
+    #:  Option = namedtuple('Option', 'option, default, dest, help')
+    #:  cli_options = [Option('--catalog-title',
     #:                       default = 'My Catalog',
     #:                       dest = 'catalog_title',
     #:                       help = (_('Title of generated catalog. \nDefault:') + " '" +
     #:                       '%default' + "'"))]
-    #: cli_options parsed in library.cli:catalog_option_parser()
-
+    #:  cli_options parsed in library.cli:catalog_option_parser()
     cli_options = []
 
 
     def search_sort_db(self, db, opts):
 
-        '''
-        # Don't add Catalogs to the generated Catalogs
-        cat = _('Catalog')
-        if opts.search_text:
-            opts.search_text += ' not tag:'+cat
-        else:
-            opts.search_text = 'not tag:'+cat
-        '''
-
         db.search(opts.search_text)
 
         if opts.sort_by:
@@ -299,7 +294,7 @@ class CatalogPlugin(Plugin):
         # Return a list of requested fields, with opts.sort_by first
         all_fields = set(
                           ['author_sort','authors','comments','cover','formats',
-                           'id','isbn','pubdate','publisher','rating',
+                           'id','isbn','ondevice','pubdate','publisher','rating',
                            'series_index','series','size','tags','timestamp',
                            'title','uuid'])
 
@@ -311,6 +306,9 @@ class CatalogPlugin(Plugin):
         else:
             fields = list(all_fields)
 
+        if not opts.connected_device['is_device_connected'] and 'ondevice' in fields:
+            fields.pop(int(fields.index('ondevice')))
+
         fields.sort()
         if opts.sort_by and opts.sort_by in fields:
             fields.insert(0,fields.pop(int(fields.index(opts.sort_by))))
@@ -349,8 +347,7 @@ class CatalogPlugin(Plugin):
         It should generate the catalog in the format specified
         in file_types, returning the absolute path to the
         generated catalog file. If an error is encountered
-        it should raise an Exception and return None. The default
-        implementation simply returns None.
+        it should raise an Exception.
 
         The generated catalog file should be created with the
         :meth:`temporary_file` method.
@@ -358,10 +355,79 @@ class CatalogPlugin(Plugin):
         :param path_to_output: Absolute path to the generated catalog file.
         :param opts: A dictionary of keyword arguments
         :param db: A LibraryDatabase2 object
-
-        :return: None
-
         '''
         # Default implementation does nothing
         raise NotImplementedError('CatalogPlugin.generate_catalog() default '
                 'method, should be overridden in subclass')
+
+# }}}
+
+class InterfaceActionBase(Plugin): # {{{
+
+    supported_platforms = ['windows', 'osx', 'linux']
+    author         = 'Kovid Goyal'
+    type = _('User Interface Action')
+    can_be_disabled = False
+
+    actual_plugin = None
+# }}}
+
+class PreferencesPlugin(Plugin): # {{{
+
+    '''
+    A plugin representing a widget displayed in the Preferences dialog.
+
+    This plugin has only one important method :meth:`create_widget`. The
+    various fields of the plugin control how it is categorized in the UI.
+    '''
+
+    supported_platforms = ['windows', 'osx', 'linux']
+    author         = 'Kovid Goyal'
+    type = _('Preferences')
+    can_be_disabled = False
+
+    #: Import path to module that contains a class named ConfigWidget
+    #: which implements the ConfigWidgetInterface. Used by
+    #: :meth:`create_widget`.
+    config_widget = None
+
+    #: Where in the list of categories the :attr:`category` of this plugin should be.
+    category_order = 100
+
+    #: Where in the list of names in a category, the :attr:`gui_name` of this
+    #: plugin should be
+    name_order = 100
+
+    #: The category this plugin should be in
+    category = None
+
+    #: The category name displayed to the user for this plugin
+    gui_category = None
+
+    #: The name displayed to the user for this plugin
+    gui_name = None
+
+    #: The icon for this plugin, should be an absolute path
+    icon = None
+
+    #: The description used for tooltips and the like
+    description = None
+
+    def create_widget(self, parent=None):
+        '''
+        Create and return the actual Qt widget used for setting this group of
+        preferences. The widget must implement the
+        :class:`calibre.gui2.preferences.ConfigWidgetInterface`.
+
+        The default implementation uses :attr:`config_widget` to instantiate
+        the widget.
+        '''
+        base, _, wc = self.config_widget.partition(':')
+        if not wc:
+            wc = 'ConfigWidget'
+        base = __import__(base, fromlist=[1])
+        widget = getattr(base, wc)
+        return widget(parent)
+
+# }}}
+
