@@ -5,10 +5,12 @@ __copyright__ = '2008, Kovid Goyal <kovid at kovidgoyal.net>'
 import textwrap
 import os
 import glob
-from calibre.customize import FileTypePlugin, MetadataReaderPlugin, MetadataWriterPlugin
+from calibre.customize import FileTypePlugin, MetadataReaderPlugin, \
+    MetadataWriterPlugin, PreferencesPlugin, InterfaceActionBase
 from calibre.constants import numeric_version
 from calibre.ebooks.metadata.archive import ArchiveExtract, get_cbz_metadata
 
+# To archive plugins {{{
 class HTML2ZIP(FileTypePlugin):
     name = 'HTML to ZIP'
     author = 'Kovid Goyal'
@@ -82,6 +84,8 @@ class PML2PMLZ(FileTypePlugin):
 
         return of.name
 
+# }}}
+
 # Metadata reader plugins {{{
 class ComicMetadataReader(MetadataReaderPlugin):
 
@@ -435,7 +439,7 @@ from calibre.ebooks.txt.output import TXTOutput
 from calibre.customize.profiles import input_profiles, output_profiles
 
 from calibre.devices.apple.driver import ITUNES
-from calibre.devices.hanlin.driver import HANLINV3, HANLINV5, BOOX
+from calibre.devices.hanlin.driver import HANLINV3, HANLINV5, BOOX, SPECTRA
 from calibre.devices.blackberry.driver import BLACKBERRY
 from calibre.devices.cybook.driver import CYBOOK
 from calibre.devices.eb600.driver import EB600, COOL_ER, SHINEBOOK, \
@@ -443,7 +447,7 @@ from calibre.devices.eb600.driver import EB600, COOL_ER, SHINEBOOK, \
                 BOOQ, ELONEX, POCKETBOOK301, MENTOR
 from calibre.devices.iliad.driver import ILIAD
 from calibre.devices.irexdr.driver import IREXDR1000, IREXDR800
-from calibre.devices.jetbook.driver import JETBOOK
+from calibre.devices.jetbook.driver import JETBOOK, MIBUK
 from calibre.devices.kindle.driver import KINDLE, KINDLE2, KINDLE_DX
 from calibre.devices.nook.driver import NOOK
 from calibre.devices.prs505.driver import PRS505
@@ -457,16 +461,22 @@ from calibre.devices.hanvon.driver import N516, EB511, ALEX, AZBOOKA, THEBOOK
 from calibre.devices.edge.driver import EDGE
 from calibre.devices.teclast.driver import TECLAST_K3, NEWSMY, IPAPYRUS
 from calibre.devices.sne.driver import SNE
-from calibre.devices.misc import PALMPRE, AVANT, SWEEX
+from calibre.devices.misc import PALMPRE, AVANT, SWEEX, PDNOVEL, KOGAN, GEMEI
 from calibre.devices.folder_device.driver import FOLDER_DEVICE_FOR_CONFIG
 from calibre.devices.kobo.driver import KOBO
 
 from calibre.ebooks.metadata.fetch import GoogleBooks, ISBNDB, Amazon, \
     LibraryThing
 from calibre.ebooks.metadata.douban import DoubanBooks
-from calibre.library.catalog import CSV_XML, EPUB_MOBI
+from calibre.ebooks.metadata.covers import OpenLibraryCovers, \
+        LibraryThingCovers
+from calibre.library.catalog import CSV_XML, EPUB_MOBI, BIBTEX
+from calibre.ebooks.epub.fix.unmanifested import Unmanifested
+from calibre.ebooks.epub.fix.epubcheck import Epubcheck
+
 plugins = [HTML2ZIP, PML2PMLZ, ArchiveExtract, GoogleBooks, ISBNDB, Amazon,
-        LibraryThing, DoubanBooks, CSV_XML, EPUB_MOBI]
+        LibraryThing, DoubanBooks, CSV_XML, EPUB_MOBI, BIBTEX, Unmanifested,
+        Epubcheck, OpenLibraryCovers, LibraryThingCovers]
 plugins += [
     ComicInput,
     EPUBInput,
@@ -511,6 +521,7 @@ plugins += [
     IREXDR1000,
     IREXDR800,
     JETBOOK,
+    MIBUK,
     SHINEBOOK,
     POCKETBOOK360,
     POCKETBOOK301,
@@ -556,6 +567,10 @@ plugins += [
     AVANT,
     MENTOR,
     SWEEX,
+    KOGAN,
+    PDNOVEL,
+    SPECTRA,
+    GEMEI,
     ITUNES,
 ]
 plugins += [x for x in list(locals().values()) if isinstance(x, type) and \
@@ -563,3 +578,282 @@ plugins += [x for x in list(locals().values()) if isinstance(x, type) and \
 plugins += [x for x in list(locals().values()) if isinstance(x, type) and \
                                         x.__name__.endswith('MetadataWriter')]
 plugins += input_profiles + output_profiles
+
+# Interface Actions {{{
+
+class ActionAdd(InterfaceActionBase):
+    name = 'Add Books'
+    actual_plugin = 'calibre.gui2.actions.add:AddAction'
+
+class ActionFetchAnnotations(InterfaceActionBase):
+    name = 'Fetch Annotations'
+    actual_plugin = 'calibre.gui2.actions.annotate:FetchAnnotationsAction'
+
+class ActionGenerateCatalog(InterfaceActionBase):
+    name = 'Generate Catalog'
+    actual_plugin = 'calibre.gui2.actions.catalog:GenerateCatalogAction'
+
+class ActionConvert(InterfaceActionBase):
+    name = 'Convert Books'
+    actual_plugin = 'calibre.gui2.actions.convert:ConvertAction'
+
+class ActionDelete(InterfaceActionBase):
+    name = 'Remove Books'
+    actual_plugin = 'calibre.gui2.actions.delete:DeleteAction'
+
+class ActionEditMetadata(InterfaceActionBase):
+    name = 'Edit Metadata'
+    actual_plugin = 'calibre.gui2.actions.edit_metadata:EditMetadataAction'
+
+class ActionView(InterfaceActionBase):
+    name = 'View'
+    actual_plugin = 'calibre.gui2.actions.view:ViewAction'
+
+class ActionFetchNews(InterfaceActionBase):
+    name = 'Fetch News'
+    actual_plugin = 'calibre.gui2.actions.fetch_news:FetchNewsAction'
+
+class ActionSaveToDisk(InterfaceActionBase):
+    name = 'Save To Disk'
+    actual_plugin = 'calibre.gui2.actions.save_to_disk:SaveToDiskAction'
+
+class ActionShowBookDetails(InterfaceActionBase):
+    name = 'Show Book Details'
+    actual_plugin = 'calibre.gui2.actions.show_book_details:ShowBookDetailsAction'
+
+class ActionRestart(InterfaceActionBase):
+    name = 'Restart'
+    actual_plugin = 'calibre.gui2.actions.restart:RestartAction'
+
+class ActionOpenFolder(InterfaceActionBase):
+    name = 'Open Folder'
+    actual_plugin = 'calibre.gui2.actions.open:OpenFolderAction'
+
+class ActionSendToDevice(InterfaceActionBase):
+    name = 'Send To Device'
+    actual_plugin = 'calibre.gui2.actions.device:SendToDeviceAction'
+
+class ActionConnectShare(InterfaceActionBase):
+    name = 'Connect Share'
+    actual_plugin = 'calibre.gui2.actions.device:ConnectShareAction'
+
+class ActionHelp(InterfaceActionBase):
+    name = 'Help'
+    actual_plugin = 'calibre.gui2.actions.help:HelpAction'
+
+class ActionPreferences(InterfaceActionBase):
+    name = 'Preferences'
+    actual_plugin = 'calibre.gui2.actions.preferences:PreferencesAction'
+
+class ActionSimilarBooks(InterfaceActionBase):
+    name = 'Similar Books'
+    actual_plugin = 'calibre.gui2.actions.similar_books:SimilarBooksAction'
+
+class ActionChooseLibrary(InterfaceActionBase):
+    name = 'Choose Library'
+    actual_plugin = 'calibre.gui2.actions.choose_library:ChooseLibraryAction'
+
+class ActionAddToLibrary(InterfaceActionBase):
+    name = 'Add To Library'
+    actual_plugin = 'calibre.gui2.actions.add_to_library:AddToLibraryAction'
+
+class ActionEditCollections(InterfaceActionBase):
+    name = 'Edit Collections'
+    actual_plugin = 'calibre.gui2.actions.edit_collections:EditCollectionsAction'
+
+class ActionCopyToLibrary(InterfaceActionBase):
+    name = 'Copy To Library'
+    actual_plugin = 'calibre.gui2.actions.copy_to_library:CopyToLibraryAction'
+
+plugins += [ActionAdd, ActionFetchAnnotations, ActionGenerateCatalog,
+        ActionConvert, ActionDelete, ActionEditMetadata, ActionView,
+        ActionFetchNews, ActionSaveToDisk, ActionShowBookDetails,
+        ActionRestart, ActionOpenFolder, ActionConnectShare,
+        ActionSendToDevice, ActionHelp, ActionPreferences, ActionSimilarBooks,
+        ActionAddToLibrary, ActionEditCollections, ActionChooseLibrary,
+        ActionCopyToLibrary]
+
+# }}}
+
+# Preferences Plugins {{{
+
+class LookAndFeel(PreferencesPlugin):
+    name = 'Look & Feel'
+    icon = I('lookfeel.png')
+    gui_name = _('Look and Feel')
+    category = 'Interface'
+    gui_category = _('Interface')
+    category_order = 1
+    name_order = 1
+    config_widget = 'calibre.gui2.preferences.look_feel'
+    description = _('Adjust the look and feel of the calibre interface'
+            ' to suit your tastes')
+
+class Behavior(PreferencesPlugin):
+    name = 'Behavior'
+    icon = I('config.png')
+    gui_name = _('Behavior')
+    category = 'Interface'
+    gui_category = _('Interface')
+    category_order = 1
+    name_order = 2
+    config_widget = 'calibre.gui2.preferences.behavior'
+    description = _('Change the way calibre behaves')
+
+class Columns(PreferencesPlugin):
+    name = 'Custom Columns'
+    icon = I('column.png')
+    gui_name = _('Add your own columns')
+    category = 'Interface'
+    gui_category = _('Interface')
+    category_order = 1
+    name_order = 3
+    config_widget = 'calibre.gui2.preferences.columns'
+    description = _('Add/remove your own columns to the calibre book list')
+
+class Toolbar(PreferencesPlugin):
+    name = 'Toolbar'
+    icon = I('wizard.png')
+    gui_name = _('Customize the toolbar')
+    category = 'Interface'
+    gui_category = _('Interface')
+    category_order = 1
+    name_order = 4
+    config_widget = 'calibre.gui2.preferences.toolbar'
+    description = _('Customize the toolbars and context menus, changing which'
+            ' actions are available in each')
+
+class InputOptions(PreferencesPlugin):
+    name = 'Input Options'
+    icon = I('arrow-down.png')
+    gui_name = _('Input Options')
+    category = 'Conversion'
+    gui_category = _('Conversion')
+    category_order = 2
+    name_order = 1
+    config_widget = 'calibre.gui2.preferences.conversion:InputOptions'
+    description = _('Set conversion options specific to each input format')
+
+class CommonOptions(PreferencesPlugin):
+    name = 'Common Options'
+    icon = I('convert.png')
+    gui_name = _('Common Options')
+    category = 'Conversion'
+    gui_category = _('Conversion')
+    category_order = 2
+    name_order = 2
+    config_widget = 'calibre.gui2.preferences.conversion:CommonOptions'
+    description = _('Set conversion options common to all formats')
+
+class OutputOptions(PreferencesPlugin):
+    name = 'Output Options'
+    icon = I('arrow-up.png')
+    gui_name = _('Output Options')
+    category = 'Conversion'
+    gui_category = _('Conversion')
+    category_order = 2
+    name_order = 3
+    config_widget = 'calibre.gui2.preferences.conversion:OutputOptions'
+    description = _('Set conversion options specific to each output format')
+
+class Adding(PreferencesPlugin):
+    name = 'Adding'
+    icon = I('add_book.png')
+    gui_name = _('Adding books')
+    category = 'Import/Export'
+    gui_category = _('Import/Export')
+    category_order = 3
+    name_order = 1
+    config_widget = 'calibre.gui2.preferences.adding'
+    description = _('Control how calibre reads metadata from files when '
+            'adding books')
+
+class Saving(PreferencesPlugin):
+    name = 'Saving'
+    icon = I('save.png')
+    gui_name = _('Saving books to disk')
+    category = 'Import/Export'
+    gui_category = _('Import/Export')
+    category_order = 3
+    name_order = 2
+    config_widget = 'calibre.gui2.preferences.saving'
+    description = _('Control how calibre exports files from its database '
+            'to disk when using Save to disk')
+
+class Sending(PreferencesPlugin):
+    name = 'Sending'
+    icon = I('sync.png')
+    gui_name = _('Sending books to devices')
+    category = 'Import/Export'
+    gui_category = _('Import/Export')
+    category_order = 3
+    name_order = 3
+    config_widget = 'calibre.gui2.preferences.sending'
+    description = _('Control how calibre transfers files to your '
+            'ebook reader')
+
+class Email(PreferencesPlugin):
+    name = 'Email'
+    icon = I('mail.png')
+    gui_name = _('Sharing books by email')
+    category = 'Sharing'
+    gui_category = _('Sharing')
+    category_order = 4
+    name_order = 1
+    config_widget = 'calibre.gui2.preferences.emailp'
+    description = _('Setup sharing of books via email. Can be used '
+            'for automatic sending of downloaded news to your devices')
+
+class Server(PreferencesPlugin):
+    name = 'Server'
+    icon = I('network-server.png')
+    gui_name = _('Sharing over the net')
+    category = 'Sharing'
+    gui_category = _('Sharing')
+    category_order = 4
+    name_order = 2
+    config_widget = 'calibre.gui2.preferences.server'
+    description = _('Setup the calibre Content Server which will '
+            'give you access to your calibre library from anywhere, '
+            'on any device, over the internet')
+
+class Plugins(PreferencesPlugin):
+    name = 'Plugins'
+    icon = I('plugins.png')
+    gui_name = _('Plugins')
+    category = 'Advanced'
+    gui_category = _('Advanced')
+    category_order = 5
+    name_order = 1
+    config_widget = 'calibre.gui2.preferences.plugins'
+    description = _('Add/remove/customize various bits of calibre '
+            'functionality')
+
+class Tweaks(PreferencesPlugin):
+    name = 'Tweaks'
+    icon = I('drawer.png')
+    gui_name = _('Tweaks')
+    category = 'Advanced'
+    gui_category = _('Advanced')
+    category_order = 5
+    name_order = 2
+    config_widget = 'calibre.gui2.preferences.tweaks'
+    description = _('Fine tune how calibre behaves in various contexts')
+
+class Misc(PreferencesPlugin):
+    name = 'Misc'
+    icon = I('exec.png')
+    gui_name = _('Miscellaneous')
+    category = 'Advanced'
+    gui_category = _('Advanced')
+    category_order = 5
+    name_order = 3
+    config_widget = 'calibre.gui2.preferences.misc'
+    description = _('Miscellaneous advanced configuration')
+
+plugins += [LookAndFeel, Behavior, Columns, Toolbar, InputOptions,
+        CommonOptions, OutputOptions, Adding, Saving, Sending, Email, Server,
+        Plugins, Tweaks, Misc]
+
+#}}}
+
