@@ -32,15 +32,8 @@ class IRIVER_STORY(USBMS):
 
     MAIN_MEMORY_VOLUME_LABEL  = 'Story Main Memory'
     STORAGE_CARD_VOLUME_LABEL = 'Story Storage Card'
+    EBOOK_DIR_MAIN = 'Book'
 
     SUPPORTS_SUB_DIRS = True
 
-    def windows_open_callback(self, drives):
-        main = drives.get('main', None)
-        card = drives.get('carda', None)
-        if card and main and card < main:
-            drives['main'] = card
-            drives['carda'] = main
-
-        return drives
 
