@@ -10,12 +10,13 @@ from base64 import b64decode
 from uuid import uuid4
 from lxml import etree
 
-from calibre import prints, guess_type
+from calibre import prints, guess_type, isbytestring
 from calibre.devices.errors import DeviceError
 from calibre.devices.usbms.driver import debug_print
-from calibre.constants import DEBUG
+from calibre.constants import DEBUG, preferred_encoding
 from calibre.ebooks.chardet import xml_to_unicode
-from calibre.ebooks.metadata import authors_to_string, title_sort
+from calibre.ebooks.metadata import authors_to_string, title_sort, \
+                                    authors_to_sort_string
 
 # Utility functions {{{
 EMPTY_CARD_CACHE = '''\
@@ -46,7 +47,11 @@ def strptime(src):
     return time.strptime(' '.join(src), '%w, %d %m %Y %H:%M:%S %Z')
 
 def strftime(epoch, zone=time.localtime):
-    src = time.strftime("%w, %d %m %Y %H:%M:%S GMT", zone(epoch)).split()
+    try:
+        src = time.strftime("%w, %d %m %Y %H:%M:%S GMT", zone(epoch)).split()
+    except:
+        src = time.strftime("%w, %d %m %Y %H:%M:%S GMT", zone()).split()
+
     src[0] = INVERSE_DAY_MAP[int(src[0][:-1])]+','
     src[2] = INVERSE_MONTH_MAP[int(src[2])]
     return ' '.join(src)
@@ -84,6 +89,10 @@ class XMLCache(object):
                         raw, strip_encoding_pats=True, assume_utf8=True,
                         verbose=DEBUG)[0],
                         parser=parser)
+            if self.roots[source_id] is None:
+                raise Exception(('The SONY database at %s is corrupted. Try '
+                        ' disconnecting and reconnecting your reader.')%path)
+
         # }}}
 
         recs = self.roots[0].xpath('//*[local-name()="records"]')
@@ -328,7 +337,10 @@ class XMLCache(object):
                             'descendant::*[local-name()="jpeg"]|'
                             'descendant::*[local-name()="png"]'):
                         if img.text:
-                            raw = b64decode(img.text.strip())
+                            try:
+                                raw = b64decode(img.text.strip())
+                            except:
+                                continue
                             book.thumbnail = raw
                             break
                     break
@@ -340,22 +352,29 @@ class XMLCache(object):
     # Update XML from JSON {{{
     def update(self, booklists, collections_attributes):
         debug_print('Starting update', collections_attributes)
+        use_tz_var = False
         for i, booklist in booklists.items():
             playlist_map = self.build_id_playlist_map(i)
             debug_print('Updating XML Cache:', i)
             root = self.record_roots[i]
             lpath_map = self.build_lpath_map(root)
+            gtz_count = ltz_count = 0
+            use_tz_var = False
             for book in booklist:
                 path = os.path.join(self.prefixes[i], *(book.lpath.split('/')))
                 record = lpath_map.get(book.lpath, None)
                 if record is None:
                     record = self.create_text_record(root, i, book.lpath)
-                self.update_text_record(record, book, path, i)
+                (gtz_count, ltz_count, use_tz_var) = \
+                    self.update_text_record(record, book, path, i,
+                                            gtz_count, ltz_count, use_tz_var)
                 # Ensure the collections in the XML database are recorded for
                 # this book
                 if book.device_collections is None:
                     book.device_collections = []
                 book.device_collections = playlist_map.get(book.lpath, [])
+            debug_print('Timezone votes: %d GMT, %d LTZ, use_tz_var=%s'%
+                                        (gtz_count, ltz_count, use_tz_var))
             self.update_playlists(i, root, booklist, collections_attributes)
         # Update the device collections because update playlist could have added
         # some new ones.
@@ -453,26 +472,72 @@ class XMLCache(object):
         root.append(ans)
         return ans
 
-    def update_text_record(self, record, book, path, bl_index):
+    def update_text_record(self, record, book, path, bl_index,
+                           gtz_count, ltz_count, use_tz_var):
         '''
         Update the Sony database from the book. This is done if the timestamp in
         the db differs from the timestamp on the file.
         '''
+
+        # It seems that a Sony device can sometimes know what timezone it is in,
+        # and apparently converts the dates to GMT when it writes them to its
+        # DB. We can detect that a device is timezone-aware because there is a
+        # 'tz' variable in the Sony DB, which we can set to "0" to tell the
+        # device to ignore its own timezone when comparing mtime to the date in
+        # the DB.
+
+        # Unfortunately, if there is no tz variable in the DB, then we can't
+        # tell when the device applies a timezone conversion. We use a horrible
+        # heuristic to work around this problem. First, set dates only for new
+        # books, trying to avoid upsetting the sony. Second, voting: if a book
+        # is not new, compare its Sony DB date against localtime and gmtime.
+        # Count the matches. When we must set a date, use the one with the most
+        # matches. Use localtime if the case of a tie, and hope it is right.
         timestamp = os.path.getmtime(path)
-        date = strftime(timestamp)
-        if date != record.get('date', None):
-            record.set('date', date)
-        record.set('size', str(os.stat(path).st_size))
+        rec_date = record.get('date', None)
+
+        def clean(x):
+            if isbytestring(x):
+                x = x.decode(preferred_encoding, 'replace')
+            x.replace(u'\0', '')
+            return x
+
+        if not getattr(book, '_new_book', False): # book is not new
+                if record.get('tz', None) is not None:
+                    use_tz_var = True
+                if strftime(timestamp, zone=time.gmtime) == rec_date:
+                    gtz_count += 1
+                elif strftime(timestamp, zone=time.localtime) == rec_date:
+                    ltz_count += 1
+        else: # book is new. Set the time using the current votes
+            if use_tz_var:
+                tz = time.localtime
+                record.set('tz', '0')
+                debug_print("Use localtime TZ and tz='0' for new book", book.lpath)
+            elif ltz_count >= gtz_count:
+                tz = time.localtime
+                debug_print("Use localtime TZ for new book", book.lpath)
+            else:
+                tz = time.gmtime
+                debug_print("Use GMT TZ for new book", book.lpath)
+            date = strftime(timestamp, zone=tz)
+            record.set('date', clean(date))
+        record.set('size', clean(str(os.stat(path).st_size)))
         title = book.title if book.title else _('Unknown')
-        record.set('title', title)
+        record.set('title', clean(title))
         ts = book.title_sort
         if not ts:
             ts = title_sort(title)
-        record.set('titleSorter', ts)
-        if self.use_author_sort and book.author_sort is not None:
-            record.set('author', book.author_sort)
+        record.set('titleSorter', clean(ts))
+        if self.use_author_sort:
+            if book.author_sort:
+                aus = book.author_sort
+            else:
+                debug_print('Author_sort is None for book', book.lpath)
+                aus = authors_to_sort_string(book.authors)
+            record.set('author', clean(aus))
         else:
-            record.set('author', authors_to_string(book.authors))
+            record.set('author', clean(authors_to_string(book.authors)))
         ext = os.path.splitext(path)[1]
         if ext:
             ext = ext[1:].lower()
@@ -480,12 +545,13 @@ class XMLCache(object):
             if mime is None:
                 mime = guess_type('a.'+ext)[0]
             if mime is not None:
-                record.set('mime', mime)
+                record.set('mime', clean(mime))
         if 'sourceid' not in record.attrib:
             record.set('sourceid', '1')
         if 'id' not in record.attrib:
             num = self.max_id(record.getroottree().getroot())
             record.set('id', str(num+1))
+        return (gtz_count, ltz_count, use_tz_var)
     # }}}
 
     # Writing the XML files {{{
