@@ -33,6 +33,10 @@ def debug_print(*args):
 # CLI must come before Device as it implements the CLI functions that
 # are inherited from the device interface in Device.
 class USBMS(CLI, Device):
+    '''
+    The base class for all USBMS devices. Implements the logic for
+    sending/getting/updating metadata/caching metadata/etc.
+    '''
 
     description    = _('Communicate with an eBook reader.')
     author         = _('John Schember')
@@ -58,7 +62,7 @@ class USBMS(CLI, Device):
 
         debug_print ('USBMS: Fetching list of books from device. oncard=', oncard)
 
-        dummy_bl = BookList(None, None, None)
+        dummy_bl = self.booklist_class(None, None, None)
 
         if oncard == 'carda' and not self._card_a_prefix:
             self.report_progress(1.0, _('Getting list of books on device...'))
@@ -78,6 +82,8 @@ class USBMS(CLI, Device):
             self.EBOOK_DIR_CARD_B if oncard == 'cardb' else \
             self.get_main_ebook_dir()
 
+        debug_print ('USBMS: dirs are:', prefix, ebook_dirs)
+
         # get the metadata cache
         bl = self.booklist_class(oncard, prefix, self.settings)
         need_sync = self.parse_metadata_cache(bl, prefix, self.METADATA_CACHE)
@@ -193,10 +199,13 @@ class USBMS(CLI, Device):
 
     def upload_cover(self, path, filename, metadata):
         '''
-        :path: the full path were the associated book is located.
-        :filename: the name of the book file without the extension.
-        :metadata: metadata belonging to the book. Use metadata.thumbnail
-        for cover
+        Upload book cover to the device. Default implementation does nothing.
+
+        :param path: the full path were the associated book is located.
+        :param filename: the name of the book file without the extension.
+        :param metadata: metadata belonging to the book. Use metadata.thumbnail
+                         for cover
+
         '''
         pass
 
