@@ -13,7 +13,9 @@ from calibre.customize.ui import input_profiles, output_profiles, \
 from calibre.ebooks.conversion.preprocess import HTMLPreProcessor
 from calibre.ptempfile import PersistentTemporaryDirectory
 from calibre.utils.date import parse_date
+from calibre.utils.zipfile import ZipFile
 from calibre import extract, walk
+from calibre.constants import __version__
 
 DEBUG_README=u'''
 This debug directory contains snapshots of the e-book as it passes through the
@@ -298,12 +300,16 @@ OptionRecommendation(name='margin_right',
         help=_('Set the right margin in pts. Default is %default. '
             'Note: 72 pts equals 1 inch')),
 
-OptionRecommendation(name='dont_justify',
-        recommended_value=False, level=OptionRecommendation.LOW,
-        help=_('Do not force text to be justified in output. Whether text '
-            'is actually displayed justified or not depends on whether '
-            'the ebook format and reading device support justification.')
-        ),
+OptionRecommendation(name='change_justification',
+        recommended_value='original', level=OptionRecommendation.LOW,
+        choices=['left','justify','original'],
+        help=_('Change text justification. A value of "left" converts all'
+            ' justified text in the source to left aligned (i.e. '
+            'unjustified) text. A value of "justify" converts all '
+            'unjustified text to justified. A value of "original" '
+            '(the default) does not change justification in the '
+            'source file. Note that only some output formats support '
+            'justification.')),
 
 OptionRecommendation(name='remove_paragraph_spacing',
         recommended_value=False, level=OptionRecommendation.LOW,
@@ -405,6 +411,18 @@ OptionRecommendation(name='asciiize',
 )
         ),
 
+OptionRecommendation(name='keep_ligatures',
+            recommended_value=False, level=OptionRecommendation.LOW,
+            help=_('Preserve ligatures present in the input document. '
+                'A ligature is a special rendering of a pair of '
+                'characters like ff, fi, fl et cetera. '
+                'Most readers do not have support for '
+                'ligatures in their default fonts, so they are '
+                'unlikely to render correctly. By default, calibre '
+                'will turn a ligature into the corresponding pair of normal '
+                'characters. This option will preserve them instead.')
+        ),
+
 OptionRecommendation(name='title',
     recommended_value=None, level=OptionRecommendation.LOW,
     help=_('Set the title.')),
@@ -671,7 +689,10 @@ OptionRecommendation(name='timestamp',
         if mi.cover:
             if mi.cover.startswith('http:') or mi.cover.startswith('https:'):
                 mi.cover = self.download_cover(mi.cover)
-            mi.cover_data = ('', open(mi.cover, 'rb').read())
+            ext = mi.cover.rpartition('.')[-1].lower().strip()
+            if ext not in ('png', 'jpg', 'jpeg'):
+                ext = 'jpg'
+            mi.cover_data = (ext, open(mi.cover, 'rb').read())
             mi.cover = None
         self.user_metadata = mi
 
@@ -685,15 +706,22 @@ OptionRecommendation(name='timestamp',
             for rec in group:
                 setattr(self.opts, rec.option.name, rec.recommended_value)
 
-        for x in input_profiles():
-            if x.short_name == self.opts.input_profile:
-                self.opts.input_profile = x
-                break
+        def set_profile(profiles, which):
+            attr = which + '_profile'
+            sval = getattr(self.opts, attr)
+            for x in profiles():
+                if x.short_name == sval:
+                    setattr(self.opts, attr, x)
+                    return
+            self.log.warn(
+                'Profile (%s) %r is no longer available, using default'%(which, sval))
+            for x in profiles():
+                if x.short_name == 'default':
+                    setattr(self.opts, attr, x)
+                    break
 
-        for x in output_profiles():
-            if x.short_name == self.opts.output_profile:
-                self.opts.output_profile = x
-                break
+        set_profile(input_profiles, 'input')
+        set_profile(output_profiles, 'output')
 
         self.read_user_metadata()
         self.opts.no_inline_navbars = self.opts.output_profile.supports_mobi_indexing \
@@ -703,6 +731,7 @@ OptionRecommendation(name='timestamp',
         if self.opts.verbose > 1:
             self.log.debug('Resolved conversion options')
             try:
+                self.log.debug('calibre version:', __version__)
                 self.log.debug(pprint.pformat(self.opts.__dict__))
             except:
                 self.log.exception('Failed to get resolved conversion options')
@@ -726,6 +755,13 @@ OptionRecommendation(name='timestamp',
         else:
             os.makedirs(out_dir)
             self.dump_oeb(ret, out_dir)
+        if self.input_fmt == 'recipe':
+            zf = ZipFile(os.path.join(self.opts.debug_pipeline,
+                'periodical.downloaded_recipe'), 'w')
+            zf.add_dir(out_dir)
+            with self.input_plugin:
+                self.input_plugin.save_download(zf)
+            zf.close()
 
         self.log.info('Input debug saved to:', out_dir)
 
@@ -773,26 +809,29 @@ OptionRecommendation(name='timestamp',
         self.ui_reporter(0.01, _('Converting input to HTML...'))
         ir = CompositeProgressReporter(0.01, 0.34, self.ui_reporter)
         self.input_plugin.report_progress = ir
-        self.oeb = self.input_plugin(stream, self.opts,
-                                    self.input_fmt, self.log,
-                                    accelerators, tdir)
-        if self.opts.debug_pipeline is not None:
-            self.dump_input(self.oeb, tdir)
-            if self.abort_after_input_dump:
-                return
-        if self.input_fmt == 'recipe':
-            self.opts_to_mi(self.user_metadata)
-        if not hasattr(self.oeb, 'manifest'):
-            self.oeb = create_oebbook(self.log, self.oeb, self.opts,
-                    self.input_plugin)
-        self.input_plugin.postprocess_book(self.oeb, self.opts, self.log)
-        self.opts.is_image_collection = self.input_plugin.is_image_collection
-        pr = CompositeProgressReporter(0.34, 0.67, self.ui_reporter)
-        self.flush()
-        if self.opts.debug_pipeline is not None:
-            out_dir = os.path.join(self.opts.debug_pipeline, 'parsed')
-            self.dump_oeb(self.oeb, out_dir)
-            self.log('Parsed HTML written to:', out_dir)
+        with self.input_plugin:
+            self.oeb = self.input_plugin(stream, self.opts,
+                                        self.input_fmt, self.log,
+                                        accelerators, tdir)
+            if self.opts.debug_pipeline is not None:
+                self.dump_input(self.oeb, tdir)
+                if self.abort_after_input_dump:
+                    return
+            if self.input_fmt in ('recipe', 'downloaded_recipe'):
+                self.opts_to_mi(self.user_metadata)
+            if not hasattr(self.oeb, 'manifest'):
+                self.oeb = create_oebbook(self.log, self.oeb, self.opts,
+                        self.input_plugin)
+            self.input_plugin.postprocess_book(self.oeb, self.opts, self.log)
+            self.opts.is_image_collection = self.input_plugin.is_image_collection
+            pr = CompositeProgressReporter(0.34, 0.67, self.ui_reporter)
+            self.flush()
+            if self.opts.debug_pipeline is not None:
+                out_dir = os.path.join(self.opts.debug_pipeline, 'parsed')
+                self.dump_oeb(self.oeb, out_dir)
+                self.log('Parsed HTML written to:', out_dir)
+            self.input_plugin.specialize(self.oeb, self.opts, self.log,
+                    self.output_fmt)
 
         pr(0., _('Running transforms on ebook...'))
 
@@ -882,7 +921,8 @@ OptionRecommendation(name='timestamp',
         our = CompositeProgressReporter(0.67, 1., self.ui_reporter)
         self.output_plugin.report_progress = our
         our(0., _('Creating')+' %s'%self.output_plugin.name)
-        self.output_plugin.convert(self.oeb, self.output, self.input_plugin,
+        with self.output_plugin:
+            self.output_plugin.convert(self.oeb, self.output, self.input_plugin,
                 self.opts, self.log)
         self.ui_reporter(1.)
         run_plugins_on_postprocess(self.output, self.output_fmt)
