@@ -86,11 +86,14 @@ class FB2MLizer(object):
         output.append(self.fb2_footer())
         output = ''.join(output).replace(u'ghji87yhjko0Caliblre-toc-placeholder-for-insertion-later8ujko0987yjk', self.get_toc())
         output = self.clean_text(output)
+        if self.opts.sectionize_chapters:
+            output = self.sectionize_chapters(output)
         return u'<?xml version="1.0" encoding="UTF-8"?>\n%s' % etree.tostring(etree.fromstring(output), encoding=unicode, pretty_print=True)
 
     def clean_text(self, text):
-        text = re.sub('<p>[ ]*</p>', '', text)
-
+        text = re.sub(r'(?miu)<p>\s*</p>', '', text)
+        text = re.sub(r'(?miu)\s+</p>', '</p>', text)
+        text = re.sub(r'(?miu)</p><p>', '</p>\n\n<p>', text)
         return text
 
     def fb2_header(self):
@@ -149,6 +152,18 @@ class FB2MLizer(object):
                     self.oeb.warn('Ignoring toc item: %s not found in document.' % item)
         return ''.join(toc)
 
+    def sectionize_chapters(self, text):
+        def remove_p(t):
+            t = t.replace('<p>', '')
+            t = t.replace('</p>', '')
+            return t
+        text = re.sub(r'(?imsu)(<p>)\s*(?P<anchor><a\s+id="calibre_link-\d+"\s*/>)\s*(</p>)\s*(<p>)\s*(?P<strong><strong>.+?</strong>)\s*(</p>)', lambda mo: '</section><section>%s<title><p>%s</p></title>' % (mo.group('anchor'), remove_p(mo.group('strong'))), text)
+        text = re.sub(r'(?imsu)(<p>)\s*(?P<anchor><a\s+id="calibre_link-\d+"\s*/>)\s*(</p>)\s*(?P<strong><strong>.+?</strong>)', lambda mo: '</section><section>%s<title><p>%s</p></title>' % (mo.group('anchor'), remove_p(mo.group('strong'))), text)
+        text = re.sub(r'(?imsu)(?P<anchor><a\s+id="calibre_link-\d+"\s*/>)\s*(<p>)\s*(?P<strong><strong>.+?</strong>)\s*(</p>)', lambda mo: '</section><section>%s<title><p>%s</p></title>' % (mo.group('anchor'), remove_p(mo.group('strong'))), text)
+        text = re.sub(r'(?imsu)(<p>)\s*(?P<anchor><a\s+id="calibre_link-\d+"\s*/>)\s*(?P<strong><strong>.+?</strong>)\s*(</p>)', lambda mo: '</section><section>%s<title><p>%s</p></title>' % (mo.group('anchor'), remove_p(mo.group('strong'))), text)
+        text = re.sub(r'(?imsu)(?P<anchor><a\s+id="calibre_link-\d+"\s*/>)\s*(?P<strong><strong>.+?</strong>)', lambda mo: '</section><section>%s<title><p>%s</p></title>' % (mo.group('anchor'), remove_p(mo.group('strong'))), text)
+        return text
+
     def get_text(self):
         text = []
         for item in self.oeb_book.spine:
