@@ -9,12 +9,29 @@ from threading import Thread
 from calibre import prints
 from calibre.utils.config import OptionParser
 from calibre.utils.logging import default_log
+from calibre.utils.titlecase import titlecase
 from calibre.customize import Plugin
-from calibre.ebooks.metadata.library_thing import check_for_cover
+from calibre.ebooks.metadata.covers import check_for_cover
 
 metadata_config = None
 
 class MetadataSource(Plugin): # {{{
+    '''
+    Represents a source to query for metadata. Subclasses must implement
+    at least the fetch method.
+
+    When :meth:`fetch` is called, the `self` object will have the following
+    useful attributes (each of which may be None)::
+
+        title, book_author, publisher, isbn, log, verbose and extra
+
+    Use these attributes to construct the search query. extra is reserved for
+    future use.
+
+    The fetch method must store the results in `self.results` as a list of
+    :class:`MetaInformation` objects. If there is an error, it should be stored
+    in `self.exception` and `self.tb` (for the traceback).
+    '''
 
     author = 'Kovid Goyal'
 
@@ -273,11 +290,10 @@ def filter_metadata_results(item):
 
 def do_cover_check(item):
     item.has_cover = False
-    if item.isbn:
-        try:
-            item.has_cover = check_for_cover(item.isbn)
-        except:
-            pass # Cover not found
+    try:
+        item.has_cover = check_for_cover(item)
+    except:
+        pass # Cover not found
 
 def check_for_covers(items):
     threads = [Thread(target=do_cover_check, args=(item,)) for item in items]
@@ -351,9 +367,13 @@ def search(title=None, author=None, publisher=None, isbn=None, isbndb_key=None,
     if len(results) > 1:
         if not results[0].comments or len(results[0].comments) == 0:
             for r in results[1:]:
-                if title.lower() == r.title[:len(title)].lower() and r.comments and len(r.comments):
-                    results[0].comments = r.comments
-                    break
+                try:
+                    if title and title.lower() == r.title[:len(title)].lower() \
+                            and r.comments and len(r.comments):
+                        results[0].comments = r.comments
+                        break
+                except:
+                    pass
         # Find a pubdate
         pubdate = None
         for r in results:
@@ -365,6 +385,16 @@ def search(title=None, author=None, publisher=None, isbn=None, isbndb_key=None,
                 if r.pubdate is None:
                     r.pubdate = pubdate
 
+    def fix_case(x):
+        if x and x.isupper():
+            x = titlecase(x)
+        return x
+
+    for r in results:
+        r.title = fix_case(r.title)
+        if r.authors:
+            r.authors = list(map(fix_case, r.authors))
+
     return results, [(x.name, x.exception, x.tb) for x in fetchers]
 
 def get_social_metadata(mi, verbose=0):
