@@ -916,7 +916,7 @@ class OPF(object):
             raw = '<?xml version="1.0"  encoding="%s"?>\n'%encoding.upper()+raw
         return raw
 
-    def smart_update(self, mi):
+    def smart_update(self, mi, replace_metadata=False):
         for attr in ('title', 'authors', 'author_sort', 'title_sort',
                      'publisher', 'series', 'series_index', 'rating',
                      'isbn', 'language', 'tags', 'category', 'comments',
@@ -1105,7 +1105,8 @@ class OPFCreator(MetaInformation):
             spine.set('toc', 'ncx')
         if self.spine is not None:
             for ref in self.spine:
-                spine.append(E.itemref(idref=ref.id))
+                if ref.id is not None:
+                    spine.append(E.itemref(idref=ref.id))
         guide = E.guide()
         if self.guide is not None:
             for ref in self.guide:
