@@ -7,6 +7,7 @@ import os, glob, re, textwrap
 from lxml import etree
 
 from calibre.customize.conversion import InputFormatPlugin
+from calibre.ebooks.conversion.preprocess import line_length
 
 class InlineClass(etree.XSLTExtension):
 
@@ -120,25 +121,19 @@ class RTFInput(InputFormatPlugin):
         return self.convert_images(imap)
 
     def convert_images(self, imap):
-        from calibre.utils.PythonMagickWand import ImageMagick
-        with ImageMagick():
-            for count, val in imap.items():
-                try:
-                    imap[count] = self.convert_image(val)
-                except:
-                    self.log.exception('Failed to convert', val)
+        for count, val in imap.items():
+            try:
+                imap[count] = self.convert_image(val)
+            except:
+                self.log.exception('Failed to convert', val)
         return imap
 
     def convert_image(self, name):
-        import calibre.utils.PythonMagickWand as p
-        img = p.NewMagickWand()
-        if img < 0:
-            raise RuntimeError('Cannot create wand.')
-        if not p.MagickReadImage(img, name):
-            self.log.warn('Failed to read image:', name)
+        from calibre.utils.magick import Image
+        img = Image()
+        img.open(name)
         name = name.replace('.wmf', '.jpg')
-        p.MagickWriteImage(img, name)
-
+        img.save(name)
         return name
 
 
@@ -190,14 +185,21 @@ class RTFInput(InputFormatPlugin):
         from calibre.ebooks.metadata.meta import get_metadata
         from calibre.ebooks.metadata.opf2 import OPFCreator
         from calibre.ebooks.rtf2xml.ParseRtf import RtfInvalidCodeException
+        self.options = options
         self.log = log
         self.log('Converting RTF to XML...')
+        #Name of the preprocesssed RTF file
         fname = self.preprocess(stream.name)
         try:
             xml = self.generate_xml(fname)
         except RtfInvalidCodeException, e:
             raise ValueError(_('This RTF file has a feature calibre does not '
             'support. Convert it to HTML first and then try it.\n%s')%e)
+
+        '''dataxml = open('dataxml.xml', 'w')
+        dataxml.write(xml)
+        dataxml.close'''
+
         d = glob.glob(os.path.join('*_rtf_pict_dir', 'picts.rtf'))
         if d:
             imap = {}
@@ -205,6 +207,7 @@ class RTFInput(InputFormatPlugin):
                 imap = self.extract_images(d[0])
             except:
                 self.log.exception('Failed to extract images...')
+
         self.log('Parsing XML...')
         parser = etree.XMLParser(recover=True, no_network=True)
         doc = etree.fromstring(xml, parser=parser)
@@ -214,10 +217,10 @@ class RTFInput(InputFormatPlugin):
             name = imap.get(num, None)
             if name is not None:
                 pict.set('num', name)
+
         self.log('Converting XML to HTML...')
         inline_class = InlineClass(self.log)
         styledoc = etree.fromstring(P('templates/rtf.xsl', data=True))
-
         extensions = { ('calibre', 'inline-class') : inline_class }
         transform = etree.XSLT(styledoc, extensions=extensions)
         result = transform(doc)
@@ -225,6 +228,17 @@ class RTFInput(InputFormatPlugin):
         with open(html, 'wb') as f:
             res = transform.tostring(result)
             res = res[:100].replace('xmlns:html', 'xmlns') + res[100:]
+            if self.options.preprocess_html:
+                self.log("*********  Preprocessing HTML  *********")
+                # Detect Chapters to match the xpath in the GUI
+                chapdetect = re.compile(r'<p[^>]*>\s*<span[^>]*>\s*(?P<chap>(<(i|b)><(i|b)>|<(i|b)>)?(.?Chapter|Epilogue|Prologue|Book|Part|Dedication)\s*([\d\w-]+(\s\w+)?)?(</(i|b)></(i|b)>|</(i|b)>)?)\s*</span>\s*</p>', re.IGNORECASE)
+                res = chapdetect.sub('<h2>'+'\g<chap>'+'</h2>\n', res)
+                # Unwrap lines using punctation if the median length of all lines is less than 150
+                length = line_length('html', res, 0.4)
+                self.log("*** Median length is " + str(length) + " ***")
+                unwrap = re.compile(r"(?<=.{%i}[a-z,;:\IA])\s*</span>\s*(</p>)?\s*(?P<up2threeblanks><p[^>]*>\s*(<span[^>]*>\s*</span>\s*)</p>\s*){0,3}\s*<p[^>]*>\s*(<span[^>]*>)?\s*" % length, re.UNICODE)
+                if length < 150:
+                    res = unwrap.sub(' ', res)
             f.write(res)
         self.write_inline_css(inline_class)
         stream.seek(0)
