@@ -4,7 +4,7 @@ __copyright__ = '2008, Kovid Goyal <kovid at kovidgoyal.net>'
 import os, sys
 from threading import RLock
 
-from PyQt4.Qt import QVariant, QFileInfo, QObject, SIGNAL, QBuffer, Qt, QSize, \
+from PyQt4.Qt import QVariant, QFileInfo, QObject, SIGNAL, QBuffer, Qt, \
                          QByteArray, QTranslator, QCoreApplication, QThread, \
                          QEvent, QTimer, pyqtSignal, QDate, QDesktopServices, \
                          QFileDialog, QMessageBox, QPixmap, QFileIconProvider, \
@@ -19,8 +19,40 @@ from calibre.ebooks.metadata.meta import get_metadata, metadata_from_formats
 from calibre.ebooks.metadata import MetaInformation
 from calibre.utils.date import UNDEFINED_DATE
 
+# Setup gprefs {{{
 gprefs = JSONConfig('gui')
 
+gprefs.defaults['action-layout-toolbar'] = (
+        'Add Books', 'Edit Metadata', None, 'Convert Books', 'View', None,
+        'Choose Library', 'Donate', None, 'Fetch News', 'Save To Disk',
+        'Connect Share', None, 'Remove Books', None, 'Help', 'Preferences',
+        )
+
+gprefs.defaults['action-layout-toolbar-device'] = (
+        'Add Books', 'Edit Metadata', None, 'Convert Books', 'View',
+        'Send To Device', None, None, 'Location Manager', None, None,
+        'Fetch News', 'Save To Disk', 'Connect Share', None,
+        'Remove Books', None, 'Help', 'Preferences',
+        )
+
+gprefs.defaults['action-layout-context-menu'] = (
+        'Edit Metadata', 'Send To Device', 'Save To Disk',
+        'Connect Share', 'Copy To Library', None,
+        'Convert Books', 'View', 'Open Folder', 'Show Book Details',
+        'Similar Books', None, 'Remove Books',
+        )
+
+gprefs.defaults['action-layout-context-menu-device'] = (
+        'View', 'Save To Disk', None, 'Remove Books', None,
+        'Add To Library', 'Edit Collections',
+        )
+
+gprefs.defaults['show_splash_screen'] = True
+gprefs.defaults['toolbar_icon_size'] = 'medium'
+gprefs.defaults['toolbar_text'] = 'auto'
+
+# }}}
+
 NONE = QVariant() #: Null value to return from the data function of item models
 UNDEFINED_QDATE = QDate(UNDEFINED_DATE)
 
@@ -33,10 +65,6 @@ def _config():
               help=_('Send file to storage card instead of main memory by default'))
     c.add_opt('confirm_delete', default=False,
               help=_('Confirm before deleting'))
-    c.add_opt('toolbar_icon_size', default=QSize(48, 48),
-              help=_('Toolbar icon size')) # value QVariant.toSize
-    c.add_opt('show_text_in_toolbar', default=True,
-              help=_('Show button labels in the toolbar'))
     c.add_opt('main_window_geometry', default=None,
               help=_('Main window geometry')) # value QVariant.toByteArray
     c.add_opt('new_version_notification', default=True,
@@ -70,8 +98,6 @@ def _config():
     c.add_opt('default_send_to_device_action', default=None,
             help=_('Default action to perform when send to device button is '
                 'clicked'))
-    c.add_opt('show_donate_button', default=True,
-            help='Show donation button')
     c.add_opt('asked_library_thing_password', default=False,
             help='Asked library thing password at least once.')
     c.add_opt('search_as_you_type', default=True,
@@ -207,7 +233,7 @@ def warning_dialog(parent, title, msg, det_msg='', show=False,
     d = MessageBox(QMessageBox.Warning, 'WARNING: '+title, msg, QMessageBox.Ok,
                     parent, det_msg)
     d.setEscapeButton(QMessageBox.Ok)
-    d.setIconPixmap(QPixmap(I('dialog_warning.svg')))
+    d.setIconPixmap(QPixmap(I('dialog_warning.png')))
     if not show_copy_button:
         d.cb.setVisible(False)
     if show:
@@ -218,7 +244,7 @@ def error_dialog(parent, title, msg, det_msg='', show=False,
         show_copy_button=True):
     d = MessageBox(QMessageBox.Critical, 'ERROR: '+title, msg, QMessageBox.Ok,
                     parent, det_msg)
-    d.setIconPixmap(QPixmap(I('dialog_error.svg')))
+    d.setIconPixmap(QPixmap(I('dialog_error.png')))
     d.setEscapeButton(QMessageBox.Ok)
     if not show_copy_button:
         d.cb.setVisible(False)
@@ -226,10 +252,11 @@ def error_dialog(parent, title, msg, det_msg='', show=False,
         return d.exec_()
     return d
 
-def question_dialog(parent, title, msg, det_msg='', show_copy_button=True):
-    d = MessageBox(QMessageBox.Question, title, msg, QMessageBox.Yes|QMessageBox.No,
+def question_dialog(parent, title, msg, det_msg='', show_copy_button=True,
+        buttons=QMessageBox.Yes|QMessageBox.No):
+    d = MessageBox(QMessageBox.Question, title, msg, buttons,
                     parent, det_msg)
-    d.setIconPixmap(QPixmap(I('dialog_information.svg')))
+    d.setIconPixmap(QPixmap(I('dialog_question.png')))
     d.setEscapeButton(QMessageBox.No)
     if not show_copy_button:
         d.cb.setVisible(False)
@@ -239,7 +266,7 @@ def question_dialog(parent, title, msg, det_msg='', show_copy_button=True):
 def info_dialog(parent, title, msg, det_msg='', show=False):
     d = MessageBox(QMessageBox.Information, title, msg, QMessageBox.Ok,
                     parent, det_msg)
-    d.setIconPixmap(QPixmap(I('dialog_information.svg')))
+    d.setIconPixmap(QPixmap(I('dialog_information.png')))
     if show:
         return d.exec_()
     return d
@@ -247,14 +274,20 @@ def info_dialog(parent, title, msg, det_msg='', show=False):
 
 
 class Dispatcher(QObject):
-    '''Convenience class to ensure that a function call always happens in the
-    thread the receiver was created in.'''
+    '''
+    Convenience class to use Qt signals with arbitrary python callables.
+    By default, ensures that a function call always happens in the
+    thread this Dispatcher was created in.
+    '''
     dispatch_signal = pyqtSignal(object, object)
 
-    def __init__(self, func):
-        QObject.__init__(self)
+    def __init__(self, func, queued=True, parent=None):
+        QObject.__init__(self, parent)
         self.func = func
-        self.dispatch_signal.connect(self.dispatch, type=Qt.QueuedConnection)
+        typ = Qt.QueuedConnection
+        if not queued:
+            typ = Qt.AutoConnection if queued is None else Qt.DirectConnection
+        self.dispatch_signal.connect(self.dispatch, type=typ)
 
     def __call__(self, *args, **kwargs):
         self.dispatch_signal.emit(args, kwargs)
@@ -332,13 +365,14 @@ class FileIconProvider(QFileIconProvider):
              'epub'    : 'epub',
              'fb2'     : 'fb2',
              'rtf'     : 'rtf',
+             'odt'     : 'odt',
              }
 
     def __init__(self):
         QFileIconProvider.__init__(self)
         self.icons = {}
         for key in self.__class__.ICONS.keys():
-            self.icons[key] = I('mimetypes/')+self.__class__.ICONS[key]+'.svg'
+            self.icons[key] = I('mimetypes/')+self.__class__.ICONS[key]+'.png'
         for i in ('dir', 'default', 'zero'):
             self.icons[i] = QIcon(self.icons[i])
 
@@ -592,8 +626,11 @@ def open_url(qurl):
 
 
 def open_local_file(path):
-    url = QUrl.fromLocalFile(path)
-    open_url(url)
+    if iswindows:
+        os.startfile(os.path.normpath(path))
+    else:
+        url = QUrl.fromLocalFile(path)
+        open_url(url)
 
 def is_ok_to_use_qt():
     global gui_thread, _store_app
