@@ -16,9 +16,10 @@ class PluginWidget(QWidget,Ui_Form):
 
     TITLE = _('E-book options')
     HELP  = _('Options specific to')+' EPUB/MOBI '+_('output')
-    OPTION_FIELDS = [('exclude_genre','\[[\w ]*\]'),
+    OPTION_FIELDS = [('exclude_genre','\[.+\]'),
                      ('exclude_tags','~,'+_('Catalog')),
                      ('generate_titles', True),
+                     ('generate_series', True),
                      ('generate_recently_added', True),
                      ('note_tag','*'),
                      ('numbers_as_text', False),
@@ -40,7 +41,7 @@ class PluginWidget(QWidget,Ui_Form):
         # Update dialog fields from stored options
         for opt in self.OPTION_FIELDS:
             opt_value = gprefs.get(self.name + '_' + opt[0], opt[1])
-            if opt[0] in ['numbers_as_text','generate_titles','generate_recently_added']:
+            if opt[0] in ['numbers_as_text','generate_titles','generate_series','generate_recently_added']:
                 getattr(self, opt[0]).setChecked(opt_value)
             else:
                 getattr(self, opt[0]).setText(opt_value)
@@ -52,13 +53,13 @@ class PluginWidget(QWidget,Ui_Form):
         # others store as lists
         opts_dict = {}
         for opt in self.OPTION_FIELDS:
-            if opt[0] in ['numbers_as_text','generate_titles','generate_recently_added']:
+            if opt[0] in ['numbers_as_text','generate_titles','generate_series','generate_recently_added']:
                 opt_value = getattr(self,opt[0]).isChecked()
             else:
                 opt_value = unicode(getattr(self, opt[0]).text())
             gprefs.set(self.name + '_' + opt[0], opt_value)
 
-            if opt[0] in ['exclude_genre','numbers_as_text','generate_titles','generate_recently_added']:
+            if opt[0] in ['exclude_genre','numbers_as_text','generate_titles','generate_series','generate_recently_added']:
                 opts_dict[opt[0]] = opt_value
             else:
                 opts_dict[opt[0]] = opt_value.split(',')
