@@ -11,9 +11,11 @@ class PluginWidget(Widget, Ui_Form):
 
     TITLE = _('PDB Input')
     HELP = _('Options specific to')+' PDB '+_('input')
+    COMMIT_NAME = 'pdb_input'
+    ICON = I('mimetypes/unknown.png')
 
     def __init__(self, parent, get_option, get_help, db=None, book_id=None):
-        Widget.__init__(self, parent, 'pdb_input',
+        Widget.__init__(self, parent,
             ['single_line_paras', 'print_formatted_paras'])
         self.db, self.book_id = db, book_id
         self.initialize_options(get_option, get_help, db, book_id)
