@@ -9,9 +9,8 @@ from itertools import repeat
 from functools import partial
 from binascii import unhexlify
 
-from PyQt4.Qt import QMenu, QAction, QActionGroup, QIcon, SIGNAL, QPixmap, \
-                     Qt, pyqtSignal, QColor, QPainter, QDialog
-from PyQt4.QtSvg import QSvgRenderer
+from PyQt4.Qt import QMenu, QAction, QActionGroup, QIcon, SIGNAL, \
+                     Qt, pyqtSignal, QDialog, QMessageBox
 
 from calibre.customize.ui import available_input_formats, available_output_formats, \
     device_plugins
@@ -21,7 +20,7 @@ from calibre.gui2.dialogs.choose_format import ChooseFormatDialog
 from calibre.utils.ipc.job import BaseJob
 from calibre.devices.scanner import DeviceScanner
 from calibre.gui2 import config, error_dialog, Dispatcher, dynamic, \
-                        pixmap_to_data, warning_dialog, \
+                        warning_dialog, \
                         question_dialog, info_dialog, choose_dir
 from calibre.ebooks.metadata import authors_to_string
 from calibre import preferred_encoding, prints
@@ -33,7 +32,8 @@ from calibre.devices.apple.driver import ITUNES_ASYNC
 from calibre.devices.folder_device.driver import FOLDER_DEVICE
 from calibre.ebooks.metadata.meta import set_metadata
 from calibre.constants import DEBUG
-
+from calibre.utils.config import prefs, tweaks
+from calibre.utils.magick.draw import thumbnail
 # }}}
 
 class DeviceJob(BaseJob): # {{{
@@ -71,7 +71,14 @@ class DeviceJob(BaseJob): # {{{
             if self._aborted:
                 return
             self.failed = True
-            self._details = unicode(err) + '\n\n' + \
+            try:
+                ex = unicode(err)
+            except:
+                try:
+                    ex = str(err).decode(preferred_encoding, 'replace')
+                except:
+                    ex = repr(err)
+            self._details = ex + '\n\n' + \
                 traceback.format_exc()
             self.exception = err
         finally:
@@ -110,6 +117,7 @@ class DeviceManager(Thread): # {{{
         self.jobs           = Queue.Queue(0)
         self.keep_going     = True
         self.job_manager    = job_manager
+        self.reported_errors = set([])
         self.current_job    = None
         self.scanner        = DeviceScanner()
         self.connected_device = None
@@ -133,13 +141,16 @@ class DeviceManager(Thread): # {{{
         for dev, detected_device in connected_devices:
             if dev.OPEN_FEEDBACK_MESSAGE is not None:
                 self.open_feedback_slot(dev.OPEN_FEEDBACK_MESSAGE)
-            dev.reset(detected_device=detected_device,
-                    report_progress=self.report_progress)
             try:
+                dev.reset(detected_device=detected_device,
+                    report_progress=self.report_progress)
                 dev.open()
             except:
-                prints('Unable to open device', str(dev))
-                traceback.print_exc()
+                tb = traceback.format_exc()
+                if DEBUG or tb not in self.reported_errors:
+                    self.reported_errors.add(tb)
+                    prints('Unable to open device', str(dev))
+                    prints(tb)
                 continue
             self.connected_device = dev
             self.connected_device_kind = device_kind
@@ -184,11 +195,13 @@ class DeviceManager(Thread): # {{{
             if possibly_connected_devices:
                 if not self.do_connect(possibly_connected_devices,
                                        device_kind='device'):
-                    prints('Connect to device failed, retrying in 5 seconds...')
+                    if DEBUG:
+                        prints('Connect to device failed, retrying in 5 seconds...')
                     time.sleep(5)
                     if not self.do_connect(possibly_connected_devices,
                                        device_kind='usb'):
-                        prints('Device connect failed again, giving up')
+                        if DEBUG:
+                            prints('Device connect failed again, giving up')
 
     # Mount devices that don't use USB, such as the folder device and iTunes
     # This will be called on the GUI thread. Because of this, we must store
@@ -394,8 +407,6 @@ class DeviceAction(QAction): # {{{
 class DeviceMenu(QMenu): # {{{
 
     fetch_annotations = pyqtSignal()
-    connect_to_folder = pyqtSignal()
-    connect_to_itunes = pyqtSignal()
     disconnect_mounted_device = pyqtSignal()
 
     def __init__(self, parent=None):
@@ -405,64 +416,37 @@ class DeviceMenu(QMenu): # {{{
         self._memory = []
 
         self.set_default_menu = QMenu(_('Set default send to device action'))
-        self.set_default_menu.setIcon(QIcon(I('config.svg')))
+        self.set_default_menu.setIcon(QIcon(I('config.png')))
 
-        opts = email_config().parse()
-        default_account = None
-        if opts.accounts:
-            self.email_to_menu = self.addMenu(_('Email to')+'...')
-            keys = sorted(opts.accounts.keys())
-            for account in keys:
-                formats, auto, default = opts.accounts[account]
-                dest = 'mail:'+account+';'+formats
-                if default:
-                    default_account = (dest, False, False, I('mail.svg'),
-                            _('Email to')+' '+account)
-                action1 = DeviceAction(dest, False, False, I('mail.svg'),
-                        _('Email to')+' '+account)
-                action2 = DeviceAction(dest, True, False, I('mail.svg'),
-                        _('Email to')+' '+account+ _(' and delete from library'))
-                map(self.email_to_menu.addAction, (action1, action2))
-                map(self._memory.append, (action1, action2))
-                self.email_to_menu.addSeparator()
-                action1.a_s.connect(self.action_triggered)
-                action2.a_s.connect(self.action_triggered)
 
         basic_actions = [
-                ('main:', False, False,  I('reader.svg'),
+                ('main:', False, False,  I('reader.png'),
                     _('Send to main memory')),
-                ('carda:0', False, False, I('sd.svg'),
+                ('carda:0', False, False, I('sd.png'),
                     _('Send to storage card A')),
-                ('cardb:0', False, False, I('sd.svg'),
+                ('cardb:0', False, False, I('sd.png'),
                     _('Send to storage card B')),
         ]
 
         delete_actions = [
-                ('main:', True, False,   I('reader.svg'),
+                ('main:', True, False,   I('reader.png'),
                     _('Main Memory')),
-                ('carda:0', True, False,  I('sd.svg'),
+                ('carda:0', True, False,  I('sd.png'),
                     _('Storage Card A')),
-                ('cardb:0', True, False,  I('sd.svg'),
+                ('cardb:0', True, False,  I('sd.png'),
                     _('Storage Card B')),
         ]
 
         specific_actions = [
-                ('main:', False, True,  I('reader.svg'),
+                ('main:', False, True,  I('reader.png'),
                     _('Main Memory')),
-                ('carda:0', False, True, I('sd.svg'),
+                ('carda:0', False, True, I('sd.png'),
                     _('Storage Card A')),
-                ('cardb:0', False, True, I('sd.svg'),
+                ('cardb:0', False, True, I('sd.png'),
                     _('Storage Card B')),
         ]
 
 
-        if default_account is not None:
-            for x in (basic_actions, delete_actions):
-                ac = list(default_account)
-                if x is delete_actions:
-                    ac[1] = True
-                x.insert(1, tuple(ac))
-
         for menu in (self, self.set_default_menu):
             for actions, desc in (
                     (basic_actions, ''),
@@ -501,23 +485,9 @@ class DeviceMenu(QMenu): # {{{
             config['default_send_to_device_action'] = repr(action)
 
         self.group.triggered.connect(self.change_default_action)
-        if opts.accounts:
-            self.addSeparator()
-            self.addMenu(self.email_to_menu)
-
         self.addSeparator()
-        mitem = self.addAction(QIcon(I('document_open.svg')), _('Connect to folder'))
-        mitem.setEnabled(True)
-        mitem.triggered.connect(lambda x : self.connect_to_folder.emit())
-        self.connect_to_folder_action = mitem
 
-        mitem = self.addAction(QIcon(I('devices/itunes.png')),
-                _('Connect to iTunes'))
-        mitem.setEnabled(True)
-        mitem.triggered.connect(lambda x : self.connect_to_itunes.emit())
-        self.connect_to_itunes_action = mitem
-
-        mitem = self.addAction(QIcon(I('eject.svg')), _('Eject device'))
+        mitem = self.addAction(QIcon(I('eject.png')), _('Eject device'))
         mitem.setEnabled(False)
         mitem.triggered.connect(lambda x : self.disconnect_mounted_device.emit())
         self.disconnect_mounted_device_action = mitem
@@ -637,22 +607,22 @@ class DeviceMixin(object): # {{{
         self.device_error_dialog = error_dialog(self, _('Error'),
                 _('Error communicating with device'), ' ')
         self.device_error_dialog.setModal(Qt.NonModal)
-        self.device_connected = None
         self.emailer = Emailer()
         self.emailer.start()
         self.device_manager = DeviceManager(Dispatcher(self.device_detected),
                 self.job_manager, Dispatcher(self.status_bar.show_message))
         self.device_manager.start()
+        if tweaks['auto_connect_to_folder']:
+            self.connect_to_folder_named(tweaks['auto_connect_to_folder'])
 
     def set_default_thumbnail(self, height):
-        r = QSvgRenderer(I('book.svg'))
-        pixmap = QPixmap(height, height)
-        pixmap.fill(QColor(255,255,255))
-        p = QPainter(pixmap)
-        r.render(p)
-        p.end()
-        self.default_thumbnail = (pixmap.width(), pixmap.height(),
-                pixmap_to_data(pixmap))
+        img = I('book.png', data=True)
+        self.default_thumbnail = thumbnail(img, height, height)
+
+    def connect_to_folder_named(self, folder):
+        if os.path.exists(folder) and os.path.isdir(folder):
+            self.device_manager.mount_device(kls=FOLDER_DEVICE, kind='folder',
+                    path=folder)
 
     def connect_to_folder(self):
         dir = choose_dir(self, 'Select Device Folder',
@@ -675,21 +645,18 @@ class DeviceMixin(object): # {{{
 
     def create_device_menu(self):
         self._sync_menu = DeviceMenu(self)
-        self.action_sync.setMenu(self._sync_menu)
+        self.iactions['Send To Device'].qaction.setMenu(self._sync_menu)
+        self.iactions['Connect Share'].build_email_entries()
         self.connect(self._sync_menu,
                 SIGNAL('sync(PyQt_PyObject, PyQt_PyObject, PyQt_PyObject)'),
                 self.dispatch_sync_event)
-        self._sync_menu.fetch_annotations.connect(self.fetch_annotations)
-        self._sync_menu.connect_to_folder.connect(self.connect_to_folder)
-        self._sync_menu.connect_to_itunes.connect(self.connect_to_itunes)
+        self._sync_menu.fetch_annotations.connect(
+                self.iactions['Fetch Annotations'].fetch_annotations)
         self._sync_menu.disconnect_mounted_device.connect(self.disconnect_mounted_device)
+        self.iactions['Connect Share'].set_state(self.device_connected)
         if self.device_connected:
-            self._sync_menu.connect_to_folder_action.setEnabled(False)
-            self._sync_menu.connect_to_itunes_action.setEnabled(False)
             self._sync_menu.disconnect_mounted_device_action.setEnabled(True)
         else:
-            self._sync_menu.connect_to_folder_action.setEnabled(True)
-            self._sync_menu.connect_to_itunes_action.setEnabled(True)
             self._sync_menu.disconnect_mounted_device_action.setEnabled(False)
 
     def device_job_exception(self, job):
@@ -725,17 +692,14 @@ class DeviceMixin(object): # {{{
     # Device connected {{{
 
     def set_device_menu_items_state(self, connected):
+        self.iactions['Connect Share'].set_state(connected)
         if connected:
-            self._sync_menu.connect_to_folder_action.setEnabled(False)
-            self._sync_menu.connect_to_itunes_action.setEnabled(False)
             self._sync_menu.disconnect_mounted_device_action.setEnabled(True)
             self._sync_menu.enable_device_actions(True,
                     self.device_manager.device.card_prefix(),
                     self.device_manager.device)
             self.eject_action.setEnabled(True)
         else:
-            self._sync_menu.connect_to_folder_action.setEnabled(True)
-            self._sync_menu.connect_to_itunes_action.setEnabled(True)
             self._sync_menu.disconnect_mounted_device_action.setEnabled(False)
             self._sync_menu.enable_device_actions(False)
             self.eject_action.setEnabled(False)
@@ -754,18 +718,14 @@ class DeviceMixin(object): # {{{
                 self.device_manager.device.__class__.get_gui_name()+\
                         _(' detected.'), 3000)
             self.device_connected = device_kind
-            self.location_view.model().device_connected(self.device_manager.device)
             self.refresh_ondevice_info (device_connected = True, reset_only = True)
         else:
             self.device_connected = None
-            self.location_view.model().update_devices()
-            self.vanity.setText(self.vanity_template%\
-                    dict(version=self.latest_version, device=' '))
-            self.device_info = ' '
+            self.status_bar.device_disconnected()
             if self.current_view() != self.library_view:
                 self.book_details.reset_info()
-                self.location_view.setCurrentIndex(self.location_view.model().index(0))
-            self.refresh_ondevice_info (device_connected = False)
+            self.location_manager.update_devices()
+            self.refresh_ondevice_info(device_connected=False)
 
     def info_read(self, job):
         '''
@@ -774,11 +734,9 @@ class DeviceMixin(object): # {{{
         if job.failed:
             return self.device_job_exception(job)
         info, cp, fs = job.result
-        self.location_view.model().update_devices(cp, fs)
-        self.device_info = _('Connected ')+info[0]
-        self.vanity.setText(self.vanity_template%\
-                dict(version=self.latest_version, device=self.device_info))
-
+        self.location_manager.update_devices(cp, fs,
+                self.device_manager.device.icon)
+        self.status_bar.device_connected(info[0])
         self.device_manager.books(Dispatcher(self.metadata_downloaded))
 
     def metadata_downloaded(self, job):
@@ -802,8 +760,8 @@ class DeviceMixin(object): # {{{
 
     def refresh_ondevice_info(self, device_connected, reset_only = False):
         '''
-        Force the library view to refresh, taking into consideration
-        books information
+        Force the library view to refresh, taking into consideration new
+        device books information
         '''
         self.book_on_device(None, reset=True)
         if reset_only:
@@ -826,18 +784,21 @@ class DeviceMixin(object): # {{{
             self.device_job_exception(job)
             return
 
-        if self.delete_memory.has_key(job):
-            paths, model = self.delete_memory.pop(job)
+        dm = self.iactions['Remove Books'].delete_memory
+        if dm.has_key(job):
+            paths, model = dm.pop(job)
             self.device_manager.remove_books_from_metadata(paths,
                     self.booklists())
             model.paths_deleted(paths)
             self.upload_booklists()
-        # Clear the ondevice info so it will be recomputed
+        # Force recomputation the library's ondevice info. We need to call
+        # set_books_in_library even though books were not added because
+        # the deleted book might have been an exact match.
+        self.set_books_in_library(self.booklists(), reset=True)
         self.book_on_device(None, None, reset=True)
-        # We want to reset all the ondevice flags in the library. Use a big
-        # hammer, so we don't need to worry about whether some succeeded or not
-        self.library_view.model().refresh()
-
+        # We need to reset the ondevice flags in the library. Use a big hammer,
+        # so we don't need to worry about whether some succeeded or not.
+        self.refresh_ondevice_info(device_connected=True, reset_only=False)
 
     def dispatch_sync_event(self, dest, delete, specific):
         rows = self.library_view.selectionModel().selectedRows()
@@ -957,8 +918,9 @@ class DeviceMixin(object): # {{{
                 autos = '\n'.join('%s'%i for i in autos)
                 if question_dialog(self, _('No suitable formats'),
                     _('Auto convert the following books before sending via '
-                        'email?'), det_msg=autos):
-                    self.auto_convert_mail(to, fmts, delete_from_library, auto, format)
+                        'email?'), det_msg=autos,
+                    buttons=QMessageBox.Yes|QMessageBox.Cancel):
+                    self.iactions['Convert Books'].auto_convert_mail(to, fmts, delete_from_library, auto, format)
 
         if bad:
             bad = '\n'.join('%s'%(i,) for i in bad)
@@ -988,15 +950,16 @@ class DeviceMixin(object): # {{{
         else:
             self.status_bar.show_message(_('Sent by email:') + ', '.join(good),
                     5000)
+            if remove:
+                self.library_view.model().delete_books_by_id(remove)
 
     def cover_to_thumbnail(self, data):
-        p = QPixmap()
-        p.loadFromData(data)
-        if not p.isNull():
-            ht = self.device_manager.device.THUMBNAIL_HEIGHT \
-                    if self.device_manager else DevicePlugin.THUMBNAIL_HEIGHT
-            p = p.scaledToHeight(ht, Qt.SmoothTransformation)
-            return (p.width(), p.height(), pixmap_to_data(p))
+        ht = self.device_manager.device.THUMBNAIL_HEIGHT \
+                if self.device_manager else DevicePlugin.THUMBNAIL_HEIGHT
+        try:
+            return thumbnail(data, ht, ht)
+        except:
+            pass
 
     def email_news(self, id):
         opts = email_config().parse()
@@ -1056,8 +1019,9 @@ class DeviceMixin(object): # {{{
                     autos = '\n'.join('%s'%i for i in autos)
                     if question_dialog(self, _('No suitable formats'),
                         _('Auto convert the following books before uploading to '
-                            'the device?'), det_msg=autos):
-                        self.auto_convert_catalogs(auto, format)
+                            'the device?'), det_msg=autos,
+                        buttons=QMessageBox.Yes|QMessageBox.Cancel):
+                        self.iactions['Convert Books'].auto_convert_catalogs(auto, format)
             files = [f for f in files if f is not None]
             if not files:
                 dynamic.set('catalogs_to_be_synced', set([]))
@@ -1077,9 +1041,9 @@ class DeviceMixin(object): # {{{
             dynamic.set('catalogs_to_be_synced', set([]))
             if files:
                 remove = []
-                space = { self.location_view.model().free[0] : None,
-                    self.location_view.model().free[1] : 'carda',
-                    self.location_view.model().free[2] : 'cardb' }
+                space = { self.location_manager.free[0] : None,
+                    self.location_manager.free[1] : 'carda',
+                    self.location_manager.free[2] : 'cardb' }
                 on_card = space.get(sorted(space.keys(), reverse=True)[0], None)
                 self.upload_books(files, names, metadata,
                         on_card=on_card,
@@ -1117,8 +1081,9 @@ class DeviceMixin(object): # {{{
                     autos = '\n'.join('%s'%i for i in autos)
                     if question_dialog(self, _('No suitable formats'),
                         _('Auto convert the following books before uploading to '
-                            'the device?'), det_msg=autos):
-                        self.auto_convert_news(auto, format)
+                            'the device?'), det_msg=autos,
+                        buttons=QMessageBox.Yes|QMessageBox.Cancel):
+                        self.iactions['Convert Books'].auto_convert_news(auto, format)
             files = [f for f in files if f is not None]
             for f in files:
                 f.deleted_after_upload = del_on_upload
@@ -1140,9 +1105,9 @@ class DeviceMixin(object): # {{{
             dynamic.set('news_to_be_synced', set([]))
             if config['upload_news_to_device'] and files:
                 remove = ids if del_on_upload else []
-                space = { self.location_view.model().free[0] : None,
-                    self.location_view.model().free[1] : 'carda',
-                    self.location_view.model().free[2] : 'cardb' }
+                space = { self.location_manager.free[0] : None,
+                    self.location_manager.free[1] : 'carda',
+                    self.location_manager.free[2] : 'cardb' }
                 on_card = space.get(sorted(space.keys(), reverse=True)[0], None)
                 self.upload_books(files, names, metadata,
                         on_card=on_card,
@@ -1235,8 +1200,9 @@ class DeviceMixin(object): # {{{
                 autos = '\n'.join('%s'%i for i in autos)
                 if question_dialog(self, _('No suitable formats'),
                     _('Auto convert the following books before uploading to '
-                        'the device?'), det_msg=autos):
-                    self.auto_convert(auto, on_card, format)
+                        'the device?'), det_msg=autos,
+                    buttons=QMessageBox.Yes|QMessageBox.Cancel):
+                    self.iactions['Convert Books'].auto_convert(auto, on_card, format)
 
         if bad:
             bad = '\n'.join('%s'%(i,) for i in bad)
@@ -1262,7 +1228,8 @@ class DeviceMixin(object): # {{{
             self.device_job_exception(job)
             return
         cp, fs = job.result
-        self.location_view.model().update_devices(cp, fs)
+        self.location_manager.update_devices(cp, fs,
+                self.device_manager.device.icon)
         # reset the views so that up-to-date info is shown. These need to be
         # here because the sony driver updates collections in sync_booklists
         self.memory_view.reset()
@@ -1321,8 +1288,14 @@ class DeviceMixin(object): # {{{
             books_to_be_deleted = memory[1]
             self.library_view.model().delete_books_by_id(books_to_be_deleted)
 
-        self.set_books_in_library(self.booklists(),
-                reset=bool(books_to_be_deleted))
+        # There are some cases where sending a book to the device overwrites a
+        # book already there with a different book. This happens frequently in
+        # news. When this happens, the book match indication will be wrong
+        # because the UUID changed. Force both the device and the library view
+        # to refresh the flags.
+        self.set_books_in_library(self.booklists(), reset=True)
+        self.book_on_device(None, reset=True)
+        self.refresh_ondevice_info(device_connected = True)
 
         view = self.card_a_view if on_card == 'carda' else self.card_b_view if on_card == 'cardb' else self.memory_view
         view.model().resort(reset=False)
@@ -1330,144 +1303,174 @@ class DeviceMixin(object): # {{{
         for f in files:
             getattr(f, 'close', lambda : True)()
 
-        self.book_on_device(None, reset=True)
-        if metadata:
-            changed = set([])
-            for mi in metadata:
-                id_ = getattr(mi, 'application_id', None)
-                if id_ is not None:
-                    changed.add(id_)
-            if changed:
-                self.library_view.model().refresh_ids(list(changed))
-
     def book_on_device(self, id, format=None, reset=False):
-        loc = [None, None, None]
+        '''
+        Return an indication of whether the given book represented by its db id
+        is on the currently connected device. It returns a 5 element list. The
+        first three elements represent memory locations main, carda, and cardb,
+        and are true if the book is identifiably in that memory. The fourth
+        is a count of how many instances of the book were found across all
+        the memory locations. The fifth is a set of paths to the
+        matching books on the device.
+        '''
+        loc = [None, None, None, 0, set([])]
 
         if reset:
-            self.book_db_title_cache = None
-            self.book_db_uuid_cache = None
+            self.book_db_id_cache = None
+            self.book_db_id_counts = None
+            self.book_db_uuid_path_map = None
             return
 
-        if self.book_db_title_cache is None:
-            self.book_db_title_cache = []
-            self.book_db_uuid_cache = []
+        if not hasattr(self, 'db_book_uuid_cache'):
+            return loc
+
+        if self.book_db_id_cache is None:
+            self.book_db_id_cache = []
+            self.book_db_id_counts = {}
+            self.book_db_uuid_path_map = {}
             for i, l in enumerate(self.booklists()):
-                self.book_db_title_cache.append({})
-                self.book_db_uuid_cache.append(set())
+                self.book_db_id_cache.append(set())
                 for book in l:
-                    book_title = book.title.lower() if book.title else ''
-                    book_title = re.sub('(?u)\W|[_]', '', book_title)
-                    if book_title not in self.book_db_title_cache[i]:
-                        self.book_db_title_cache[i][book_title] = \
-                                {'authors':set(), 'db_ids':set(), 'uuids':set()}
-                    book_authors = authors_to_string(book.authors).lower()
-                    book_authors = re.sub('(?u)\W|[_]', '', book_authors)
-                    self.book_db_title_cache[i][book_title]['authors'].add(book_authors)
                     db_id = getattr(book, 'application_id', None)
                     if db_id is None:
                         db_id = book.db_id
                     if db_id is not None:
-                        self.book_db_title_cache[i][book_title]['db_ids'].add(db_id)
-                    uuid = getattr(book, 'uuid', None)
-                    if uuid is not None:
-                        self.book_db_uuid_cache[i].add(uuid)
+                        # increment the count of books on the device with this
+                        # db_id.
+                        self.book_db_id_cache[i].add(db_id)
+                        if db_id not in self.book_db_uuid_path_map:
+                            self.book_db_uuid_path_map[db_id] = set()
+                        if getattr(book, 'lpath', False):
+                            self.book_db_uuid_path_map[db_id].add(book.lpath)
+                        c = self.book_db_id_counts.get(db_id, 0)
+                        self.book_db_id_counts[db_id] = c + 1
 
-        mi = self.library_view.model().db.get_metadata(id, index_is_id=True)
         for i, l in enumerate(self.booklists()):
-            if mi.uuid in self.book_db_uuid_cache[i]:
+            if id in self.book_db_id_cache[i]:
                 loc[i] = True
-                continue
-            db_title = re.sub('(?u)\W|[_]', '', mi.title.lower())
-            cache = self.book_db_title_cache[i].get(db_title, None)
-            if cache:
-                if id in cache['db_ids']:
-                    loc[i] = True
-                    continue
-                if mi.authors and \
-                        re.sub('(?u)\W|[_]', '', authors_to_string(mi.authors).lower()) \
-                        in cache['authors']:
-                    loc[i] = True
-                    continue
-                # Also check author sort, because it can be used as author in
-                # some formats
-                if mi.author_sort and \
-                        re.sub('(?u)\W|[_]', '', mi.author_sort.lower()) \
-                        in cache['authors']:
-                    loc[i] = True
-                    continue
+                loc[3] = self.book_db_id_counts.get(id, 0)
+                loc[4] |= self.book_db_uuid_path_map[id]
         return loc
 
     def set_books_in_library(self, booklists, reset=False):
+        '''
+        Set the ondevice indications in the device database.
+        This method should be called before book_on_device is called, because
+        it sets the application_id for matched books. Book_on_device uses that
+        to both speed up matching and to count matches.
+        '''
+
+        string_pat = re.compile('(?u)\W|[_]')
+        def clean_string(x):
+            x = x.lower() if x else ''
+            return string_pat.sub('', x)
+
         # Force a reset if the caches are not initialized
         if reset or not hasattr(self, 'db_book_title_cache'):
             # It might be possible to get here without having initialized the
             # library view. In this case, simply give up
-            if not hasattr(self, 'library_view') or self.library_view is None:
-                return
-            db = getattr(self.library_view.model(), 'db', None)
-            if db is None:
+            try:
+                db = self.library_view.model().db
+            except:
                 return
             # Build a cache (map) of the library, so the search isn't On**2
             self.db_book_title_cache = {}
             self.db_book_uuid_cache = {}
             for id in db.data.iterallids():
                 mi = db.get_metadata(id, index_is_id=True)
-                title = re.sub('(?u)\W|[_]', '', mi.title.lower())
+                title = clean_string(mi.title)
                 if title not in self.db_book_title_cache:
                     self.db_book_title_cache[title] = \
                                 {'authors':{}, 'author_sort':{}, 'db_ids':{}}
+                # If there are multiple books in the library with the same title
+                # and author, then remember the last one. That is OK, because as
+                # we can't tell the difference between the books, one is as good
+                # as another.
                 if mi.authors:
-                    authors = authors_to_string(mi.authors).lower()
-                    authors = re.sub('(?u)\W|[_]', '', authors)
+                    authors = clean_string(authors_to_string(mi.authors))
                     self.db_book_title_cache[title]['authors'][authors] = mi
                 if mi.author_sort:
-                    aus = mi.author_sort.lower()
-                    aus = re.sub('(?u)\W|[_]', '', aus)
+                    aus = clean_string(mi.author_sort)
                     self.db_book_title_cache[title]['author_sort'][aus] = mi
                 self.db_book_title_cache[title]['db_ids'][mi.application_id] = mi
-                self.db_book_uuid_cache[mi.uuid] = mi.application_id
+                self.db_book_uuid_cache[mi.uuid] = mi
 
         # Now iterate through all the books on the device, setting the
-        # in_library field Fastest and most accurate key is the uuid. Second is
-        # the application_id, which is really the db key, but as this can
-        # accidentally match across libraries we also verify the title. The
-        # db_id exists on Sony devices. Fallback is title and author match
+        # in_library field. If the UUID matches a book in the library, then
+        # do not consider that book for other matching. In all cases set
+        # the application_id to the db_id of the matching book. This value
+        # will be used by books_on_device to indicate matches.
+
+        update_metadata = prefs['manage_device_metadata'] == 'on_connect'
         for booklist in booklists:
             for book in booklist:
+                book.in_library = None
                 if getattr(book, 'uuid', None) in self.db_book_uuid_cache:
+                    if update_metadata:
+                        book.smart_update(self.db_book_uuid_cache[book.uuid],
+                                          replace_metadata=True)
                     book.in_library = True
                     # ensure that the correct application_id is set
-                    book.application_id = self.db_book_uuid_cache[book.uuid]
+                    book.application_id = \
+                        self.db_book_uuid_cache[book.uuid].application_id
                     continue
-
-                book_title = book.title.lower() if book.title else ''
-                book_title = re.sub('(?u)\W|[_]', '', book_title)
-                book.in_library = None
+                # No UUID exact match. Try metadata matching.
+                book_title = clean_string(book.title)
                 d = self.db_book_title_cache.get(book_title, None)
                 if d is not None:
+                    # At this point we know that the title matches. The book
+                    # will match if any of the db_id, author, or author_sort
+                    # also match.
                     if getattr(book, 'application_id', None) in d['db_ids']:
                         book.in_library = True
-                        book.smart_update(d['db_ids'][book.application_id])
+                        # app_id already matches a db_id. No need to set it.
+                        if update_metadata:
+                            book.smart_update(d['db_ids'][book.application_id],
+                                              replace_metadata=True)
                         continue
-                    if book.db_id in d['db_ids']:
+                    # Sonys know their db_id independent of the application_id
+                    # in the metadata cache. Check that as well.
+                    if getattr(book, 'db_id', None) in d['db_ids']:
                         book.in_library = True
-                        book.smart_update(d['db_ids'][book.db_id])
+                        book.application_id = \
+                                    d['db_ids'][book.db_id].application_id
+                        if update_metadata:
+                            book.smart_update(d['db_ids'][book.db_id],
+                                              replace_metadata=True)
                         continue
+                    # We now know that the application_id is not right. Set it
+                    # to None to prevent book_on_device from accidentally
+                    # matching on it. It will be set to a correct value below if
+                    # the book is matched with one in the library
+                    book.application_id = None
                     if book.authors:
                         # Compare against both author and author sort, because
                         # either can appear as the author
-                        book_authors = authors_to_string(book.authors).lower()
-                        book_authors = re.sub('(?u)\W|[_]', '', book_authors)
+                        book_authors = clean_string(authors_to_string(book.authors))
                         if book_authors in d['authors']:
                             book.in_library = True
-                            book.smart_update(d['authors'][book_authors])
+                            book.application_id = \
+                                    d['authors'][book_authors].application_id
+                            if update_metadata:
+                                book.smart_update(d['authors'][book_authors],
+                                                  replace_metadata=True)
                         elif book_authors in d['author_sort']:
                             book.in_library = True
-                            book.smart_update(d['author_sort'][book_authors])
+                            book.application_id = \
+                                d['author_sort'][book_authors].application_id
+                            if update_metadata:
+                                book.smart_update(d['author_sort'][book_authors],
+                                                  replace_metadata=True)
+                else:
+                    # Book definitely not matched. Clear its application ID
+                    book.application_id = None
                 # Set author_sort if it isn't already
                 asort = getattr(book, 'author_sort', None)
                 if not asort and book.authors:
                     book.author_sort = self.library_view.model().db.author_sort_from_authors(book.authors)
 
+        if update_metadata:
+            if self.device_manager.is_device_connected:
+                self.device_manager.sync_booklists(None, booklists)
     # }}}
 
