@@ -12,7 +12,7 @@ import time
 import traceback
 
 from PyQt4.Qt import SIGNAL, QObject, Qt, QTimer, QThread, QDate, \
-    QPixmap, QListWidgetItem, QDialog
+    QPixmap, QListWidgetItem, QDialog, pyqtSignal
 
 from calibre.gui2 import error_dialog, file_icon_provider, dynamic, \
                            choose_files, choose_images, ResizableDialog, \
@@ -24,13 +24,15 @@ from calibre.gui2.widgets import ProgressIndicator
 from calibre.ebooks import BOOK_EXTENSIONS
 from calibre.ebooks.metadata import string_to_authors, \
         authors_to_string, check_isbn
-from calibre.ebooks.metadata.library_thing import cover_from_isbn
+from calibre.ebooks.metadata.covers import download_cover
 from calibre.ebooks.metadata.meta import get_metadata
+from calibre.ebooks.metadata import MetaInformation
 from calibre.utils.config import prefs, tweaks
-from calibre.utils.date import qt_to_dt
+from calibre.utils.date import qt_to_dt, local_tz, utcfromtimestamp
 from calibre.customize.ui import run_plugins_on_import, get_isbndb_key
-from calibre.gui2.dialogs.config.social import SocialMetadata
+from calibre.gui2.preferences.social import SocialMetadata
 from calibre.gui2.custom_column_widgets import populate_metadata_page
+from calibre import strftime
 
 class CoverFetcher(QThread):
 
@@ -47,12 +49,13 @@ class CoverFetcher(QThread):
 
     def run(self):
         try:
+            au = self.author if self.author else None
+            mi = MetaInformation(self.title, [au])
             if not self.isbn:
                 from calibre.ebooks.metadata.fetch import search
                 if not self.title:
                     self.needs_isbn = True
                     return
-                au = self.author if self.author else None
                 key = get_isbndb_key()
                 if not key:
                     key = None
@@ -65,8 +68,10 @@ class CoverFetcher(QThread):
                     return
                 self.isbn = results[0]
 
-            self.cover_data = cover_from_isbn(self.isbn, timeout=self.timeout,
-                    username=self.username, password=self.password)[0]
+            mi.isbn = self.isbn
+
+            self.cover_data, self.errors = download_cover(mi,
+                    timeout=self.timeout)
         except Exception, e:
             self.exception = e
             self.traceback = traceback.format_exc()
@@ -75,21 +80,29 @@ class CoverFetcher(QThread):
 
 
 class Format(QListWidgetItem):
-    def __init__(self, parent, ext, size, path=None):
+
+    def __init__(self, parent, ext, size, path=None, timestamp=None):
         self.path = path
         self.ext = ext
         self.size = float(size)/(1024*1024)
         text = '%s (%.2f MB)'%(self.ext.upper(), self.size)
         QListWidgetItem.__init__(self, file_icon_provider().icon_from_ext(ext),
                                  text, parent, QListWidgetItem.UserType)
+        if timestamp is not None:
+            ts = timestamp.astimezone(local_tz)
+            t = strftime('%a, %d %b %Y [%H:%M:%S]', ts.timetuple())
+            text = _('Last modified: %s')%t
+            self.setToolTip(text)
+            self.setStatusTip(text)
 
 
 class MetadataSingleDialog(ResizableDialog, Ui_MetadataSingleDialog):
 
     COVER_FETCH_TIMEOUT = 240 # seconds
+    view_format = pyqtSignal(object)
 
     def do_reset_cover(self, *args):
-        pix = QPixmap(I('default_cover.svg'))
+        pix = QPixmap(I('default_cover.png'))
         self.cover.setPixmap(pix)
         self.cover_changed = True
         self.cover_data = None
@@ -130,6 +143,29 @@ class MetadataSingleDialog(ResizableDialog, Ui_MetadataSingleDialog):
                     self.cpixmap = pix
                     self.cover_data = cover
 
+    def generate_cover(self, *args):
+        from calibre.ebooks import calibre_cover
+        from calibre.ebooks.metadata import fmt_sidx
+        from calibre.gui2 import config
+        title = unicode(self.title.text()).strip()
+        author = unicode(self.authors.text()).strip()
+        if not title or not author:
+            return error_dialog(self, _('Specify title and author'),
+                    _('You must specify a title and author before generating '
+                        'a cover'), show=True)
+        series = unicode(self.series.text()).strip()
+        series_string = None
+        if series:
+            series_string = _('Book %s of %s')%(
+                    fmt_sidx(self.series_index.value(),
+                        use_roman=config['use_roman_numerals_for_series_number']), series)
+        self.cover_data = calibre_cover(title, author,
+                series_string=series_string)
+        pix = QPixmap()
+        pix.loadFromData(self.cover_data)
+        self.cover.setPixmap(pix)
+        self.cover_changed = True
+        self.cpixmap = pix
 
     def add_format(self, x):
         files = choose_files(self, 'add formats dialog',
@@ -151,14 +187,16 @@ class MetadataSingleDialog(ResizableDialog, Ui_MetadataSingleDialog):
             nfile = run_plugins_on_import(_file)
             if nfile is not None:
                 _file = nfile
-            size = os.stat(_file).st_size
+            stat = os.stat(_file)
+            size = stat.st_size
             ext = os.path.splitext(_file)[1].lower().replace('.', '')
+            timestamp = utcfromtimestamp(stat.st_mtime)
             for row in range(self.formats.count()):
                 fmt = self.formats.item(row)
                 if fmt.ext.lower() == ext:
                     self.formats.takeItem(row)
                     break
-            Format(self.formats, ext, size, path=_file)
+            Format(self.formats, ext, size, path=_file, timestamp=timestamp)
             self.formats_changed = True
             added = True
         if bad_perms:
@@ -379,9 +417,10 @@ class MetadataSingleDialog(ResizableDialog, Ui_MetadataSingleDialog):
                 if not ext:
                     ext = ''
                 size = self.db.sizeof_format(row, ext)
+                timestamp = self.db.format_last_modified(self.id, ext)
                 if size is None:
                     continue
-                Format(self.formats, ext, size)
+                Format(self.formats, ext, size, timestamp=timestamp)
 
 
         self.initialize_combos()
@@ -401,7 +440,7 @@ class MetadataSingleDialog(ResizableDialog, Ui_MetadataSingleDialog):
         if cover:
             pm.loadFromData(cover)
         if pm.isNull():
-            pm = QPixmap(I('default_cover.svg'))
+            pm = QPixmap(I('default_cover.png'))
         else:
             self.cover_data = cover
         self.cover.setPixmap(pm)
@@ -410,6 +449,7 @@ class MetadataSingleDialog(ResizableDialog, Ui_MetadataSingleDialog):
             self.central_widget.tabBar().setVisible(False)
         else:
             self.create_custom_column_editors()
+        self.generate_cover_button.clicked.connect(self.generate_cover)
 
     def create_custom_column_editors(self):
         w = self.central_widget.widget(1)
@@ -443,7 +483,7 @@ class MetadataSingleDialog(ResizableDialog, Ui_MetadataSingleDialog):
 
     def show_format(self, item, *args):
         fmt = item.ext
-        self.emit(SIGNAL('view_format(PyQt_PyObject)'), fmt)
+        self.view_format.emit(fmt)
 
     def deduce_author_sort(self):
         au = unicode(self.authors.text())
@@ -565,6 +605,13 @@ class MetadataSingleDialog(ResizableDialog, Ui_MetadataSingleDialog):
                 error_dialog(self, _('Cannot fetch cover'),
                     _('<b>Could not fetch cover.</b><br/>')+unicode(err)).exec_()
                 return
+            if self.cover_fetcher.errors and self.cover_fetcher.cover_data is None:
+                details = u'\n\n'.join([e[-1] + ': ' + e[1] for e in self.cover_fetcher.errors])
+                error_dialog(self, _('Cannot fetch cover'),
+                    _('<b>Could not fetch cover.</b><br/>') +
+                    _('For the error message from each cover source, '
+                      'click Show details below.'), det_msg=details, show=True)
+                return
 
             pix = QPixmap()
             pix.loadFromData(self.cover_fetcher.cover_data)
@@ -629,9 +676,9 @@ class MetadataSingleDialog(ResizableDialog, Ui_MetadataSingleDialog):
                             self.tags.setText(', '.join(book.tags))
                         if book.series is not None:
                             if self.series.text() is None or self.series.text() == '':
-                               self.series.setText(book.series)
-                               if book.series_index is not None:
-                                  self.series_index.setValue(book.series_index)
+                                self.series.setText(book.series)
+                                if book.series_index is not None:
+                                    self.series_index.setValue(book.series_index)
         else:
             error_dialog(self, _('Cannot fetch metadata'),
                          _('You must specify at least one of ISBN, Title, '
@@ -666,6 +713,10 @@ class MetadataSingleDialog(ResizableDialog, Ui_MetadataSingleDialog):
 
 
     def accept(self):
+        cf = getattr(self, 'cover_fetcher', None)
+        if cf is not None and hasattr(cf, 'terminate'):
+            cf.terminate()
+            cf.wait()
         try:
             if self.formats_changed:
                 self.sync_formats()
@@ -676,24 +727,31 @@ class MetadataSingleDialog(ResizableDialog, Ui_MetadataSingleDialog):
                 self.db.set_authors(self.id, string_to_authors(au), notify=False)
             aus = unicode(self.author_sort.text())
             if aus:
-                self.db.set_author_sort(self.id, aus, notify=False)
+                self.db.set_author_sort(self.id, aus, notify=False, commit=False)
             self.db.set_isbn(self.id,
-                    re.sub(r'[^0-9a-zA-Z]', '', unicode(self.isbn.text())), notify=False)
-            self.db.set_rating(self.id, 2*self.rating.value(), notify=False)
-            self.db.set_publisher(self.id, unicode(self.publisher.currentText()), notify=False)
+                             re.sub(r'[^0-9a-zA-Z]', '', unicode(self.isbn.text())),
+                             notify=False, commit=False)
+            self.db.set_rating(self.id, 2*self.rating.value(), notify=False,
+                               commit=False)
+            self.db.set_publisher(self.id, unicode(self.publisher.currentText()),
+                                  notify=False, commit=False)
             self.db.set_tags(self.id, [x.strip() for x in
-                unicode(self.tags.text()).split(',')], notify=False)
+                unicode(self.tags.text()).split(',')], notify=False, commit=False)
             self.db.set_series(self.id,
-                    unicode(self.series.currentText()).strip(), notify=False)
-            self.db.set_series_index(self.id, self.series_index.value(), notify=False)
-            self.db.set_comment(self.id, unicode(self.comments.toPlainText()), notify=False)
+                    unicode(self.series.currentText()).strip(), notify=False,
+                    commit=False)
+            self.db.set_series_index(self.id, self.series_index.value(),
+                                     notify=False, commit=False)
+            self.db.set_comment(self.id, unicode(self.comments.toPlainText()),
+                                notify=False, commit=False)
             d = self.pubdate.date()
             d = qt_to_dt(d)
-            self.db.set_pubdate(self.id, d, notify=False)
+            self.db.set_pubdate(self.id, d, notify=False, commit=False)
             d = self.date.date()
             d = qt_to_dt(d)
             if d.date() != self.orig_timestamp.date():
-                self.db.set_timestamp(self.id, d, notify=False)
+                self.db.set_timestamp(self.id, d, notify=False, commit=False)
+            self.db.commit()
 
             if self.cover_changed:
                 if self.cover_data is not None:
@@ -702,6 +760,7 @@ class MetadataSingleDialog(ResizableDialog, Ui_MetadataSingleDialog):
                     self.db.remove_cover(self.id)
             for w in getattr(self, 'custom_column_widgets', []):
                 w.commit(self.id)
+            self.db.commit()
         except IOError, err:
             if err.errno == 13: # Permission denied
                 fname = err.filename if err.filename else 'file'
@@ -726,9 +785,9 @@ class MetadataSingleDialog(ResizableDialog, Ui_MetadataSingleDialog):
         wg = dynamic.get('metasingle_window_geometry', None)
         ss = dynamic.get('metasingle_splitter_state', None)
         if wg is not None:
-          self.restoreGeometry(wg)
+            self.restoreGeometry(wg)
         if ss is not None:
-          self.splitter.restoreState(ss)
+            self.splitter.restoreState(ss)
 
     def save_state(self):
         dynamic.set('metasingle_window_geometry', bytes(self.saveGeometry()))
