@@ -49,7 +49,7 @@ _default_image = None
 def default_image():
     global _default_image
     if _default_image is None:
-        _default_image = QImage(I('default_cover.svg'))
+        _default_image = QImage(I('default_cover.png'))
     return _default_image
 
 class BooksModel(QAbstractTableModel): # {{{
@@ -89,9 +89,9 @@ class BooksModel(QAbstractTableModel): # {{{
         self.alignment_map = {}
         self.buffer_size = buffer
         self.cover_cache = None
-        self.bool_yes_icon = QIcon(I('ok.svg'))
-        self.bool_no_icon = QIcon(I('list_remove.svg'))
-        self.bool_blank_icon = QIcon(I('blank.svg'))
+        self.bool_yes_icon = QIcon(I('ok.png'))
+        self.bool_no_icon = QIcon(I('list_remove.png'))
+        self.bool_blank_icon = QIcon(I('blank.png'))
         self.device_connected = False
         self.read_config()
 
@@ -121,10 +121,11 @@ class BooksModel(QAbstractTableModel): # {{{
     def set_device_connected(self, is_connected):
         self.device_connected = is_connected
         self.db.refresh_ondevice()
+        self.refresh()
+        self.research()
         if is_connected and self.sorted_on[0] == 'ondevice':
             self.resort()
 
-
     def set_book_on_device_func(self, func):
         self.book_on_device = func
 
@@ -236,8 +237,8 @@ class BooksModel(QAbstractTableModel): # {{{
     def search(self, text, reset=True):
         try:
             self.db.search(text)
-        except ParseException:
-            self.searched.emit(False)
+        except ParseException as e:
+            self.searched.emit(e.msg)
             return
         self.last_search = text
         if reset:
@@ -492,6 +493,11 @@ class BooksModel(QAbstractTableModel): # {{{
     def title(self, row_number):
         return self.db.title(row_number)
 
+    def rating(self, row_number):
+        ans = self.db.rating(row_number)
+        ans = ans/2 if ans else 0
+        return int(ans)
+
     def cover(self, row_number):
         data = None
         try:
@@ -937,8 +943,9 @@ class DeviceBooksModel(BooksModel): # {{{
             cname = self.column_map[index.column()]
             if cname in ('title', 'authors') or \
                     (cname == 'collections' and \
+                     callable(getattr(self.db, 'supports_collections', None)) and \
                      self.db.supports_collections() and \
-                     prefs['preserve_user_collections']):
+                     prefs['manage_device_metadata']=='manual'):
                 flags |= Qt.ItemIsEditable
         return flags
 
@@ -1210,7 +1217,9 @@ class DeviceBooksModel(BooksModel): # {{{
         return done
 
     def set_editable(self, editable):
-        self.editable = editable
+        # Cannot edit if metadata is sent on connect. Reason: changes will
+        # revert to what is in the library on next connect.
+        self.editable = editable and prefs['manage_device_metadata']!='on_connect'
 
     def set_search_restriction(self, s):
         pass
