@@ -9,7 +9,7 @@ import os
 from functools import partial
 
 from PyQt4.Qt import QTableView, Qt, QAbstractItemView, QMenu, pyqtSignal, \
-    QModelIndex
+    QModelIndex, QIcon
 
 from calibre.gui2.library.delegates import RatingDelegate, PubDateDelegate, \
     TextDelegate, DateDelegate, TagsDelegate, CcTextDelegate, \
@@ -23,6 +23,7 @@ from calibre.gui2.library import DEFAULT_SORT
 class BooksView(QTableView): # {{{
 
     files_dropped = pyqtSignal(object)
+    add_column_signal = pyqtSignal()
 
     def __init__(self, parent, modelcls=BooksModel):
         QTableView.__init__(self, parent)
@@ -54,6 +55,7 @@ class BooksView(QTableView): # {{{
         self.selectionModel().currentRowChanged.connect(self._model.current_changed)
 
         # {{{ Column Header setup
+        self.can_add_columns = True
         self.was_restored = False
         self.column_header = self.horizontalHeader()
         self.column_header.setMovable(True)
@@ -93,6 +95,8 @@ class BooksView(QTableView): # {{{
             self.sortByColumn(idx, Qt.DescendingOrder)
         elif action == 'defaults':
             self.apply_state(self.get_default_state())
+        elif action == 'addcustcol':
+            self.add_column_signal.emit()
         elif action.startswith('align_'):
             alignment = action.partition('_')[-1]
             self._model.change_alignment(column, alignment)
@@ -166,6 +170,13 @@ class BooksView(QTableView): # {{{
                     partial(self.column_header_context_handler,
                         action='defaults', column=col))
 
+            if self.can_add_columns:
+                self.column_header_context_menu.addAction(
+                        QIcon(I('column.png')),
+                        _('Add your own columns'),
+                        partial(self.column_header_context_handler,
+                            action='addcustcol', column=col))
+
             self.column_header_context_menu.popup(self.column_header.mapToGlobal(pos))
     # }}}
 
@@ -214,17 +225,23 @@ class BooksView(QTableView): # {{{
                 state['column_sizes'][name] = h.sectionSize(i)
         return state
 
+    def write_state(self, state):
+        db = getattr(self.model(), 'db', None)
+        name = unicode(self.objectName())
+        if name and db is not None:
+            db.prefs.set(name + ' books view state', state)
+
     def save_state(self):
         # Only save if we have been initialized (set_database called)
         if len(self.column_map) > 0 and self.was_restored:
             state = self.get_state()
-            name = unicode(self.objectName())
-            if name:
-                gprefs.set(name + ' books view state', state)
+            self.write_state(state)
 
     def cleanup_sort_history(self, sort_history):
         history = []
         for col, order in sort_history:
+            if col == 'date':
+                col = 'timestamp'
             if col in self.column_map and (not history or history[0][0] != col):
                 history.append([col, order])
         return history
@@ -298,11 +315,27 @@ class BooksView(QTableView): # {{{
                     old_state['column_sizes'][name] += 12
         return old_state
 
-    def restore_state(self):
+    def get_old_state(self):
+        ans = None
         name = unicode(self.objectName())
-        old_state = None
         if name:
-            old_state = gprefs.get(name + ' books view state', None)
+            name += ' books view state'
+            db = getattr(self.model(), 'db', None)
+            if db is not None:
+                ans = db.prefs.get(name, None)
+                if ans is None:
+                    ans = gprefs.get(name, None)
+                    try:
+                        del gprefs[name]
+                    except:
+                        pass
+                    if ans is not None:
+                        db.prefs[name] = ans
+        return ans
+
+
+    def restore_state(self):
+        old_state = self.get_old_state()
         if old_state is None:
             old_state = self.get_default_state()
 
@@ -369,35 +402,10 @@ class BooksView(QTableView): # {{{
         #}}}
 
     # Context Menu {{{
-    def set_context_menu(self, edit_metadata, send_to_device, convert, view,
-                         save, open_folder, book_details, delete,
-                         similar_menu=None, add_to_library=None,
-                         edit_device_collections=None):
+    def set_context_menu(self, menu, edit_collections_action):
         self.setContextMenuPolicy(Qt.DefaultContextMenu)
-        self.context_menu = QMenu(self)
-        if edit_metadata is not None:
-            self.context_menu.addAction(edit_metadata)
-        if send_to_device is not None:
-            self.context_menu.addAction(send_to_device)
-        if convert is not None:
-            self.context_menu.addAction(convert)
-        self.context_menu.addAction(view)
-        self.context_menu.addAction(save)
-        if open_folder is not None:
-            self.context_menu.addAction(open_folder)
-        if delete is not None:
-            self.context_menu.addAction(delete)
-        if book_details is not None:
-            self.context_menu.addAction(book_details)
-        if similar_menu is not None:
-            self.context_menu.addMenu(similar_menu)
-        if add_to_library is not None:
-            func = partial(add_to_library[1], view=self)
-            self.context_menu.addAction(add_to_library[0], func)
-        if edit_device_collections is not None:
-            func = partial(edit_device_collections[1], view=self)
-            self.edit_collections_menu = \
-                self.context_menu.addAction(edit_device_collections[0], func)
+        self.context_menu = menu
+        self.edit_collections_action = edit_collections_action
 
     def contextMenuEvent(self, event):
         self.context_menu.popup(event.globalPos())
@@ -456,14 +464,20 @@ class BooksView(QTableView): # {{{
     def set_current_row(self, row, select=True):
         if row > -1:
             h = self.horizontalHeader()
-            for i in range(h.count()):
-                if not h.isSectionHidden(i):
-                    index = self.model().index(row, i)
-                    self.setCurrentIndex(index)
-                    if select:
-                        sm = self.selectionModel()
-                        sm.select(index, sm.ClearAndSelect|sm.Rows)
-                    break
+            logical_indices = list(range(h.count()))
+            logical_indices = [x for x in logical_indices if not
+                    h.isSectionHidden(x)]
+            pairs = [(x, h.visualIndex(x)) for x in logical_indices if
+                    h.visualIndex(x) > -1]
+            if not pairs:
+                pairs = [(0, 0)]
+            pairs.sort(cmp=lambda x,y:cmp(x[1], y[1]))
+            i = pairs[0][0]
+            index = self.model().index(row, i)
+            self.setCurrentIndex(index)
+            if select:
+                sm = self.selectionModel()
+                sm.select(index, sm.ClearAndSelect|sm.Rows)
 
     def close(self):
         self._model.close()
@@ -491,6 +505,7 @@ class DeviceBooksView(BooksView): # {{{
 
     def __init__(self, parent):
         BooksView.__init__(self, parent, DeviceBooksModel)
+        self.can_add_columns = False
         self.columns_resized = False
         self.resize_on_select = False
         self.rating_delegate = None
@@ -500,12 +515,27 @@ class DeviceBooksView(BooksView): # {{{
         self.setAcceptDrops(False)
 
     def contextMenuEvent(self, event):
-        self.edit_collections_menu.setVisible(
-                            self._model.db.supports_collections() and \
-                            prefs['preserve_user_collections'])
+        edit_collections = callable(getattr(self._model.db, 'supports_collections', None)) and \
+            self._model.db.supports_collections() and \
+            prefs['manage_device_metadata'] == 'manual'
+
+        self.edit_collections_action.setVisible(edit_collections)
         self.context_menu.popup(event.globalPos())
         event.accept()
 
+    def get_old_state(self):
+        ans = None
+        name = unicode(self.objectName())
+        if name:
+            name += ' books view state'
+            ans = gprefs.get(name, None)
+        return ans
+
+    def write_state(self, state):
+        name = unicode(self.objectName())
+        if name:
+            gprefs.set(name + ' books view state', state)
+
     def set_database(self, db):
         self._model.set_database(db)
         self.restore_state()
