@@ -155,7 +155,7 @@ class GuiRunner(QObject):
         main.initialize(self.library_path, self.db, self.listener, self.actions)
         if DEBUG:
             prints('Started up in', time.time() - self.startup_time)
-        add_filesystem_book = partial(main.add_filesystem_book, allow_device=False)
+        add_filesystem_book = partial(main.iactions['Add Books'].add_filesystem_book, allow_device=False)
         sys.excepthook = main.unhandled_exception
         if len(self.args) > 1:
             p = os.path.abspath(self.args[1])
@@ -241,7 +241,7 @@ class GuiRunner(QObject):
         QApplication.instance().processEvents()
 
     def initialize(self, *args):
-        if gprefs.get('show_splash_screen', True):
+        if gprefs['show_splash_screen']:
             self.show_splash_screen()
 
         self.library_path = get_library_path(parent=self.splash_screen)
