@@ -17,7 +17,6 @@ from PyQt4.Qt import Qt, QTreeView, QApplication, pyqtSignal, \
 
 from calibre.ebooks.metadata import title_sort
 from calibre.gui2 import config, NONE
-from calibre.utils.config import prefs
 from calibre.library.field_metadata import TagsIcons
 from calibre.utils.search_query_parser import saved_searches
 from calibre.gui2 import error_dialog
@@ -79,6 +78,7 @@ class TagsView(QTreeView): # {{{
         self.setAnimated(True)
         self.setHeaderHidden(True)
         self.setItemDelegate(TagDelegate(self))
+        self.made_connections = False
 
     def set_database(self, db, tag_match, sort_by):
         self.hidden_categories = config['tag_browser_hidden_categories']
@@ -91,12 +91,14 @@ class TagsView(QTreeView): # {{{
         self.search_restriction = None
         self.setModel(self._model)
         self.setContextMenuPolicy(Qt.CustomContextMenu)
-        self.clicked.connect(self.toggle)
-        self.customContextMenuRequested.connect(self.show_context_menu)
         pop = config['sort_tags_by']
         self.sort_by.setCurrentIndex(self.db.CATEGORY_SORTS.index(pop))
-        self.sort_by.currentIndexChanged.connect(self.sort_changed)
-        self.refresh_required.connect(self.recount, type=Qt.QueuedConnection)
+        if not self.made_connections:
+            self.clicked.connect(self.toggle)
+            self.customContextMenuRequested.connect(self.show_context_menu)
+            self.refresh_required.connect(self.recount, type=Qt.QueuedConnection)
+            self.sort_by.currentIndexChanged.connect(self.sort_changed)
+            self.made_connections = True
         db.add_listener(self.database_changed)
 
     def database_changed(self, event, ids):
@@ -224,7 +226,7 @@ class TagsView(QTreeView): # {{{
 
             # Always show the user categories editor
             self.context_menu.addSeparator()
-            if category in prefs['user_categories'].keys():
+            if category in self.db.prefs.get('user_categories', {}).keys():
                 self.context_menu.addAction(_('Manage User Categories'),
                         partial(self.context_menu_handler, action='manage_categories',
                                 category=category))
@@ -370,19 +372,19 @@ class TagsModel(QAbstractItemModel): # {{{
         # before a QPaintDevice'. The ':' in front avoids polluting either the
         # user-defined categories (':' at end) or columns namespaces (no ':').
         self.category_icon_map = TagsIcons({
-                    'authors'   : QIcon(I('user_profile.svg')),
-                    'series'    : QIcon(I('series.svg')),
-                    'formats'   : QIcon(I('book.svg')),
+                    'authors'   : QIcon(I('user_profile.png')),
+                    'series'    : QIcon(I('series.png')),
+                    'formats'   : QIcon(I('book.png')),
                     'publisher' : QIcon(I('publisher.png')),
                     'rating'    : QIcon(I('star.png')),
-                    'news'      : QIcon(I('news.svg')),
-                    'tags'      : QIcon(I('tags.svg')),
-                    ':custom'   : QIcon(I('column.svg')),
-                    ':user'     : QIcon(I('drawer.svg')),
-                    'search'    : QIcon(I('search.svg'))})
+                    'news'      : QIcon(I('news.png')),
+                    'tags'      : QIcon(I('tags.png')),
+                    ':custom'   : QIcon(I('column.png')),
+                    ':user'     : QIcon(I('drawer.png')),
+                    'search'    : QIcon(I('search.png'))})
         self.categories_with_ratings = ['authors', 'series', 'publisher', 'tags']
 
-        self.icon_state_map = [None, QIcon(I('plus.svg')), QIcon(I('minus.svg'))]
+        self.icon_state_map = [None, QIcon(I('plus.png')), QIcon(I('minus.png'))]
         self.db = db
         self.tags_view = parent
         self.hidden_categories = hidden_categories
@@ -422,14 +424,12 @@ class TagsModel(QAbstractItemModel): # {{{
         self.categories = []
 
         # Reconstruct the user categories, putting them into metadata
+        self.db.field_metadata.remove_dynamic_categories()
         tb_cats = self.db.field_metadata
-        for k in tb_cats.keys():
-            if tb_cats[k]['kind'] in ['user', 'search']:
-                del tb_cats[k]
-        for user_cat in sorted(prefs['user_categories'].keys()):
+        for user_cat in sorted(self.db.prefs.get('user_categories', {}).keys()):
             cat_name = user_cat+':' # add the ':' to avoid name collision
             tb_cats.add_user_category(label=cat_name, name=user_cat)
-        if len(saved_searches.names()):
+        if len(saved_searches().names()):
             tb_cats.add_search_category(label='search', name=_('Searches'))
 
         # Now get the categories
@@ -507,11 +507,11 @@ class TagsModel(QAbstractItemModel): # {{{
         if key not in self.db.field_metadata:
             return
         if key == 'search':
-            if val in saved_searches.names():
+            if val in saved_searches().names():
                 error_dialog(self.tags_view, _('Duplicate search name'),
                     _('The saved search name %s is already used.')%val).exec_()
                 return False
-            saved_searches.rename(unicode(item.data(role).toString()), val)
+            saved_searches().rename(unicode(item.data(role).toString()), val)
             self.tags_view.search_item_renamed.emit()
         else:
             if key == 'series':
@@ -768,6 +768,9 @@ class TagBrowserWidget(QWidget): # {{{
         for x in (_('Sort by name'), _('Sort by popularity'),
                   _('Sort by average rating')):
             parent.sort_by.addItem(x)
+        parent.sort_by.setToolTip(
+                _('Set the sort order for entries in the Tag Browser'))
+        parent.sort_by.setStatusTip(parent.sort_by.toolTip())
         parent.sort_by.setCurrentIndex(0)
         self._layout.addWidget(parent.sort_by)
 
@@ -776,9 +779,16 @@ class TagBrowserWidget(QWidget): # {{{
             parent.tag_match.addItem(x)
         parent.tag_match.setCurrentIndex(0)
         self._layout.addWidget(parent.tag_match)
+        parent.tag_match.setToolTip(
+                _('When selecting multiple entries in the Tag Browser '
+                    'match any or all of them'))
+        parent.tag_match.setStatusTip(parent.tag_match.toolTip())
 
         parent.edit_categories = QPushButton(_('Manage &user categories'), parent)
         self._layout.addWidget(parent.edit_categories)
+        parent.edit_categories.setToolTip(
+                _('Add your own categories to the Tag Browser'))
+        parent.edit_categories.setStatusTip(parent.edit_categories.toolTip())
 
 
 # }}}
