@@ -22,6 +22,7 @@ from calibre.customize.conversion import OptionRecommendation
 from calibre.utils.config import prefs
 from calibre.ebooks.conversion.config import GuiRecommendations, \
     load_defaults, load_specifics, save_specifics
+from calibre.gui2.convert import bulk_defaults_for_input_format
 
 def convert_single_ebook(parent, db, book_ids, auto_conversion=False, out_format=None):
     changed = False
@@ -148,7 +149,7 @@ class QueueBulk(QProgressDialog):
             temp_files = []
 
             combined_recs = GuiRecommendations()
-            default_recs = load_defaults('%s_input' % input_format)
+            default_recs = bulk_defaults_for_input_format(input_format)
             for key in default_recs:
                 combined_recs[key] = default_recs[key]
             if self.use_saved_single_settings:
@@ -208,7 +209,6 @@ class QueueBulk(QProgressDialog):
         self.queue(self.jobs, self.changed, self.bad, *self.args)
 
 def fetch_scheduled_recipe(arg):
-    from calibre.ebooks.conversion.config import load_defaults
     fmt = prefs['output_format'].lower()
     pt = PersistentTemporaryFile(suffix='_recipe_out.%s'%fmt.lower())
     pt.close()
@@ -221,6 +221,8 @@ def fetch_scheduled_recipe(arg):
     if lf.get('base_font_size', 0.0) != 0.0:
         recs.append(('base_font_size', lf['base_font_size'],
             OptionRecommendation.HIGH))
+        recs.append(('keep_ligatures', lf.get('keep_ligatures', False),
+            OptionRecommendation.HIGH))
 
     lr = load_defaults('lrf_output')
     if lr.get('header', False):
@@ -236,7 +238,7 @@ def fetch_scheduled_recipe(arg):
 
     return 'gui_convert', args, _('Fetch news from ')+arg['title'], fmt.upper(), [pt]
 
-def generate_catalog(parent, dbspec, ids, device):
+def generate_catalog(parent, dbspec, ids, device_manager):
     from calibre.gui2.dialogs.catalog import Catalog
 
     # Build the Catalog dialog in gui2.dialogs.catalog
@@ -250,9 +252,18 @@ def generate_catalog(parent, dbspec, ids, device):
 
     # Profile the connected device
     # Parallel initialization in calibre.library.cli:command_catalog()
-    connected_device = { 'storage':None,'serial':None,'save_template':None,'name':None}
+    connected_device = {
+                         'is_device_connected': device_manager.is_device_connected,
+                         'kind': device_manager.connected_device_kind,
+                         'name': None,
+                         'save_template': None,
+                         'serial': None,
+                         'storage': None
+                       }
 
-    if device:
+    if device_manager.is_device_connected:
+        device = device_manager.device
+        connected_device['name'] = device.gui_name
         try:
             storage = []
             if device._main_prefix:
@@ -261,11 +272,10 @@ def generate_catalog(parent, dbspec, ids, device):
                 storage.append(os.path.join(device._card_a_prefix, device.EBOOK_DIR_CARD_A))
             if device._card_b_prefix:
                 storage.append(os.path.join(device._card_b_prefix, device.EBOOK_DIR_CARD_B))
-            connected_device = {      'storage': storage,
-                                       'serial': device.detected_device.serial if \
-                                                 hasattr(device.detected_device,'serial') else None,
-                                'save_template': device.save_template(),
-                                         'name': device.gui_name}
+            connected_device['storage'] = storage
+            connected_device['serial'] = device.detected_device.serial if \
+                                          hasattr(device.detected_device,'serial') else None
+            connected_device['save_template'] = device.save_template()
         except:
             pass
 
