@@ -12,39 +12,32 @@ __docformat__ = 'restructuredtext en'
 import collections, os, sys, textwrap, time
 from Queue import Queue, Empty
 from threading import Thread
-from PyQt4.Qt import Qt, SIGNAL, QObject, QTimer, \
+from PyQt4.Qt import Qt, SIGNAL, QTimer, \
                      QPixmap, QMenu, QIcon, pyqtSignal, \
                      QDialog, \
-                     QSystemTrayIcon, QApplication, QKeySequence, QAction, \
+                     QSystemTrayIcon, QApplication, QKeySequence, \
                      QMessageBox, QHelpEvent
 
-from calibre import  prints, patheq
-from calibre.constants import __version__, __appname__, isosx
+from calibre import  prints
+from calibre.constants import __appname__, isosx
 from calibre.ptempfile import PersistentTemporaryFile
 from calibre.utils.config import prefs, dynamic
 from calibre.utils.ipc.server import Server
+from calibre.library.database2 import LibraryDatabase2
+from calibre.customize.ui import interface_actions
 from calibre.gui2 import error_dialog, GetMetadata, open_local_file, \
-        gprefs, max_available_height, config, info_dialog
+        gprefs, max_available_height, config, info_dialog, Dispatcher
 from calibre.gui2.cover_flow import CoverFlowMixin
 from calibre.gui2.widgets import ProgressIndicator
-from calibre.gui2.wizard import move_library
-from calibre.gui2.dialogs.scheduler import Scheduler
 from calibre.gui2.update import UpdateMixin
 from calibre.gui2.main_window import MainWindow
-from calibre.gui2.main_ui import Ui_MainWindow
+from calibre.gui2.layout import MainWindowMixin
 from calibre.gui2.device import DeviceMixin
 from calibre.gui2.jobs import JobManager, JobsDialog, JobsButton
-from calibre.gui2.dialogs.config import ConfigDialog
-
-from calibre.gui2.dialogs.book_info import BookInfo
-from calibre.library.database2 import LibraryDatabase2
-from calibre.gui2.init import ToolbarMixin, LibraryViewMixin, LayoutMixin
+from calibre.gui2.init import LibraryViewMixin, LayoutMixin
 from calibre.gui2.search_box import SearchBoxMixin, SavedSearchBoxMixin
 from calibre.gui2.search_restriction_mixin import SearchRestrictionMixin
 from calibre.gui2.tag_view import TagBrowserMixin
-from calibre.gui2.actions import AnnotationsAction, AddAction, DeleteAction, \
-    EditMetadataAction, SaveToDiskAction, GenerateCatalogAction, FetchNewsAction, \
-    ConvertAction, ViewAction
 
 
 class Listener(Thread): # {{{
@@ -57,6 +50,8 @@ class Listener(Thread): # {{{
         self.start()
 
     def run(self):
+        if self.listener is None:
+            return
         while self._run:
             try:
                 conn = self.listener.accept()
@@ -91,23 +86,35 @@ class SystemTrayIcon(QSystemTrayIcon): # {{{
 
 # }}}
 
-class Main(MainWindow, Ui_MainWindow, DeviceMixin, ToolbarMixin, # {{{
+class Main(MainWindow, MainWindowMixin, DeviceMixin, # {{{
         TagBrowserMixin, CoverFlowMixin, LibraryViewMixin, SearchBoxMixin,
-        SavedSearchBoxMixin, SearchRestrictionMixin, LayoutMixin, UpdateMixin,
-        AnnotationsAction, AddAction, DeleteAction,
-        EditMetadataAction, SaveToDiskAction, GenerateCatalogAction, FetchNewsAction,
-        ConvertAction, ViewAction):
+        SavedSearchBoxMixin, SearchRestrictionMixin, LayoutMixin, UpdateMixin
+        ):
     'The main GUI'
 
 
     def __init__(self, opts, parent=None):
         MainWindow.__init__(self, opts, parent)
         self.opts = opts
+        self.device_connected = None
+        acmap = {}
+        for action in interface_actions():
+            mod, cls = action.actual_plugin.split(':')
+            ac = getattr(__import__(mod, fromlist=['1'], level=0), cls)(self,
+                    action.site_customization)
+            if ac.name in acmap:
+                if ac.priority >= acmap[ac.name].priority:
+                    acmap[ac.name] = ac
+            else:
+                acmap[ac.name] = ac
 
-    def initialize(self, library_path, db, listener, actions):
+        self.iactions = acmap
+
+    def initialize(self, library_path, db, listener, actions, show_gui=True):
         opts = self.opts
         self.preferences_action, self.quit_action = actions
         self.library_path = library_path
+        self.content_server = None
         self.spare_servers = []
         self.must_restart_before_config = False
         # Initialize fontconfig in a separate thread as this can be a lengthy
@@ -120,12 +127,14 @@ class Main(MainWindow, Ui_MainWindow, DeviceMixin, ToolbarMixin, # {{{
                 self.another_instance_wants_to_talk)
         self.check_messages_timer.start(1000)
 
-        Ui_MainWindow.__init__(self)
+        for ac in self.iactions.values():
+            ac.do_genesis()
+        MainWindowMixin.__init__(self, db)
 
         # Jobs Button {{{
         self.job_manager = JobManager()
         self.jobs_dialog = JobsDialog(self, self.job_manager)
-        self.jobs_button = JobsButton(horizontal=True)
+        self.jobs_button = JobsButton(horizontal=True, parent=self)
         self.jobs_button.initialize(self.jobs_dialog, self.job_manager)
         # }}}
 
@@ -141,14 +150,10 @@ class Main(MainWindow, Ui_MainWindow, DeviceMixin, ToolbarMixin, # {{{
         self.verbose = opts.verbose
         self.get_metadata = GetMetadata()
         self.upload_memory = {}
-        self.delete_memory = {}
-        self.conversion_jobs = {}
-        self.persistent_files = []
         self.metadata_dialogs = []
         self.default_thumbnail = None
         self.tb_wrapper = textwrap.TextWrapper(width=40)
         self.viewers = collections.deque()
-        self.content_server = None
         self.system_tray_icon = SystemTrayIcon(QIcon(I('library.png')), self)
         self.system_tray_icon.setToolTip('calibre')
         self.system_tray_icon.tooltip_requested.connect(
@@ -159,32 +164,22 @@ class Main(MainWindow, Ui_MainWindow, DeviceMixin, ToolbarMixin, # {{{
             self.system_tray_icon.show()
         self.system_tray_menu = QMenu(self)
         self.restore_action = self.system_tray_menu.addAction(
-                QIcon(I('page.svg')), _('&Restore'))
+                QIcon(I('page.png')), _('&Restore'))
         self.donate_action  = self.system_tray_menu.addAction(
-                QIcon(I('donate.svg')), _('&Donate to support calibre'))
+                QIcon(I('donate.png')), _('&Donate to support calibre'))
         self.donate_button.setDefaultAction(self.donate_action)
+        self.donate_button.setStatusTip(self.donate_button.toolTip())
         self.eject_action = self.system_tray_menu.addAction(
-                QIcon(I('eject.svg')), _('&Eject connected device'))
+                QIcon(I('eject.png')), _('&Eject connected device'))
         self.eject_action.setEnabled(False)
-        if not config['show_donate_button']:
-            self.donate_button.setVisible(False)
         self.addAction(self.quit_action)
-        self.action_restart = QAction(_('&Restart'), self)
-        self.addAction(self.action_restart)
         self.system_tray_menu.addAction(self.quit_action)
         self.quit_action.setShortcut(QKeySequence(Qt.CTRL + Qt.Key_Q))
-        self.action_restart.setShortcut(QKeySequence(Qt.CTRL + Qt.Key_R))
-        self.action_show_book_details.setShortcut(QKeySequence(Qt.Key_I))
-        self.addAction(self.action_show_book_details)
         self.system_tray_icon.setContextMenu(self.system_tray_menu)
         self.connect(self.quit_action, SIGNAL('triggered(bool)'), self.quit)
         self.connect(self.donate_action, SIGNAL('triggered(bool)'), self.donate)
         self.connect(self.restore_action, SIGNAL('triggered()'),
                         self.show_windows)
-        self.connect(self.action_show_book_details,
-                     SIGNAL('triggered(bool)'), self.show_book_info)
-        self.connect(self.action_restart, SIGNAL('triggered()'),
-                     self.restart)
         self.connect(self.system_tray_icon,
                      SIGNAL('activated(QSystemTrayIcon::ActivationReason)'),
                      self.system_tray_icon_activated)
@@ -193,32 +188,14 @@ class Main(MainWindow, Ui_MainWindow, DeviceMixin, ToolbarMixin, # {{{
         ####################### Start spare job server ########################
         QTimer.singleShot(1000, self.add_spare_server)
 
-        ####################### Location View ########################
-        QObject.connect(self.location_view,
-                SIGNAL('location_selected(PyQt_PyObject)'),
-                        self.location_selected)
-        QObject.connect(self.location_view,
-                SIGNAL('umount_device()'),
-                        self.device_manager.umount_device)
+        ####################### Location Manager ########################
+        self.location_manager.location_selected.connect(self.location_selected)
+        self.location_manager.unmount_device.connect(self.device_manager.umount_device)
         self.eject_action.triggered.connect(self.device_manager.umount_device)
 
-        ####################### Vanity ########################
-        self.vanity_template  = _('<p>For help see the: <a href="%s">User Manual</a>'
-                '<br>')%'http://calibre-ebook.com/user_manual'
-        dv = os.environ.get('CALIBRE_DEVELOP_FROM', None)
-        v = __version__
-        if getattr(sys, 'frozen', False) and dv and os.path.abspath(dv) in sys.path:
-            v += '*'
-        self.vanity_template += _('<b>%s</b>: %s by <b>Kovid Goyal '
-            '%%(version)s</b><br>%%(device)s</p>')%(__appname__, v)
-        self.latest_version = ' '
-        self.vanity.setText(self.vanity_template%dict(version=' ', device=' '))
-        self.device_info = ' '
+        #################### Update notification ###################
         UpdateMixin.__init__(self, opts)
 
-        ####################### Setup Toolbar #####################
-        ToolbarMixin.__init__(self)
-
         ####################### Search boxes ########################
         SavedSearchBoxMixin.__init__(self)
         SearchBoxMixin.__init__(self)
@@ -226,12 +203,13 @@ class Main(MainWindow, Ui_MainWindow, DeviceMixin, ToolbarMixin, # {{{
         ####################### Library view ########################
         LibraryViewMixin.__init__(self, db)
 
-        self.show()
+        if show_gui:
+            self.show()
 
         if self.system_tray_icon.isVisible() and opts.start_in_tray:
             self.hide_windows()
-        self.library_view.model().count_changed_signal.connect \
-                                            (self.location_view.count_changed)
+        self.library_view.model().count_changed_signal.connect(
+                self.iactions['Choose Library'].count_changed)
         if not gprefs.get('quick_start_guide_added', False):
             from calibre.ebooks.metadata import MetaInformation
             mi = MetaInformation(_('Calibre Quick Start Guide'), ['John Schember'])
@@ -246,8 +224,8 @@ class Main(MainWindow, Ui_MainWindow, DeviceMixin, ToolbarMixin, # {{{
                 self.db_images.reset()
 
         self.library_view.model().count_changed()
-        self.location_view.model().database_changed(self.library_view.model().db)
-        self.library_view.model().database_changed.connect(self.location_view.model().database_changed,
+        self.tool_bar.database_changed(self.library_view.model().db)
+        self.library_view.model().database_changed.connect(self.tool_bar.database_changed,
                 type=Qt.QueuedConnection)
 
         ########################### Tags Browser ##############################
@@ -255,6 +233,7 @@ class Main(MainWindow, Ui_MainWindow, DeviceMixin, ToolbarMixin, # {{{
 
         ######################### Search Restriction ##########################
         SearchRestrictionMixin.__init__(self)
+        self.apply_named_search_restriction(db.prefs['gui_restriction'])
 
         ########################### Cover Flow ################################
 
@@ -266,31 +245,30 @@ class Main(MainWindow, Ui_MainWindow, DeviceMixin, ToolbarMixin, # {{{
 
 
         if config['autolaunch_server']:
-            from calibre.library.server.main import start_threaded_server
-            from calibre.library.server import server_config
-            self.content_server = start_threaded_server(
-                    db, server_config().parse())
-            self.test_server_timer = QTimer.singleShot(10000, self.test_server)
-
-
-        self.scheduler = Scheduler(self, self.library_view.model().db)
-        self.action_news.setMenu(self.scheduler.news_menu)
-        self.connect(self.action_news, SIGNAL('triggered(bool)'),
-                self.scheduler.show_dialog)
-        self.connect(self.scheduler, SIGNAL('delete_old_news(PyQt_PyObject)'),
-                self.library_view.model().delete_books_by_id,
-                Qt.QueuedConnection)
-        self.connect(self.scheduler,
-                SIGNAL('start_recipe_fetch(PyQt_PyObject)'),
-                self.download_scheduled_recipe, Qt.QueuedConnection)
-
-        self.location_view.setCurrentIndex(self.location_view.model().index(0))
+            self.start_content_server()
 
         self.keyboard_interrupt.connect(self.quit, type=Qt.QueuedConnection)
-        AddAction.__init__(self)
+
 
         self.read_settings()
         self.finalize_layout()
+        if self.tool_bar.showing_donate:
+            self.donate_button.start_animation()
+        self.set_window_title()
+
+        for ac in self.iactions.values():
+            ac.initialization_complete()
+
+    def start_content_server(self):
+        from calibre.library.server.main import start_threaded_server
+        from calibre.library.server import server_config
+        self.content_server = start_threaded_server(
+                self.library_view.model().db, server_config().parse())
+        self.content_server.state_callback = Dispatcher(
+                self.iactions['Connect Share'].content_server_state_changed)
+        self.content_server.state_callback(True)
+        self.test_server_timer = QTimer.singleShot(10000, self.test_server)
+
 
     def resizeEvent(self, ev):
         MainWindow.resizeEvent(self, ev)
@@ -336,7 +314,8 @@ class Main(MainWindow, Ui_MainWindow, DeviceMixin, ToolbarMixin, # {{{
                 setattr(window, '__systray_minimized', False)
 
     def test_server(self, *args):
-        if self.content_server.exception is not None:
+        if self.content_server is not None and \
+                self.content_server.exception is not None:
             error_dialog(self, _('Failed to start content server'),
                          unicode(self.content_server.exception)).exec_()
 
@@ -351,7 +330,7 @@ class Main(MainWindow, Ui_MainWindow, DeviceMixin, ToolbarMixin, # {{{
             if len(argv) > 1:
                 path = os.path.abspath(argv[1])
                 if os.access(path, os.R_OK):
-                    self.add_filesystem_book(path)
+                    self.iactions['Add Books'].add_filesystem_book(path)
             self.setWindowState(self.windowState() & \
                     ~Qt.WindowMinimized|Qt.WindowActive)
             self.show_windows()
@@ -378,41 +357,6 @@ class Main(MainWindow, Ui_MainWindow, DeviceMixin, ToolbarMixin, # {{{
     def booklists(self):
         return self.memory_view.model().db, self.card_a_view.model().db, self.card_b_view.model().db
 
-
-    def do_config(self, *args):
-        if self.job_manager.has_jobs():
-            d = error_dialog(self, _('Cannot configure'),
-                    _('Cannot configure while there are running jobs.'))
-            d.exec_()
-            return
-        if self.must_restart_before_config:
-            d = error_dialog(self, _('Cannot configure'),
-                    _('Cannot configure before calibre is restarted.'))
-            d.exec_()
-            return
-        d = ConfigDialog(self, self.library_view,
-                server=self.content_server)
-
-        d.exec_()
-        self.content_server = d.server
-        if d.result() == d.Accepted:
-            self.read_toolbar_settings()
-            self.search.search_as_you_type(config['search_as_you_type'])
-            self.save_menu.actions()[2].setText(
-                _('Save only %s format to disk')%
-                prefs['output_format'].upper())
-            self.save_menu.actions()[3].setText(
-                _('Save only %s format to disk in a single directory')%
-                prefs['output_format'].upper())
-            self.tags_view.set_new_model() # in case columns changed
-            self.tags_view.recount()
-            self.create_device_menu()
-            self.set_device_menu_items_state(bool(self.device_connected))
-            if not patheq(self.library_path, d.database_location):
-                newloc = d.database_location
-                move_library(self.library_path, newloc, self,
-                        self.library_moved)
-
     def library_moved(self, newloc):
         if newloc is None: return
         db = LibraryDatabase2(newloc)
@@ -424,19 +368,20 @@ class Main(MainWindow, Ui_MainWindow, DeviceMixin, ToolbarMixin, # {{{
         self.library_view.model().set_book_on_device_func(self.book_on_device)
         self.status_bar.clear_message()
         self.search.clear_to_help()
+        self.saved_search.clear_to_help()
         self.book_details.reset_info()
         self.library_view.model().count_changed()
         prefs['library_path'] = self.library_path
+        db = self.library_view.model().db
+        for action in self.iactions.values():
+            action.library_changed(db)
+        self.set_window_title()
+        self.apply_named_search_restriction('') # reset restriction to null
+        self.saved_searches_changed() # reload the search restrictions combo box
+        self.apply_named_search_restriction(db.prefs['gui_restriction'])
 
-    def show_book_info(self, *args):
-        if self.current_view() is not self.library_view:
-            error_dialog(self, _('No detailed info available'),
-                _('No detailed information is available for books '
-                  'on the device.')).exec_()
-            return
-        index = self.library_view.currentIndex()
-        if index.isValid():
-            BookInfo(self, self.library_view, index).show()
+    def set_window_title(self):
+        self.setWindowTitle(__appname__ + u' - ||%s||'%self.iactions['Choose Library'].library_name())
 
     def location_selected(self, location):
         '''
@@ -448,26 +393,12 @@ class Main(MainWindow, Ui_MainWindow, DeviceMixin, ToolbarMixin, # {{{
         for x in ('tb', 'cb'):
             splitter = getattr(self, x+'_splitter')
             splitter.button.setEnabled(location == 'library')
+        for action in self.iactions.values():
+            action.location_selected(location)
         if location == 'library':
-            self.action_edit.setEnabled(True)
-            self.action_merge.setEnabled(True)
-            self.action_convert.setEnabled(True)
-            self.view_menu.actions()[1].setEnabled(True)
-            self.action_open_containing_folder.setEnabled(True)
-            self.action_sync.setEnabled(True)
             self.search_restriction.setEnabled(True)
-            for action in list(self.delete_menu.actions())[1:]:
-                action.setEnabled(True)
         else:
-            self.action_edit.setEnabled(False)
-            self.action_merge.setEnabled(False)
-            self.action_convert.setEnabled(False)
-            self.view_menu.actions()[1].setEnabled(False)
-            self.action_open_containing_folder.setEnabled(False)
-            self.action_sync.setEnabled(False)
             self.search_restriction.setEnabled(False)
-            for action in list(self.delete_menu.actions())[1:]:
-                action.setEnabled(False)
             # Reset the view in case something changed while it was invisible
             self.current_view().reset()
         self.set_number_of_books_shown()
@@ -522,7 +453,6 @@ class Main(MainWindow, Ui_MainWindow, DeviceMixin, ToolbarMixin, # {{{
         geometry = config['main_window_geometry']
         if geometry is not None:
             self.restoreGeometry(geometry)
-        self.read_toolbar_settings()
         self.read_layout_settings()
 
     def write_settings(self):
@@ -530,9 +460,6 @@ class Main(MainWindow, Ui_MainWindow, DeviceMixin, ToolbarMixin, # {{{
         dynamic.set('sort_history', self.library_view.model().sort_history)
         self.save_layout_state()
 
-    def restart(self):
-        self.quit(restart=True)
-
     def quit(self, checked=True, restart=False):
         if not self.confirm_quit():
             return
@@ -554,7 +481,8 @@ class Main(MainWindow, Ui_MainWindow, DeviceMixin, ToolbarMixin, # {{{
         '''
         MSG = _('is the result of the efforts of many volunteers from all '
                 'over the world. If you find it useful, please consider '
-                'donating to support its development.')
+                'donating to support its development. Your donation helps '
+                'keep calibre development going.')
         HTML = u'''
         <html>
             <head>
@@ -586,7 +514,7 @@ class Main(MainWindow, Ui_MainWindow, DeviceMixin, ToolbarMixin, # {{{
 
             d = QMessageBox(QMessageBox.Warning, _('WARNING: Active jobs'), msg,
                             QMessageBox.Yes|QMessageBox.No, self)
-            d.setIconPixmap(QPixmap(I('dialog_warning.svg')))
+            d.setIconPixmap(QPixmap(I('dialog_warning.png')))
             d.setDefaultButton(QMessageBox.No)
             if d.exec_() != QMessageBox.Yes:
                 return False
@@ -594,6 +522,19 @@ class Main(MainWindow, Ui_MainWindow, DeviceMixin, ToolbarMixin, # {{{
 
 
     def shutdown(self, write_settings=True):
+        try:
+            db = self.library_view.model().db
+            cf = db.clean
+        except:
+            pass
+        else:
+            cf()
+            # Save the current field_metadata for applications like calibre2opds
+            # Goes here, because if cf is valid, db is valid.
+            db.prefs['field_metadata'] = db.field_metadata.all_metadata()
+        for action in self.iactions.values():
+            if not action.shutting_down():
+                return
         if write_settings:
             self.write_settings()
         self.check_messages_timer.stop()
@@ -611,7 +552,9 @@ class Main(MainWindow, Ui_MainWindow, DeviceMixin, ToolbarMixin, # {{{
         try:
             try:
                 if self.content_server is not None:
-                    self.content_server.exit()
+                    s = self.content_server
+                    self.content_server = None
+                    s.exit()
             except:
                 pass
             time.sleep(2)
