@@ -81,7 +81,8 @@ class ConfigDialog(QDialog, Ui_Dialog):
         self.css.setToolTip(_('Set the user CSS stylesheet. This can be used to customize the look of all books.'))
         self.max_view_width.setValue(opts.max_view_width)
         pats = [os.path.basename(x).split('.')[0] for x in
-            glob.glob(P('viewer/hyphenate/patterns/*.js'))]
+            glob.glob(P('viewer/hyphenate/patterns/*.js',
+                allow_user_override=False))]
         names = list(map(get_language, pats))
         pmap = {}
         for i in range(len(pats)):
@@ -443,7 +444,7 @@ class DocumentView(QWebView):
         self.connect(self.document, SIGNAL('animated_scroll_done()'),
                 self.animated_scroll_done, Qt.QueuedConnection)
         copy_action = self.pageAction(self.document.Copy)
-        copy_action.setIcon(QIcon(I('convert.svg')))
+        copy_action.setIcon(QIcon(I('convert.png')))
         d = self.document
         self.unimplemented_actions = list(map(self.pageAction,
             [d.DownloadImageToDisk, d.OpenLinkInNewWindow, d.DownloadLinkToDisk,
