@@ -5,26 +5,25 @@ Miscellaneous widgets used in the GUI
 '''
 import re, os, traceback
 
-from PyQt4.Qt import QListView, QIcon, QFont, QLabel, QListWidget, \
+from PyQt4.Qt import QIcon, QFont, QLabel, QListWidget, QAction, \
                         QListWidgetItem, QTextCharFormat, QApplication, \
                         QSyntaxHighlighter, QCursor, QColor, QWidget, \
-                        QPixmap, QPalette, QSplitterHandle, QToolButton, \
+                        QPixmap, QSplitterHandle, QToolButton, \
                         QAbstractListModel, QVariant, Qt, SIGNAL, pyqtSignal, \
-                        QRegExp, QSettings, QSize, QModelIndex, QSplitter, \
-                        QAbstractButton, QPainter, QLineEdit, QComboBox, \
+                        QRegExp, QSettings, QSize, QSplitter, \
+                        QPainter, QLineEdit, QComboBox, QPen, \
                         QMenu, QStringListModel, QCompleter, QStringList, \
                         QTimer, QRect
 
 from calibre.gui2 import NONE, error_dialog, pixmap_to_data, gprefs
 
 from calibre.gui2.filename_pattern_ui import Ui_Form
-from calibre import fit_image, human_readable
+from calibre import fit_image
 from calibre.utils.fonts import fontconfig
 from calibre.ebooks import BOOK_EXTENSIONS
 from calibre.ebooks.metadata.meta import metadata_from_filename
 from calibre.utils.config import prefs, XMLConfig
 from calibre.gui2.progress_indicator import ProgressIndicator as _ProgressIndicator
-from calibre.constants import filesystem_encoding
 
 history = XMLConfig('history')
 
@@ -62,13 +61,24 @@ class ProgressIndicator(QWidget):
 
 class FilenamePattern(QWidget, Ui_Form):
 
+    changed_signal = pyqtSignal()
+
     def __init__(self, parent):
         QWidget.__init__(self, parent)
         self.setupUi(self)
 
         self.connect(self.test_button, SIGNAL('clicked()'), self.do_test)
         self.connect(self.re, SIGNAL('returnPressed()'), self.do_test)
-        self.re.setText(prefs['filename_pattern'])
+        self.initialize()
+        self.re.textChanged.connect(lambda x: self.changed_signal.emit())
+
+    def initialize(self, defaults=False):
+        if defaults:
+            val = prefs.defaults['filename_pattern']
+        else:
+            val = prefs['filename_pattern']
+        self.re.setText(val)
+
 
     def do_test(self):
         try:
@@ -152,11 +162,14 @@ class FormatList(QListWidget):
 
 class ImageView(QWidget):
 
+    BORDER_WIDTH = 1
+
     def __init__(self, parent=None):
         QWidget.__init__(self, parent)
         self._pixmap = QPixmap(self)
         self.setMinimumSize(QSize(150, 200))
         self.setAcceptDrops(True)
+        self.draw_border = True
 
     # Drag 'n drop {{{
     DROPABBLE_EXTENSIONS = IMAGE_EXTENSIONS
@@ -230,6 +243,11 @@ class ImageView(QWidget):
         p = QPainter(self)
         p.setRenderHints(QPainter.Antialiasing | QPainter.SmoothPixmapTransform)
         p.drawPixmap(target, pmap)
+        pen = QPen()
+        pen.setWidth(self.BORDER_WIDTH)
+        p.setPen(pen)
+        if self.draw_border:
+            p.drawRect(target)
         p.end()
 
 
@@ -259,189 +277,6 @@ class ImageView(QWidget):
     # }}}
 
 
-class LocationModel(QAbstractListModel):
-
-    def __init__(self, parent):
-        QAbstractListModel.__init__(self, parent)
-        self.icons = [QVariant(QIcon(I('library.png'))),
-                      QVariant(QIcon(I('reader.svg'))),
-                      QVariant(QIcon(I('sd.svg'))),
-                      QVariant(QIcon(I('sd.svg')))]
-        self.text = [_('Library\n%d\nbooks'),
-                     _('Reader\n%s\navailable'),
-                     _('Card A\n%s\navailable'),
-                     _('Card B\n%s\navailable')]
-        self.free = [-1, -1, -1]
-        self.count = 0
-        self.highlight_row = 0
-        self.library_tooltip = _('Click to see the books available on your computer')
-        self.tooltips = [
-                         self.library_tooltip,
-                         _('Click to see the books in the main memory of your reader'),
-                         _('Click to see the books on storage card A in your reader'),
-                         _('Click to see the books on storage card B in your reader')
-                         ]
-
-    def database_changed(self, db):
-        lp = db.library_path
-        if not isinstance(lp, unicode):
-            lp = lp.decode(filesystem_encoding, 'replace')
-        self.tooltips[0] = self.library_tooltip + '\n\n' + \
-                _('Books located at') + ' ' + lp
-        self.dataChanged.emit(self.index(0), self.index(0))
-
-    def rowCount(self, *args):
-        return 1 + len([i for i in self.free if i >= 0])
-
-    def get_device_row(self, row):
-        if row == 2 and self.free[1] == -1 and self.free[2] > -1:
-            row = 3
-        return row
-
-    def data(self, index, role):
-        row = index.row()
-        drow = self.get_device_row(row)
-        data = NONE
-        if role == Qt.DisplayRole:
-            text = self.text[drow]%(human_readable(self.free[drow-1])) if row > 0 \
-                            else self.text[drow]%self.count
-            data = QVariant(text)
-        elif role == Qt.DecorationRole:
-            data = self.icons[drow]
-        elif role == Qt.ToolTipRole:
-            data = QVariant(self.tooltips[drow])
-        elif role == Qt.SizeHintRole:
-            data = QVariant(QSize(155, 90))
-        elif role == Qt.FontRole:
-            font = QFont('monospace')
-            font.setBold(row == self.highlight_row)
-            data = QVariant(font)
-        elif role == Qt.ForegroundRole and row == self.highlight_row:
-            return QVariant(QApplication.palette().brush(
-                QPalette.HighlightedText))
-        elif role == Qt.BackgroundRole and row == self.highlight_row:
-            return QVariant(QApplication.palette().brush(
-                QPalette.Highlight))
-
-        return data
-
-    def device_connected(self, dev):
-        self.icons[1] = QIcon(dev.icon)
-        self.dataChanged.emit(self.index(1), self.index(1))
-
-    def headerData(self, section, orientation, role):
-        return NONE
-
-    def update_devices(self, cp=(None, None), fs=[-1, -1, -1]):
-        if cp is None:
-            cp = (None, None)
-        if isinstance(cp, (str, unicode)):
-            cp = (cp, None)
-        if len(fs) < 3:
-            fs = list(fs) + [0]
-        self.free[0] = fs[0]
-        self.free[1] = fs[1]
-        self.free[2] = fs[2]
-        cpa, cpb = cp
-        self.free[1] = fs[1] if fs[1] is not None and cpa is not None else -1
-        self.free[2] = fs[2] if fs[2] is not None and cpb is not None else -1
-        self.reset()
-        self.emit(SIGNAL('devicesChanged()'))
-
-    def location_changed(self, row):
-        self.highlight_row = row
-        self.emit(SIGNAL('dataChanged(QModelIndex,QModelIndex)'),
-                self.index(0), self.index(self.rowCount(QModelIndex())-1))
-
-    def location_for_row(self, row):
-        if row == 0: return 'library'
-        if row == 1: return 'main'
-        if row == 3: return 'cardb'
-        return 'carda' if self.free[1] > -1 else 'cardb'
-
-class LocationView(QListView):
-
-    def __init__(self, parent):
-        QListView.__init__(self, parent)
-        self.setModel(LocationModel(self))
-        self.reset()
-        self.currentChanged = self.current_changed
-
-        self.eject_button = EjectButton(self)
-        self.eject_button.hide()
-
-        self.connect(self, SIGNAL('entered(QModelIndex)'), self.item_entered)
-        self.connect(self, SIGNAL('viewportEntered()'), self.viewport_entered)
-        self.connect(self.eject_button, SIGNAL('clicked()'), lambda: self.emit(SIGNAL('umount_device()')))
-        self.connect(self.model(), SIGNAL('devicesChanged()'), self.eject_button.hide)
-
-    def count_changed(self, new_count):
-        self.model().count = new_count
-        self.model().reset()
-
-    def current_changed(self, current, previous):
-        if current.isValid():
-            i = current.row()
-            location = self.model().location_for_row(i)
-            self.emit(SIGNAL('location_selected(PyQt_PyObject)'), location)
-            self.model().location_changed(i)
-
-    def location_changed(self, row):
-        if 0 <= row and row <= 3:
-            self.model().location_changed(row)
-
-    def leaveEvent(self, event):
-        self.unsetCursor()
-        self.eject_button.hide()
-
-    def item_entered(self, location):
-        self.setCursor(Qt.PointingHandCursor)
-        self.eject_button.hide()
-
-        if location.row() == 1:
-            rect = self.visualRect(location)
-
-            self.eject_button.resize(rect.height()/2, rect.height()/2)
-
-            x, y = rect.left(), rect.top()
-            x = x + (rect.width() - self.eject_button.width() - 2)
-            y += 6
-
-            self.eject_button.move(x, y)
-            self.eject_button.show()
-
-    def viewport_entered(self):
-        self.unsetCursor()
-        self.eject_button.hide()
-
-
-class EjectButton(QAbstractButton):
-
-    def __init__(self, parent):
-        QAbstractButton.__init__(self, parent)
-        self.mouse_over = False
-
-    def enterEvent(self, event):
-        self.mouse_over = True
-
-    def leaveEvent(self, event):
-        self.mouse_over = False
-
-    def paintEvent(self, event):
-        painter = QPainter(self)
-        painter.setClipRect(event.rect())
-        image = QPixmap(I('eject')).scaledToHeight(event.rect().height(),
-            Qt.SmoothTransformation)
-
-        if not self.mouse_over:
-            alpha_mask = QPixmap(image.width(), image.height())
-            color = QColor(128, 128, 128)
-            alpha_mask.fill(color)
-            image.setAlphaChannel(alpha_mask)
-
-        painter.drawPixmap(0, 0, image)
-
-
 
 
 class FontFamilyModel(QAbstractListModel):
@@ -674,6 +509,7 @@ class EnComboBox(QComboBox):
         QComboBox.__init__(self, *args)
         self.setLineEdit(EnLineEdit(self))
         self.setAutoCompletionCaseSensitivity(Qt.CaseSensitive)
+        self.setMinimumContentsLength(20)
 
     def text(self):
         return unicode(self.currentText())
@@ -722,6 +558,53 @@ class HistoryLineEdit(QComboBox):
     def text(self):
         return self.currentText()
 
+class ComboBoxWithHelp(QComboBox):
+    '''
+    A combobox where item 0 is help text. CurrentText will return '' for item 0.
+    Be sure to always fetch the text with currentText. Don't use the signals
+    that pass a string, because they will not correct the text.
+    '''
+    def __init__(self, parent=None):
+        QComboBox.__init__(self, parent)
+        self.currentIndexChanged[int].connect(self.index_changed)
+        self.help_text = ''
+        self.state_set = False
+
+    def initialize(self, help_text=_('Search')):
+        self.help_text = help_text
+        self.set_state()
+
+    def set_state(self):
+        if not self.state_set:
+            if self.currentIndex() == 0:
+                self.setItemText(0, self.help_text)
+                self.setStyleSheet('QComboBox { color: gray }')
+            else:
+                self.setItemText(0, '')
+                self.setStyleSheet('QComboBox { color: black }')
+
+    def index_changed(self, index):
+        self.state_set = False
+        self.set_state()
+
+    def currentText(self):
+        if self.currentIndex() == 0:
+            return ''
+        return QComboBox.currentText(self)
+
+    def itemText(self, idx):
+        if idx == 0:
+            return ''
+        return QComboBox.itemText(self, idx)
+
+    def showPopup(self):
+        self.setItemText(0, '')
+        QComboBox.showPopup(self)
+
+    def hidePopup(self):
+        QComboBox.hidePopup(self)
+        self.set_state()
+
 class PythonHighlighter(QSyntaxHighlighter):
 
     Rules = []
@@ -991,7 +874,7 @@ class SplitterHandle(QSplitterHandle):
 
 class LayoutButton(QToolButton):
 
-    def __init__(self, icon, text, splitter, parent=None):
+    def __init__(self, icon, text, splitter, parent=None, shortcut=None):
         QToolButton.__init__(self, parent)
         self.label = text
         self.setIcon(QIcon(icon))
@@ -1000,18 +883,23 @@ class LayoutButton(QToolButton):
         self.splitter = splitter
         splitter.state_changed.connect(self.update_state)
         self.setCursor(Qt.PointingHandCursor)
+        self.shortcut = ''
+        if shortcut:
+            self.shortcut = shortcut
 
     def set_state_to_show(self, *args):
         self.setChecked(False)
         label =_('Show')
-        self.setText(label + ' ' + self.label)
+        self.setText(label + ' ' + self.label + u' (%s)'%self.shortcut)
         self.setToolTip(self.text())
+        self.setStatusTip(self.text())
 
     def set_state_to_hide(self, *args):
         self.setChecked(True)
         label = _('Hide')
-        self.setText(label + ' ' + self.label)
+        self.setText(label + ' ' + self.label+ u' (%s)'%self.shortcut)
         self.setToolTip(self.text())
+        self.setStatusTip(self.text())
 
     def update_state(self, *args):
         if self.splitter.is_side_index_hidden:
@@ -1025,7 +913,7 @@ class Splitter(QSplitter):
 
     def __init__(self, name, label, icon, initial_show=True,
             initial_side_size=120, connect_button=True,
-            orientation=Qt.Horizontal, side_index=0, parent=None):
+            orientation=Qt.Horizontal, side_index=0, parent=None, shortcut=None):
         QSplitter.__init__(self, parent)
         self.resize_timer = QTimer(self)
         self.resize_timer.setSingleShot(True)
@@ -1040,10 +928,21 @@ class Splitter(QSplitter):
         self.initial_side_size = initial_side_size
         self.initial_show = initial_show
         self.splitterMoved.connect(self.splitter_moved, type=Qt.QueuedConnection)
-        self.button = LayoutButton(icon, label, self)
+        self.button = LayoutButton(icon, label, self, shortcut=shortcut)
         if connect_button:
             self.button.clicked.connect(self.double_clicked)
 
+        if shortcut is not None:
+            self.action_toggle = QAction(QIcon(icon), _('Toggle') + ' ' + label,
+                    self)
+            self.action_toggle.triggered.connect(self.toggle_triggered)
+            self.action_toggle.setShortcut(shortcut)
+            if parent is not None:
+                parent.addAction(self.action_toggle)
+
+    def toggle_triggered(self, *args):
+        self.toggle_side_pane()
+
     def createHandle(self):
         return SplitterHandle(self.orientation(), self)
 
@@ -1061,7 +960,10 @@ class Splitter(QSplitter):
     @property
     def is_side_index_hidden(self):
         sizes = list(self.sizes())
-        return sizes[self.side_index] == 0
+        try:
+            return sizes[self.side_index] == 0
+        except IndexError:
+            return True
 
     @property
     def save_name(self):
@@ -1130,6 +1032,9 @@ class Splitter(QSplitter):
 
     # Public API {{{
 
+    def update_desired_state(self):
+        self.desired_show = not self.is_side_index_hidden
+
     def save_state(self):
         if self.count() > 1:
             gprefs[self.save_name+'_state'] = self.get_state()
