@@ -209,13 +209,13 @@ class ResultCache(SearchQueryParser):
         if query == 'false':
             for item in self._data:
                 if item is None: continue
-                if item[loc] is None or item[loc] == UNDEFINED_DATE:
+                if item[loc] is None or item[loc] <= UNDEFINED_DATE:
                     matches.add(item[0])
             return matches
         if query == 'true':
             for item in self._data:
                 if item is None: continue
-                if item[loc] is not None and item[loc] != UNDEFINED_DATE:
+                if item[loc] is not None and item[loc] > UNDEFINED_DATE:
                     matches.add(item[0])
             return matches
 
@@ -319,12 +319,18 @@ class ResultCache(SearchQueryParser):
                 matches.add(item[0])
         return matches
 
-    def get_matches(self, location, query):
+    def get_matches(self, location, query, allow_recursion=True):
         matches = set([])
         if query and query.strip():
             # get metadata key associated with the search term. Eliminates
             # dealing with plurals and other aliases
             location = self.field_metadata.search_term_to_key(location.lower().strip())
+            if isinstance(location, list):
+                if allow_recursion:
+                    for loc in location:
+                        matches |= self.get_matches(loc, query, allow_recursion=False)
+                    return matches
+                raise ParseException(query, len(query), 'Recursive query group detected', self)
 
             # take care of dates special case
             if location in self.field_metadata and \
@@ -542,6 +548,8 @@ class ResultCache(SearchQueryParser):
         if field is not None:
             self.sort(field, ascending)
         self._map_filtered = list(self._map)
+        if self.search_restriction:
+            self.search('', return_matches=False, ignore_search_restriction=False)
 
     def seriescmp(self, sidx, siidx, x, y, library_order=None):
         try:
@@ -607,26 +615,24 @@ class ResultCache(SearchQueryParser):
         self._map.sort(cmp=fcmp, reverse=not ascending)
         self._map_filtered = [id for id in self._map if id in self._map_filtered]
 
-    def search(self, query, return_matches=False,
-            ignore_search_restriction=False):
-        q = ''
-        if not query or not query.strip():
-            if not ignore_search_restriction:
-                q = self.search_restriction
-        else:
-            q = query
-            if not ignore_search_restriction:
-                q = u'%s (%s)' % (self.search_restriction, query)
-        if not q:
-            if return_matches:
-                return list(self._map) # when return_matches, do not update the maps!
-            self._map_filtered = list(self._map)
-            return
-        matches = sorted(self.parse(q))
-        ans = [id for id in self._map if id in matches]
+    def search(self, query, return_matches=False):
+        ans = self.search_getting_ids(query, self.search_restriction)
         if return_matches:
             return ans
         self._map_filtered = ans
 
+    def search_getting_ids(self, query, search_restriction):
+        q = ''
+        if not query or not query.strip():
+            q = search_restriction
+        else:
+            q = query
+            if search_restriction:
+                q = u'%s (%s)' % (search_restriction, query)
+        if not q:
+            return list(self._map)
+        matches = sorted(self.parse(q))
+        return [id for id in self._map if id in matches]
+
     def set_search_restriction(self, s):
         self.search_restriction = s
