@@ -324,6 +324,7 @@ def do_remove(db, ids):
                 db.delete_book(y)
 
         send_message()
+    db.clean()
 
 def remove_option_parser():
     return get_parser(_(
@@ -449,6 +450,7 @@ def command_show_metadata(args, dbpath):
 def do_set_metadata(db, id, stream):
     mi = OPF(stream)
     db.set_metadata(id, mi)
+    db.clean()
     do_show_metadata(db, id, False)
     send_message()
 
@@ -574,6 +576,9 @@ def command_add_custom_column(args, dbpath):
         return 1
     do_add_custom_column(get_db(dbpath, opts), args[0], args[1], args[2],
             opts.is_multiple, json.loads(opts.display))
+    # Re-open the DB so that  field_metadata is reflects the column changes
+    db = get_db(dbpath, opts)
+    db.prefs['field_metadata'] = db.field_metadata.all_metadata()
     return 0
 
 def catalog_option_parser(args):
@@ -672,7 +677,14 @@ def command_catalog(args, dbpath):
 
     # No support for connected device in CLI environment
     # Parallel initialization in calibre.gui2.tools:generate_catalog()
-    opts.connected_device = { 'storage':None,'serial':None,'save_template':None,'name':None}
+    opts.connected_device = {
+                             'is_device_connected': False,
+                             'kind': None,
+                             'name': None,
+                             'save_template': None,
+                             'serial': None,
+                             'storage': None,
+                            }
 
     with plugin:
         plugin.run(args[1], opts, get_db(dbpath, opts))
@@ -797,12 +809,71 @@ def command_remove_custom_column(args, dbpath):
         return 1
 
     do_remove_custom_column(get_db(dbpath, opts), args[0], opts.force)
+    # Re-open the DB so that  field_metadata is reflects the column changes
+    db = get_db(dbpath, opts)
+    db.prefs['field_metadata'] = db.field_metadata.all_metadata()
+    return 0
+
+def saved_searches_option_parser():
+    parser = get_parser(_(
+    '''
+    %prog saved_searches [options] list
+    %prog saved_searches add name search
+    %prog saved_searches remove name
+
+    Manage the saved searches stored in this database.
+    If you try to add a query with a name that already exists, it will be
+    replaced.
+    '''))
+    return parser
+
+def command_saved_searches(args, dbpath):
+    parser = saved_searches_option_parser()
+    opts, args = parser.parse_args(args)
+    if len(args) < 1:
+        parser.print_help()
+        print
+        prints(_('Error: You must specify an action (add|remove|list)'), file=sys.stderr)
+        return 1
+    from calibre.utils.search_query_parser import saved_searches
+    db = get_db(dbpath, opts)
+    db
+    ss = saved_searches()
+    if args[0] == 'list':
+        for name in ss.names():
+            prints(_('Name:'), name)
+            prints(_('Search string:'), ss.lookup(name))
+            print
+    elif args[0] == 'add':
+        if len(args) < 3:
+            parser.print_help()
+            print
+            prints(_('Error: You must specify a name and a search string'), file=sys.stderr)
+            return 1
+        ss.add(args[1], args[2])
+        prints(args[1], _('added'))
+    elif args[0] == 'remove':
+        if len(args) < 2:
+            parser.print_help()
+            print
+            prints(_('Error: You must specify a name'), file=sys.stderr)
+            return 1
+        ss.delete(args[1])
+        prints(args[1], _('removed'))
+    else:
+        parser.print_help()
+        print
+        prints(_('Error: Action %s not recognized, must be one '
+            'of: (add|remove|list)') % args[1], file=sys.stderr)
+        return 1
+
     return 0
 
 
 COMMANDS = ('list', 'add', 'remove', 'add_format', 'remove_format',
             'show_metadata', 'set_metadata', 'export', 'catalog',
-            'add_custom_column', 'custom_columns', 'remove_custom_column', 'set_custom')
+            'saved_searches', 'add_custom_column', 'custom_columns',
+            'remove_custom_column', 'set_custom')
 
 
 def option_parser():
