@@ -1080,9 +1080,11 @@ ALTER TABLE books ADD COLUMN isbn TEXT DEFAULT "" COLLATE NOCASE;
             return cPickle.loads(str(data))
         return None
 
-    def delete_conversion_options(self, id, format):
+    def delete_conversion_options(self, id, format, commit=True):
         self.conn.execute('DELETE FROM conversion_options WHERE book=? AND format=?',
                 (id, format.upper()))
+        if commit:
+            self.conn.commit()
 
 
     def add_format(self, index, ext, stream, index_is_id=False):
