@@ -30,13 +30,13 @@ class FieldMetadata(dict):
 
     label: the actual column label. No prefixing.
 
-    datatype: the type of the information in the field. Valid values are float,
-    int, rating, bool, comments, datetime, text.
+    datatype: the type of information in the field. Valid values are listed in
+    VALID_DATA_TYPES below.
     is_multiple: valid for the text datatype. If None, the field is to be
     treated as a single term. If not None, it contains a string, and the field
     is assumed to contain a list of terms separated by that string
 
-    kind == standard: is a db field.
+    kind == field: is a db field.
     kind == category: standard tag category that isn't a field. see news.
     kind == user: user-defined tag category.
     kind == search: saved-searches category.
@@ -65,6 +65,10 @@ class FieldMetadata(dict):
     rec_index: the index of the field in the db metadata record.
 
     '''
+
+    VALID_DATA_TYPES = frozenset([None, 'rating', 'text', 'comments', 'datetime',
+                                  'int', 'float', 'bool', 'series'])
+
     _field_metadata = [
             ('authors',   {'table':'authors',
                            'column':'name',
@@ -235,7 +239,7 @@ class FieldMetadata(dict):
                              'is_multiple':None,
                              'kind':'field',
                              'name':None,
-                             'search_terms':[],
+                             'search_terms':['series_index'],
                              'is_custom':False,
                              'is_category':False}),
             ('sort',      {'table':None,
@@ -296,11 +300,13 @@ class FieldMetadata(dict):
         self._search_term_map = {}
         self.custom_label_to_key_map = {}
         for k,v in self._field_metadata:
+            if v['kind'] == 'field' and v['datatype'] not in self.VALID_DATA_TYPES:
+                raise ValueError('Unknown datatype %s for field %s'%(v['datatype'], k))
             self._tb_cats[k] = v
             self._tb_cats[k]['label'] = k
             self._tb_cats[k]['display'] = {}
             self._tb_cats[k]['is_editable'] = True
-            self._add_search_terms_to_map(k, self._tb_cats[k]['search_terms'])
+            self._add_search_terms_to_map(k, v['search_terms'])
         self.custom_field_prefix = '#'
         self.get = self._tb_cats.get
 
@@ -365,6 +371,12 @@ class FieldMetadata(dict):
     def get_custom_fields(self):
         return [l for l in self._tb_cats if self._tb_cats[l]['is_custom']]
 
+    def all_metadata(self):
+        l = {}
+        for k in self._tb_cats:
+            l[k] = self._tb_cats[k]
+        return l
+
     def get_custom_field_metadata(self):
         l = {}
         for k in self._tb_cats:
@@ -377,6 +389,8 @@ class FieldMetadata(dict):
         key = self.custom_field_prefix + label
         if key in self._tb_cats:
             raise ValueError('Duplicate custom field [%s]'%(label))
+        if datatype not in self.VALID_DATA_TYPES:
+            raise ValueError('Unknown datatype %s for field %s'%(datatype, key))
         self._tb_cats[key] = {'table':table,       'column':column,
                              'datatype':datatype,  'is_multiple':is_multiple,
                              'kind':'field',       'name':name,
@@ -387,10 +401,18 @@ class FieldMetadata(dict):
                              'is_editable': is_editable,}
         self._add_search_terms_to_map(key, [key])
         self.custom_label_to_key_map[label] = key
-
-    def remove_custom_fields(self):
-        for key in self.get_custom_fields():
-            del self._tb_cats[key]
+        if datatype == 'series':
+            key += '_index'
+            self._tb_cats[key] = {'table':None,        'column':None,
+                                 'datatype':'float',   'is_multiple':False,
+                                 'kind':'field',       'name':'',
+                                 'search_terms':[key], 'label':label+'_index',
+                                 'colnum':None,        'display':{},
+                                 'is_custom':False,    'is_category':False,
+                                 'link_column':None,   'category_sort':None,
+                                 'is_editable': False,}
+            self._add_search_terms_to_map(key, [key])
+            self.custom_label_to_key_map[label+'_index'] = key
 
     def remove_dynamic_categories(self):
         for key in list(self._tb_cats.keys()):
@@ -455,9 +477,7 @@ class FieldMetadata(dict):
 #                 ])
 
     def get_search_terms(self):
-        s_keys = []
-        for v in self._tb_cats.itervalues():
-            map((lambda x:s_keys.append(x)), v['search_terms'])
+        s_keys = sorted(self._search_term_map.keys())
         for v in self.search_items:
             s_keys.append(v)
 #        if set(s_keys) != self.DEFAULT_LOCATIONS:
@@ -468,6 +488,9 @@ class FieldMetadata(dict):
     def _add_search_terms_to_map(self, key, terms):
         if terms is not None:
             for t in terms:
+                t = t.lower()
+                if t in self._search_term_map:
+                    raise ValueError('Attempt to add duplicate search term "%s"'%t)
                 self._search_term_map[t] = key
 
     def search_term_to_key(self, term):
