@@ -8,13 +8,13 @@ __docformat__ = 'restructuredtext en'
 
 import os, traceback, cStringIO, re
 
-from calibre.utils.config import Config, StringConfig
+from calibre.utils.config import Config, StringConfig, tweaks
 from calibre.utils.filenames import shorten_components_to, supports_long_names, \
                                     ascii_filename, sanitize_file_name
 from calibre.ebooks.metadata.opf2 import metadata_to_opf
 from calibre.ebooks.metadata.meta import set_metadata
 from calibre.constants import preferred_encoding, filesystem_encoding
-
+from calibre.ebooks.metadata import title_sort
 from calibre import strftime
 
 DEFAULT_TEMPLATE = '{author_sort}/{title}/{title} - {authors}'
@@ -61,7 +61,7 @@ def config(defaults=None):
                 'actual e-book file(s).'))
     x('formats', default='all',
             help=_('Comma separated list of formats to save for each book.'
-                ' By default all available books are saved.'))
+                ' By default all available formats are saved.'))
     x('template', default=DEFAULT_TEMPLATE,
             help=_('The template to control the filename and directory structure of the saved files. '
                 'Default is "%s" which will save books into a per-author '
@@ -99,7 +99,8 @@ def preprocess_template(template):
 
 def safe_format(x, format_args):
     try:
-        return x.format(**format_args).strip()
+        ans = x.format(**format_args).strip()
+        return re.sub(r'\s+', ' ', ans)
     except IndexError: # Thrown if user used [] and index is out of bounds
         pass
     except AttributeError: # Thrown if user used a non existing attribute
@@ -109,9 +110,11 @@ def safe_format(x, format_args):
 def get_components(template, mi, id, timefmt='%b %Y', length=250,
         sanitize_func=ascii_filename, replace_whitespace=False,
         to_lowercase=False):
+    library_order = tweaks['save_template_title_series_sorting'] == 'library_order'
+    tsfmt = title_sort if library_order else lambda x: x
     format_args = dict(**FORMAT_ARGS)
     if mi.title:
-        format_args['title'] = mi.title
+        format_args['title'] = tsfmt(mi.title)
     if mi.authors:
         format_args['authors'] = mi.format_authors()
         format_args['author'] = format_args['authors']
@@ -122,9 +125,11 @@ def get_components(template, mi, id, timefmt='%b %Y', length=250,
         if format_args['tags'].startswith('/'):
             format_args['tags'] = format_args['tags'][1:]
     if mi.series:
-        format_args['series'] = mi.series
+        format_args['series'] = tsfmt(mi.series)
         if mi.series_index is not None:
             format_args['series_index'] = mi.format_series_index()
+    else:
+        template = re.sub(r'\{series_index[^}]*?\}', '', template)
     if mi.rating is not None:
         format_args['rating'] = mi.format_rating()
     if mi.isbn:
