@@ -3,12 +3,14 @@ __copyright__ = '2008, Kovid Goyal <kovid at kovidgoyal.net>'
 
 ''' Post installation script for linux '''
 
-import sys, os, shutil, cPickle, textwrap, stat
+import sys, os, cPickle, textwrap, stat
 from subprocess import check_call
 
 from calibre import  __appname__, prints, guess_type
 from calibre.constants import islinux, isfreebsd
 from calibre.customize.ui import all_input_formats
+from calibre.ptempfile import TemporaryDirectory
+from calibre import CurrentDir
 
 
 entry_points = {
@@ -29,6 +31,7 @@ entry_points = {
              'calibre-complete   = calibre.utils.complete:main',
              'pdfmanipulate      = calibre.ebooks.pdf.manipulate.cli:main',
              'fetch-ebook-metadata = calibre.ebooks.metadata.fetch:main',
+             'epub-fix           = calibre.ebooks.epub.fix.main:main',
              'calibre-smtp = calibre.utils.smtp:main',
         ],
         'gui_scripts'    : [
@@ -38,6 +41,7 @@ entry_points = {
                             ],
       }
 
+# Uninstall script {{{
 UNINSTALL = '''\
 #!{python}
 euid = {euid}
@@ -78,6 +82,8 @@ for f in mr:
 os.remove(os.path.abspath(__file__))
 '''
 
+# }}}
+
 class PostInstall:
 
     def task_failed(self, msg):
@@ -170,7 +176,7 @@ class PostInstall:
             self.task_failed('Creating uninstaller failed')
 
 
-    def setup_completion(self):
+    def setup_completion(self): # {{{
         try:
             self.info('Setting up bash completion...')
             from calibre.ebooks.metadata.cli import option_parser as metaop, filetypes as meta_filetypes
@@ -180,6 +186,7 @@ class PostInstall:
             from calibre.ebooks.metadata.fetch import option_parser as fem_op
             from calibre.gui2.main import option_parser as guiop
             from calibre.utils.smtp import option_parser as smtp_op
+            from calibre.ebooks.epub.fix.main import option_parser as fix_op
             any_formats = ['epub', 'htm', 'html', 'xhtml', 'xhtm', 'rar', 'zip',
                 'txt', 'lit', 'rtf', 'pdf', 'prc', 'mobi', 'fb2', 'odt', 'lrf']
             bc = os.path.join(os.path.dirname(self.opts.staging_sharedir),
@@ -201,6 +208,7 @@ class PostInstall:
                 f.write(opts_and_exts('ebook-viewer', viewer_op, any_formats))
                 f.write(opts_and_words('fetch-ebook-metadata', fem_op, []))
                 f.write(opts_and_words('calibre-smtp', smtp_op, []))
+                f.write(opts_and_exts('epub-fix', fix_op, ['epub']))
                 f.write(textwrap.dedent('''
                 _ebook_device_ls()
                 {
@@ -284,8 +292,9 @@ class PostInstall:
             if self.opts.fatal_errors:
                 raise
             self.task_failed('Setting up completion failed')
+    # }}}
 
-    def install_man_pages(self):
+    def install_man_pages(self): # {{{
         try:
             from calibre.utils.help2man import create_man_page
             if isfreebsd:
@@ -315,73 +324,69 @@ class PostInstall:
             if self.opts.fatal_errors:
                 raise
             self.task_failed('Installing MAN pages failed')
+    # }}}
 
-    def setup_desktop_integration(self):
+    def setup_desktop_integration(self): # {{{
         try:
-            from PyQt4.QtCore import QFile
-            from tempfile import mkdtemp
 
             self.info('Setting up desktop integration...')
 
 
-            tdir = mkdtemp()
-            cwd = os.getcwdu()
-            try:
-                os.chdir(tdir)
-                render_svg(QFile(I('mimetypes/lrf.svg')), os.path.join(tdir, 'calibre-lrf.png'))
-                check_call('xdg-icon-resource install --noupdate --context mimetypes --size 128 calibre-lrf.png application-lrf', shell=True)
-                self.icon_resources.append(('mimetypes', 'application-lrf', '128'))
-                check_call('xdg-icon-resource install --noupdate --context mimetypes --size 128 calibre-lrf.png text-lrs', shell=True)
-                self.icon_resources.append(('mimetypes', 'application-lrs',
-                '128'))
-                QFile(I('library.png')).copy(os.path.join(tdir, 'calibre-gui.png'))
-                check_call('xdg-icon-resource install --noupdate --size 128 calibre-gui.png calibre-gui', shell=True)
-                self.icon_resources.append(('apps', 'calibre-gui', '128'))
-                render_svg(QFile(I('viewer.svg')), os.path.join(tdir, 'calibre-viewer.png'))
-                check_call('xdg-icon-resource install --size 128 calibre-viewer.png calibre-viewer', shell=True)
-                self.icon_resources.append(('apps', 'calibre-viewer', '128'))
+            with TemporaryDirectory() as tdir:
+                with CurrentDir(tdir):
+                    render_img('mimetypes/lrf.png', 'calibre-lrf.png')
+                    check_call('xdg-icon-resource install --noupdate --context mimetypes --size 128 calibre-lrf.png application-lrf', shell=True)
+                    self.icon_resources.append(('mimetypes', 'application-lrf', '128'))
+                    check_call('xdg-icon-resource install --noupdate --context mimetypes --size 128 calibre-lrf.png text-lrs', shell=True)
+                    self.icon_resources.append(('mimetypes', 'application-lrs',
+                    '128'))
+                    render_img('lt.png', 'calibre-gui.png')
+                    check_call('xdg-icon-resource install --noupdate --size 128 calibre-gui.png calibre-gui', shell=True)
+                    self.icon_resources.append(('apps', 'calibre-gui', '128'))
+                    render_img('viewer.png', 'calibre-viewer.png')
+                    check_call('xdg-icon-resource install --size 128 calibre-viewer.png calibre-viewer', shell=True)
+                    self.icon_resources.append(('apps', 'calibre-viewer', '128'))
 
-                mimetypes = set([])
-                for x in all_input_formats():
-                    mt = guess_type('dummy.'+x)[0]
-                    if mt and 'chemical' not in mt:
-                        mimetypes.add(mt)
+                    mimetypes = set([])
+                    for x in all_input_formats():
+                        mt = guess_type('dummy.'+x)[0]
+                        if mt and 'chemical' not in mt:
+                            mimetypes.add(mt)
 
-                def write_mimetypes(f):
-                    f.write('MimeType=%s;\n'%';'.join(mimetypes))
+                    def write_mimetypes(f):
+                        f.write('MimeType=%s;\n'%';'.join(mimetypes))
 
-                f = open('calibre-lrfviewer.desktop', 'wb')
-                f.write(VIEWER)
-                f.close()
-                f = open('calibre-ebook-viewer.desktop', 'wb')
-                f.write(EVIEWER)
-                write_mimetypes(f)
-                f.close()
-                f = open('calibre-gui.desktop', 'wb')
-                f.write(GUI)
-                write_mimetypes(f)
-                f.close()
-                des = ('calibre-gui.desktop', 'calibre-lrfviewer.desktop',
-                        'calibre-ebook-viewer.desktop')
-                for x in des:
-                    cmd = ['xdg-desktop-menu', 'install', './'+x]
-                    if x != des[-1]:
-                        cmd.insert(2, '--noupdate')
-                    check_call(' '.join(cmd), shell=True)
-                    self.menu_resources.append(x)
-                f = open('calibre-mimetypes', 'wb')
-                f.write(MIME)
-                f.close()
-                self.mime_resources.append('calibre-mimetypes')
-                check_call('xdg-mime install ./calibre-mimetypes', shell=True)
-            finally:
-                os.chdir(cwd)
-                shutil.rmtree(tdir)
+                    f = open('calibre-lrfviewer.desktop', 'wb')
+                    f.write(VIEWER)
+                    f.close()
+                    f = open('calibre-ebook-viewer.desktop', 'wb')
+                    f.write(EVIEWER)
+                    write_mimetypes(f)
+                    f.close()
+                    f = open('calibre-gui.desktop', 'wb')
+                    f.write(GUI)
+                    write_mimetypes(f)
+                    f.close()
+                    des = ('calibre-gui.desktop', 'calibre-lrfviewer.desktop',
+                            'calibre-ebook-viewer.desktop')
+                    for x in des:
+                        cmd = ['xdg-desktop-menu', 'install', './'+x]
+                        if x != des[-1]:
+                            cmd.insert(2, '--noupdate')
+                        check_call(' '.join(cmd), shell=True)
+                        self.menu_resources.append(x)
+                    f = open('calibre-mimetypes', 'wb')
+                    f.write(MIME)
+                    f.close()
+                    self.mime_resources.append('calibre-mimetypes')
+                    check_call('xdg-mime install ./calibre-mimetypes', shell=True)
         except Exception:
             if self.opts.fatal_errors:
                 raise
             self.task_failed('Setting up desktop integration failed')
 
+    # }}}
+
 def option_parser():
     from calibre.utils.config import OptionParser
     parser = OptionParser()
@@ -539,21 +544,10 @@ MIME = '''\
 </mime-info>
 '''
 
-def render_svg(image, dest, width=128, height=128):
-    from PyQt4.QtGui import QPainter, QImage
-    from PyQt4.QtSvg import QSvgRenderer
-    image = image.readAll() if hasattr(image, 'readAll') else image
-    svg = QSvgRenderer(image)
-    painter = QPainter()
-    image = QImage(width, height, QImage.Format_ARGB32)
-    painter.begin(image)
-    painter.setRenderHints(QPainter.Antialiasing|QPainter.TextAntialiasing|QPainter.SmoothPixmapTransform|QPainter.HighQualityAntialiasing)
-    painter.setCompositionMode(QPainter.CompositionMode_SourceOver)
-    svg.render(painter)
-    painter.end()
-    if dest is None:
-        return image
-    image.save(dest)
+def render_img(image, dest, width=128, height=128):
+    from PyQt4.Qt import QImage, Qt
+    img = QImage(I(image)).scaled(width, height, Qt.IgnoreAspectRatio, Qt.SmoothTransformation)
+    img.save(dest)
 
 def main():
     p = option_parser()
