@@ -18,10 +18,11 @@ If this module is run, it will perform a series of unit tests.
 
 import sys, string, operator
 
-from calibre.utils.pyparsing import Keyword, Group, Forward, CharsNotIn, Suppress, \
-                      OneOrMore, oneOf, CaselessLiteral, Optional, NoMatch, ParseException
+from calibre.utils.pyparsing import CaselessKeyword, Group, Forward, CharsNotIn, Suppress, \
+                      OneOrMore, MatchFirst, CaselessLiteral, Optional, NoMatch, ParseException
 from calibre.constants import preferred_encoding
-from calibre.utils.config import prefs
+
+
 
 '''
 This class manages access to the preference holding the saved search queries.
@@ -32,9 +33,13 @@ class SavedSearchQueries(object):
     queries = {}
     opt_name = ''
 
-    def __init__(self, _opt_name):
+    def __init__(self, db, _opt_name):
         self.opt_name = _opt_name;
-        self.queries = prefs[self.opt_name]
+        self.db = db
+        if db is not None:
+            self.queries = db.prefs.get(self.opt_name, {})
+        else:
+            self.queries = {}
 
     def force_unicode(self, x):
         if not isinstance(x, unicode):
@@ -43,20 +48,20 @@ class SavedSearchQueries(object):
 
     def add(self, name, value):
         self.queries[self.force_unicode(name)] = self.force_unicode(value).strip()
-        prefs[self.opt_name] = self.queries
+        self.db.prefs[self.opt_name] = self.queries
 
     def lookup(self, name):
         return self.queries.get(self.force_unicode(name), None)
 
     def delete(self, name):
         self.queries.pop(self.force_unicode(name), False)
-        prefs[self.opt_name] = self.queries
+        self.db.prefs[self.opt_name] = self.queries
 
     def rename(self, old_name, new_name):
         self.queries[self.force_unicode(new_name)] = \
                     self.queries.get(self.force_unicode(old_name), None)
         self.queries.pop(self.force_unicode(old_name), False)
-        prefs[self.opt_name] = self.queries
+        self.db.prefs[self.opt_name] = self.queries
 
     def names(self):
         return sorted(self.queries.keys(),
@@ -66,8 +71,15 @@ class SavedSearchQueries(object):
 Create a global instance of the saved searches. It is global so that the searches
 are common across all instances of the parser (devices, library, etc).
 '''
-saved_searches = SavedSearchQueries('saved_searches')
+ss = SavedSearchQueries(None, None)
 
+def set_saved_searches(db, opt_name):
+    global ss
+    ss = SavedSearchQueries(db, opt_name)
+
+def saved_searches():
+    global ss
+    return ss
 
 class SearchQueryParser(object):
     '''
@@ -139,18 +151,19 @@ class SearchQueryParser(object):
 
         Not = Forward()
         Not << (Group(
-            Suppress(Keyword("not", caseless=True)) + Not
+            Suppress(CaselessKeyword("not")) + Not
         ).setResultsName("not") | Parenthesis)
 
         And = Forward()
         And << (Group(
-            Not + Suppress(Keyword("and", caseless=True)) + And
+            Not + Suppress(CaselessKeyword("and")) + And
         ).setResultsName("and") | Group(
-            Not + OneOrMore(~oneOf("and or", caseless=True) + And)
+            Not + OneOrMore(~MatchFirst(list(map(CaselessKeyword,
+                ('and', 'or')))) + And)
         ).setResultsName("and") | Not)
 
         Or << (Group(
-            And + Suppress(Keyword("or", caseless=True)) + Or
+            And + Suppress(CaselessKeyword("or")) + Or
         ).setResultsName("or") | And)
 
         if test:
@@ -158,8 +171,6 @@ class SearchQueryParser(object):
             self._tests_failed = bool(failed)
 
         self._parser = Or
-        #self._parser.setDebug(True)
-        #self.parse('(tolstoy)')
         self._parser.setDebug(False)
 
 
@@ -209,7 +220,7 @@ class SearchQueryParser(object):
                     raise ParseException(query, len(query), 'undefined saved search', self)
                 if self.recurse_level > 5:
                     self.searches_seen.add(query)
-                return self._parse(saved_searches.lookup(query))
+                return self._parse(saved_searches().lookup(query))
             except: # convert all exceptions (e.g., missing key) to a parse error
                 raise ParseException(query, len(query), 'undefined saved search', self)
         return self.get_matches(location, query)
@@ -283,7 +294,7 @@ class Tester(SearchQueryParser):
  28: [u"Kushiel's Scion", u'Jacqueline Carey', None, u'lrf,rar'],
  29: [u'Underworld', u'Don DeLillo', None, u'lrf,rar'],
  30: [u'Genghis Khan and The Making of the Modern World',
-      u'Jack Weatherford',
+      u'Jack Weatherford Orc',
       u'Three Rivers Press',
       u'lrf,zip'],
  31: [u'The Best and the Brightest',
@@ -535,6 +546,7 @@ class Tester(SearchQueryParser):
              'london:thames': set([13]),
              'publisher:london:thames': set([13]),
              '"(1977)"': set([13]),
+             'jack weatherford orc': set([30]),
              }
     fields = {'title':0, 'author':1, 'publisher':2, 'tag':3}
 
@@ -574,7 +586,10 @@ class Tester(SearchQueryParser):
 
 
 def main(args=sys.argv):
-    tester = Tester(test=True)
+    tester = Tester(['authors', 'author', 'series', 'formats', 'format',
+        'publisher', 'rating', 'tags', 'tag', 'comments', 'comment', 'cover',
+        'isbn', 'ondevice', 'pubdate', 'size', 'date', 'title', u'#read',
+        'all', 'search'], test=True)
     failed = tester.run_tests()
     if tester._tests_failed or failed:
         print '>>>>>>>>>>>>>> Tests Failed <<<<<<<<<<<<<<<'
