@@ -24,7 +24,6 @@ from calibre.ebooks.metadata import MetaInformation
 from calibre.web.feeds import feed_from_xml, templates, feeds_from_index, Feed
 from calibre.web.fetch.simple import option_parser as web2disk_option_parser
 from calibre.web.fetch.simple import RecursiveFetcher
-from calibre.utils.magick_draw import add_borders_to_image
 from calibre.utils.threadpool import WorkRequest, ThreadPool, NoResultsPending
 from calibre.ptempfile import PersistentTemporaryFile
 from calibre.utils.date import now as nowf
@@ -37,7 +36,10 @@ class DownloadDenied(ValueError):
 
 class BasicNewsRecipe(Recipe):
     '''
-    Abstract base class that contains logic needed in all feed fetchers.
+    Base class that contains logic needed in all recipes. By overriding
+    progressively more of the functionality in this class, you can make
+    progressively more customized/powerful recipes. For a tutorial introduction
+    to creating recipes, see :doc:`news`.
     '''
 
     #: The title to use for the ebook
@@ -127,7 +129,7 @@ class BasicNewsRecipe(Recipe):
     #: embedded content.
     use_embedded_content   = None
 
-    #: Set to True and implement :method:`get_obfuscated_article` to handle
+    #: Set to True and implement :meth:`get_obfuscated_article` to handle
     #: websites that try to make it difficult to scrape content.
     articles_are_obfuscated = False
 
@@ -147,7 +149,7 @@ class BasicNewsRecipe(Recipe):
     #: If True empty feeds are removed from the output.
     #: This option has no effect if parse_index is overriden in
     #: the sub class. It is meant only for recipes that return a list
-    #: of feeds using `feeds` or :method:`get_feeds`.
+    #: of feeds using `feeds` or :meth:`get_feeds`.
     remove_empty_feeds = False
 
     #: List of regular expressions that determines which links to follow
@@ -264,7 +266,7 @@ class BasicNewsRecipe(Recipe):
             }
 
             .article_description {
-                font-family: sans; text-indent: 0pt;
+                text-indent: 0pt;
             }
 
             a.article {
@@ -538,8 +540,7 @@ class BasicNewsRecipe(Recipe):
         HTML fetching engine, so it can contain links to pages/images on the web.
 
         This method is typically useful for sites that try to make it difficult to
-        access article content automatically. See for example the
-        :module:`calibre.web.recipes.iht` recipe.
+        access article content automatically.
         '''
         raise NotImplementedError
 
@@ -683,13 +684,15 @@ class BasicNewsRecipe(Recipe):
             base.extract()
 
         ans = self.postprocess_html(soup, first_fetch)
-        try:
-            article = self.feed_objects[f].articles[a]
-        except:
-            self.log.exception('Failed to get article object for postprocessing')
-            pass
-        else:
-            self.populate_article_metadata(article, ans, first_fetch)
+        if job_info:
+            url, f, a, feed_len = job_info
+            try:
+                article = self.feed_objects[f].articles[a]
+            except:
+                self.log.exception('Failed to get article object for postprocessing')
+                pass
+            else:
+                self.populate_article_metadata(article, ans, first_fetch)
         return ans
 
 
@@ -698,8 +701,7 @@ class BasicNewsRecipe(Recipe):
         Download and pre-process all articles from the feeds in this recipe.
         This method should be called only once on a particular Recipe instance.
         Calling it more than once will lead to undefined behavior.
-        @return: Path to index.html
-        @rtype: string
+        :return: Path to index.html
         '''
         try:
             res = self.build_index()
@@ -961,6 +963,7 @@ class BasicNewsRecipe(Recipe):
                 with nested(open(cpath, 'wb'), closing(self.browser.open(cu))) as (cfile, r):
                     cfile.write(r.read())
                 if self.cover_margins[0] or self.cover_margins[1]:
+                    from calibre.utils.magick.draw import add_borders_to_image
                     add_borders_to_image(cpath,
                                          left=self.cover_margins[0],right=self.cover_margins[0],
                                          top=self.cover_margins[1],bottom=self.cover_margins[1],
@@ -1015,12 +1018,11 @@ class BasicNewsRecipe(Recipe):
         Create a generic cover for recipes that dont have a cover
         '''
         try:
-            from calibre.utils.magick_draw import create_cover_page, TextLine
+            from calibre.ebooks import calibre_cover
             title = self.title if isinstance(self.title, unicode) else \
                     self.title.decode(preferred_encoding, 'replace')
             date = strftime(self.timefmt)
-            lines = [TextLine(title, 44), TextLine(date, 32)]
-            img_data = create_cover_page(lines, I('library.png'), output_format='jpg')
+            img_data = calibre_cover(title, date)
             cover_file.write(img_data)
             cover_file.flush()
         except:
@@ -1072,51 +1074,30 @@ class BasicNewsRecipe(Recipe):
         img.save(open(out_path, 'wb'), 'JPEG')
 
     def prepare_masthead_image(self, path_to_image, out_path):
-        import calibre.utils.PythonMagickWand as pw
-        from ctypes import byref
         from calibre import fit_image
+        from calibre.utils.magick import Image, create_canvas
 
-        with pw.ImageMagick():
-            img = pw.NewMagickWand()
-            img2 = pw.NewMagickWand()
-            frame = pw.NewMagickWand()
-            p = pw.NewPixelWand()
-            if img < 0 or img2 < 0 or p < 0 or frame < 0:
-                raise RuntimeError('Out of memory')
-            if not pw.MagickReadImage(img, path_to_image):
-                severity = pw.ExceptionType(0)
-                msg = pw.MagickGetException(img, byref(severity))
-                raise IOError('Failed to read image from: %s: %s'
-                        %(path_to_image, msg))
-            pw.PixelSetColor(p, 'white')
-            width, height = pw.MagickGetImageWidth(img),pw.MagickGetImageHeight(img)
-            scaled, nwidth, nheight = fit_image(width, height, self.MI_WIDTH, self.MI_HEIGHT)
-            if not pw.MagickNewImage(img2, width, height, p):
-                raise RuntimeError('Out of memory')
-            if not pw.MagickNewImage(frame,  self.MI_WIDTH, self.MI_HEIGHT, p):
-                raise RuntimeError('Out of memory')
-            if not pw.MagickCompositeImage(img2, img, pw.OverCompositeOp, 0, 0):
-                raise RuntimeError('Out of memory')
-            if scaled:
-                if not pw.MagickResizeImage(img2, nwidth, nheight, pw.LanczosFilter,
-                        0.5):
-                    raise RuntimeError('Out of memory')
-            left = int((self.MI_WIDTH - nwidth)/2.0)
-            top = int((self.MI_HEIGHT - nheight)/2.0)
-            if not pw.MagickCompositeImage(frame, img2, pw.OverCompositeOp,
-                    left, top):
-                raise RuntimeError('Out of memory')
-            if not pw.MagickWriteImage(frame, out_path):
-                raise RuntimeError('Failed to save image to %s'%out_path)
-
-            pw.DestroyPixelWand(p)
-            for x in (img, img2, frame):
-                pw.DestroyMagickWand(x)
+        img = Image()
+        img.open(path_to_image)
+        width, height = img.size
+        scaled, nwidth, nheight = fit_image(width, height, self.MI_WIDTH, self.MI_HEIGHT)
+        img2 = create_canvas(width, height)
+        frame = create_canvas(self.MI_WIDTH, self.MI_HEIGHT)
+        img2.compose(img)
+        if scaled:
+            img2.size = (nwidth, nheight, 'LanczosFilter', 0.5)
+        left = int((self.MI_WIDTH - nwidth)/2.0)
+        top = int((self.MI_HEIGHT - nheight)/2.0)
+        frame.compose(img2, left, top)
+        frame.save(out_path)
 
     def create_opf(self, feeds, dir=None):
         if dir is None:
             dir = self.output_dir
-        mi = MetaInformation(self.short_title() + strftime(self.timefmt), [__appname__])
+        title = self.short_title()
+        if self.output_profile.periodical_date_in_title:
+            title += strftime(self.timefmt)
+        mi = MetaInformation(title, [__appname__])
         mi.publisher = __appname__
         mi.author_sort = __appname__
         mi.publication_type = 'periodical:'+self.publication_type
@@ -1357,7 +1338,7 @@ class BasicNewsRecipe(Recipe):
          '''
          If your recipe when converted to EPUB has problems with images when
          viewed in Adobe Digital Editions, call this method from within
-         :method:`postprocess_html`.
+         :meth:`postprocess_html`.
          '''
          for item in soup.findAll('img'):
              for attrib in ['height','width','border','align','style']:
